@@ -179,7 +179,14 @@ and trigger-new is:
 			  RDB$FILE_NAME in character_sets and collations
 			  RDB$BASE_COLLATION_NAME and RDB$SPECIFIC_ATTRIBUTES in collations
  */
-const int ATT_BACKUP_FORMAT		= 7;	// ASF: when change this, change the text of the message gbak_inv_bkup_ver too
+
+/* Version 8: RDB$RELATION_TYPE in relations
+			  RDB$PROCEDURE_TYPE and RDB$VALID_BLR in procedures
+			  RDB$VALID_BLR in triggers
+			  RDB$DEFAULT_VALUE, RDB$DEFAULT_SOURCE and RDB$COLLATION_ID in procedure_parameters
+
+ */
+const int ATT_BACKUP_FORMAT		= 8;	// ASF: when change this, change the text of the message gbak_inv_bkup_ver too
 
 // format version number for ranges for arrays 
 
@@ -243,7 +250,8 @@ enum att_type {
 	att_relation_view_source2,
 	att_relation_ext_description2,
 	att_relation_flags,  
-	att_relation_ext_file_name, // name of file for external tables 
+	att_relation_ext_file_name, // name of file for external tables
+	att_relation_type,
 
 	// Field attributes (used for both global and local fields) 
 
@@ -362,7 +370,9 @@ enum att_type {
 	att_trig_inactive,
 	att_trig_source2,
 	att_trig_description2,
-	att_trig_flags,	 
+	att_trig_flags,
+	att_trig_valid_blr,
+	att_trig_debug_info,
 
 	// Function attributes 
 
@@ -447,6 +457,9 @@ enum att_type {
 	att_procedure_blr,
 	att_procedure_security_class,
 	att_procedure_owner_name,
+	att_procedure_type,
+	att_procedure_valid_blr,
+	att_procedure_debug_info,
 
 	// Stored procedure parameter attributes 
 
@@ -456,6 +469,11 @@ enum att_type {
 	att_procedureprm_field_source,
 	att_procedureprm_description,
 	att_procedureprm_description2,
+	att_procedureprm_default_value,
+	att_procedureprm_default_source,
+	att_procedureprm_collation_id,
+	att_procedureprm_null_flag,
+	att_procedureprm_mechanism,
 
 	// Exception attributes 
 
@@ -631,6 +649,9 @@ struct gfld {
 	ISC_QUAD	gfld_vb;
 	ISC_QUAD	gfld_vs;
 	ISC_QUAD	gfld_vs2;
+	ISC_QUAD	gfld_computed_blr;
+	ISC_QUAD	gfld_computed_source;
+	ISC_QUAD	gfld_computed_source2;
 	gfld*		gfld_next;
 	USHORT		gfld_flags;
 };
@@ -640,12 +661,15 @@ typedef gfld* GFLD;
 enum gfld_flags_vals {
 	GFLD_validation_blr		= 1,
 	GFLD_validation_source	= 2,
-	GFLD_validation_source2	= 4
+	GFLD_validation_source2	= 4,
+	GFLD_computed_blr		= 8,
+	GFLD_computed_source	= 16,
+	GFLD_computed_source2	= 32
 };
 
 // CVC: Could use MAXPATHLEN, but what about restoring in a different system?
 // I need to review if we tolerate different lengths for different OS's here.
-const int MAX_FILE_NAME_SIZE		= 256;
+const unsigned int MAX_FILE_NAME_SIZE		= 256;
 
 // Note that this typedef is also defined in JRD.H and REMOTE.H 
 // but for some reason we are avoiding including JRD.H
@@ -742,7 +766,7 @@ const size_t ACT_LEN = sizeof(act);
 
 const ULONG MAX_LENGTH = -1UL; // Keep in sync with burp_fil.fil_length
 
-// This structure has been cloned from spit.c 
+// This structure has been cloned from spit.cpp
 
 struct hdr_split {
 	TEXT hdr_split_tag[18];
@@ -781,6 +805,8 @@ public:
 		// this is VERY dirty hack to keep current behaviour
 		memset (&gbl_database_file_name, 0,
 			&veryEnd - reinterpret_cast<char*>(&gbl_database_file_name));
+
+		// normal code follows
 		exit_code = FINI_ERROR;	// prevent FINI_OK in case of unknown error thrown
 								// would be set to FINI_OK (==0) in exit_local
 	}
@@ -791,6 +817,7 @@ public:
 	bool		gbl_sw_ignore_limbo;
 	bool		gbl_sw_meta;
 	bool		gbl_sw_novalidity;
+	bool		gbl_sw_nodbtriggers;
 	USHORT		gbl_sw_page_size;
 	bool		gbl_sw_compress;
 	bool		gbl_sw_version;
@@ -810,6 +837,9 @@ public:
 	const SCHAR*	gbl_sw_sql_role;
 	const SCHAR*	gbl_sw_user;
 	const SCHAR*	gbl_sw_password;
+#ifdef TRUSTED_SERVICES
+	const SCHAR*	gbl_sw_tr_user;
+#endif
 	SLONG		gbl_sw_skip_count;
 	SLONG		gbl_sw_page_buffers;
 	burp_fil*	gbl_sw_files;
@@ -825,6 +855,7 @@ public:
 	burp_prc*	procedures;
 	SLONG		BCK_capabilities;
 	USHORT		RESTORE_format;
+	int         RESTORE_ods;
 	ULONG		mvol_io_buffer_size;
 	ULONG		mvol_actual_buffer_size;
 	FB_UINT64	mvol_cumul_count;
@@ -972,7 +1003,7 @@ inline static ULONG BURP_UP_TO_BLOCK(const ULONG size)
 	return (((size) + BURP_BLOCK - 1) & ~(BURP_BLOCK - 1));
 }
 
-/* Move the read and write mode declarations in here from burp.c
+/* Move the read and write mode declarations in here from burp.cpp
    so that other files can see them for multivolume opens */
 
 #ifdef WIN_NT
@@ -998,5 +1029,9 @@ enum burp_messages_vals {
 	msgVerbose_restore_collation	= 216
 };
 
+
+// BLOB buffer
+typedef Firebird::HalfStaticArray<UCHAR, 1024> BlobBuffer;
+
 #endif // BURP_BURP_H 
 
