@@ -131,13 +131,6 @@ const size_t REDIRECT_THRESHOLD = 32768;
 // Declare thread-specific variable for context memory pool
 TLS_DECLARE(MemoryPool*, contextPool);
 
-// Helper function to reduce code size, since many compilers
-// generate quite a bit of code at the point of the throw.
-void pool_out_of_memory()
-{
-	throw std::bad_alloc();
-}
-
 // Support for memory mapping facilities
 #if defined(WIN_NT)
 size_t get_page_size()
@@ -179,7 +172,6 @@ inline size_t get_map_page_size()
 	}
 	return map_page_size;
 }
-
 #endif
 
 #ifdef USE_VALGRIND
@@ -236,7 +228,7 @@ inline void MemoryPool::decrement_mapping(size_t size)
 	mapped_memory -= size;
 }
 
-MemoryPool* MemoryPool::setContextPool(MemoryPool *newPool)
+MemoryPool* MemoryPool::setContextPool(MemoryPool* newPool)
 {
 	MemoryPool* old = TLS_GET(contextPool);
 	TLS_SET(contextPool, newPool);
@@ -258,7 +250,7 @@ namespace {
 	char msBuffer[sizeof(MemoryStats) + ALLOC_ALIGNMENT];
 	MemoryPool* createProcessMemoryPool()
 	{
-		MemoryPool::default_stats_group = 
+		MemoryPool::default_stats_group =
 			new((void*)(IPTR) MEM_ALIGN((size_t)(IPTR) msBuffer)) MemoryStats;
 		MemoryPool* p = MemoryPool::createPool();
 		fb_assert(p);
@@ -271,7 +263,7 @@ namespace {
 MemoryPool* MemoryPool::processMemoryPool = createProcessMemoryPool();
 
 
-void MemoryPool::setStatsGroup(MemoryStats &statsL)
+void MemoryPool::setStatsGroup(MemoryStats& statsL)
 {
 	// This locking pattern is necessary to ensure thread-safety of this routine.
 	// It is deadlock-free only as long other code takes locks in the same order.
@@ -322,7 +314,7 @@ void MemoryPool::updateSpare()
 			spareLeafs.add(temp);
 		}
 		while ( (int) spareNodes.getCount() <= freeBlocks.level + 1 &&
-				spareNodes.getCount() < spareNodes.getCapacity() ) 
+				spareNodes.getCount() < spareNodes.getCapacity() )
 		{
 			void* temp = internal_alloc(MEM_ALIGN(sizeof(FreeBlocksTree::NodeList)), TYPE_TREEPAGE);
 			if (!temp)
@@ -450,30 +442,37 @@ void* MemoryPool::external_alloc(size_t &size)
 	return VirtualAlloc(NULL, size, MEM_COMMIT, 
 						PAGE_READWRITE);
 # elif defined (HAVE_MMAP) && !defined(SOLARIS)
-	size = FB_ALIGN(size, get_map_page_size());
-#  ifdef MAP_ANONYMOUS
-	return  mmap(NULL, size, PROT_READ | PROT_WRITE, 
-				MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
-#  else	
-	// This code is needed for Solaris 2.6, AFAIK  (only?)
-	if (dev_zero_fd < 0)
-		dev_zero_fd = open("/dev/zero", O_RDWR);
-	return mmap(NULL, size, PROT_READ | PROT_WRITE, 
-				MAP_PRIVATE, dev_zero_fd, 0);
-#  endif //MAP_ANONYMOUS
-# elif  defined(SOLARIS)
 
 // No successful return from mmap()  will  return    the value MAP_FAILED.
-//The  symbol  MAP_FAILED  is  defined  in  the header     <sys/mman.h>
-//Solaris 2.9 #define MAP_FAILED      ((void *) -1)
-
+//The  symbol  MAP_FAILED  is  defined:
+//#define MAP_FAILED      ((void *) -1)
 
 	size = FB_ALIGN(size, get_map_page_size());
 	void *result = NULL;
 #  ifdef MAP_ANONYMOUS
-    
+	result = mmap(NULL, size, PROT_READ | PROT_WRITE, 
+				  MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
+#  else	
+	// This code is needed for Solaris 2.6, AFAIK  (only?)
+	if (dev_zero_fd < 0)
+		dev_zero_fd = open("/dev/zero", O_RDWR);
+	result = mmap(NULL, size, PROT_READ | PROT_WRITE, 
+				  MAP_PRIVATE, dev_zero_fd, 0);
+#  endif //MAP_ANONYMOUS
+	return result == MAP_FAILED ? NULL : result;	
+# elif  defined(SOLARIS)
+	size = FB_ALIGN(size, get_map_page_size());
+	void *result = NULL;
+#  ifdef MAP_ANONYMOUS
 	result = mmap(0, size, PROT_READ | PROT_WRITE, 
 					MAP_PRIVATE | MAP_ANON , -1, 0);
+#  else	
+	// This code is needed for Solaris 2.6, AFAIK
+	if (dev_zero_fd < 0)
+		dev_zero_fd = open("/dev/zero", O_RDWR);
+	result = mmap(NULL, size, PROT_READ | PROT_WRITE, 
+					MAP_PRIVATE, dev_zero_fd, 0);
+#  endif //MAP_ANONYMOUS
 	if (result == MAP_FAILED) {
 		// failure happens!
 		return NULL;
@@ -481,19 +480,6 @@ void* MemoryPool::external_alloc(size_t &size)
 	else {
 		return result;	
 	}	
-#  else	
-	// This code is needed for Solaris 2.6, AFAIK
-	if (dev_zero_fd < 0)
-		dev_zero_fd = open("/dev/zero", O_RDWR);
-	result = mmap(NULL, size, PROT_READ | PROT_WRITE, 
-					MAP_PRIVATE, dev_zero_fd, 0);
-	if (result == MAP_FAILED) {
-		return NULL;
-	}	
-	else {
-		return result;	
-	}			
-#  endif //MAP_ANONYMOUS
 # else
 	return malloc(size);
 # endif
@@ -539,7 +525,7 @@ void* MemoryPool::tree_alloc(size_t size) {
 			spareLeafs.getCount()) 
 		{
 			if (!spareLeafs.getCount())
-				pool_out_of_memory();
+				Firebird::BadAlloc::raise();
 			void *temp = spareLeafs[spareLeafs.getCount() - 1];
 			spareLeafs.shrink(spareLeafs.getCount() - 1);
 			needSpare = true;
@@ -547,7 +533,7 @@ void* MemoryPool::tree_alloc(size_t size) {
 		}
 	if (size == sizeof(FreeBlocksTree::NodeList)) {
 		if (!spareNodes.getCount())
-			pool_out_of_memory();
+			Firebird::BadAlloc::raise();
 		void *temp = spareNodes[spareNodes.getCount() - 1];
 		spareNodes.shrink(spareNodes.getCount() - 1);
 		needSpare = true;
@@ -751,7 +737,7 @@ void* MemoryPool::allocate(size_t size, SSHORT type
 #endif
 	);
 	if (!result)
-		pool_out_of_memory();
+		Firebird::BadAlloc::raise();
 	return result;
 }
 
@@ -1078,10 +1064,10 @@ MemoryPool* MemoryPool::internal_create(size_t instance_size, MemoryPool* parent
 		void* mem = parent->internal_alloc(size, TYPE_POOL);
 		if (!mem) {
 			parent->lock.leave();
-			pool_out_of_memory();
+			Firebird::BadAlloc::raise();
 		}
 		pool = new(mem) MemoryPool(parent, stats, NULL, NULL);
-		
+
 		MemoryBlock* blk = ptrToBlock(mem);
 		blk->mbk_pool = pool;
 		blk->mbk_flags |= MBK_PARENT;
@@ -1113,7 +1099,7 @@ MemoryPool* MemoryPool::internal_create(size_t instance_size, MemoryPool* parent
 		fb_assert(ext_size == EXTENT_SIZE); // Make sure exent size is a multiply of page size
 	
 		if (!mem)
-			pool_out_of_memory();
+			Firebird::BadAlloc::raise();
 		((MemoryExtent *)mem)->mxt_next = NULL;
 		((MemoryExtent *)mem)->mxt_prev = NULL;
 		
@@ -1263,7 +1249,7 @@ void* MemoryPool::internal_alloc(size_t size, SSHORT type
 	// This method assumes already aligned block sizes
 	fb_assert(size % ALLOC_ALIGNMENT == 0);
 	// Make sure block can fit into extent
-	fb_assert(size < EXTENT_SIZE - MEM_ALIGN(sizeof(MemoryBlock)) - MEM_ALIGN(sizeof(MemoryExtent)));
+	fb_assert(size <= EXTENT_SIZE - MEM_ALIGN(sizeof(MemoryBlock)) - MEM_ALIGN(sizeof(MemoryExtent)));
 	
 	// Lookup a block greater or equal than size in freeBlocks tree
 	MemoryBlock* blk;
@@ -1465,7 +1451,8 @@ inline void MemoryPool::addFreeBlock(MemoryBlock *blk)
 	BlockInfo info = {blk->mbk_small.mbk_length, fragmentToAdd};
 	try {
 		freeBlocks.add(info);
-	} catch(const std::exception&) {
+	}
+	catch (const Firebird::Exception&) {
 		// Add item to the list of pending free blocks in case of critically-low memory condition
 		PendingFreeBlock* temp = blockToPtr<PendingFreeBlock*>(blk);
 		temp->next = pendingFree;
@@ -1797,44 +1784,3 @@ void AutoStorage::ProbeStack() const {
 #endif
 
 } // namespace Firebird
-
-/******************************** Global functions *****************************/
-
-void* operator new(size_t s) THROW_BAD_ALLOC
-{
-#if defined(DEV_BUILD)
-// Do not complain here. It causes client tools to crash on Red Hat 8.0
-//	fprintf(stderr, "You MUST allocate all memory from a pool.  Don't use the default global new().\n");
-#endif	// DEV_BUILD
-//	return getDefaultMemoryPool()->calloc(s, 0
-	return getDefaultMemoryPool()->allocate(s, 0
-#ifdef DEBUG_GDS_ALLOC
-	  ,__FILE__, __LINE__
-#endif
-	);
-}
-
-void* operator new[](size_t s) THROW_BAD_ALLOC
-{
-#if defined(DEV_BUILD)
-// Do not complain here. It causes client tools to crash on Red Hat 8.0
-//	fprintf(stderr, "You MUST allocate all memory from a pool.  Don't use the default global new[]().\n");
-#endif	// DEV_BUILD
-//	return getDefaultMemoryPool()->->calloc(s, 0
-	return getDefaultMemoryPool()->allocate(s, 0
-#ifdef DEBUG_GDS_ALLOC
-	  ,__FILE__, __LINE__
-#endif
-	);
-}
-
-void operator delete(void* mem) throw()
-{
-	Firebird::MemoryPool::globalFree(mem);
-}
-
-void operator delete[](void* mem) throw()
-{
-	Firebird::MemoryPool::globalFree(mem);
-}
-
