@@ -21,6 +21,7 @@
  *
  * All Rights Reserved.
  * Contributor(s): ______________________________________.
+ * Adriano dos Santos Fernandes
  */
  
 #ifndef CLASSES_ARRAY_H
@@ -98,6 +99,8 @@ protected:
 			this->getPool().deallocate(data);
 	}
 public:
+	typedef T* iterator;
+	typedef const T* const_iterator;
 	Array<T, Storage>& operator =(const Array<T, Storage>& L) 
 	{
 		ensureCapacity(L.count);
@@ -156,27 +159,41 @@ public:
 		data[count++] = item;
   		return count;
 	}
-	void remove(size_t index) {
+	void add(const T* items, size_t itemsSize)
+	{
+		ensureCapacity(count + itemsSize);
+		memcpy(data + count, items, sizeof(T) * itemsSize);
+		count += itemsSize;
+	}
 	// NOTE: remove method must be signal safe
 	// This function may be called in AST. The function doesn't wait.
+	T* remove(size_t index) {
   		fb_assert(index < count);
   		memmove(data + index, data + index + 1, sizeof(T) * (--count - index));
+		return &data[index];
 	}
-	void removeRange(size_t from, size_t to) {
+	T* removeRange(size_t from, size_t to) {
   		fb_assert(from <= to);
   		fb_assert(to <= count);
   		memmove(data + from, data + to, sizeof(T) * (count - to));
 		count -= (to - from);
+		return &data[from];
 	}
-	void removeCount(size_t index, size_t n) {
+	T* removeCount(size_t index, size_t n) {
   		fb_assert(index + n <= count);
   		memmove(data + index, data + index + n, sizeof(T) * (count - index - n));
 		count -= n;
+		return &data[index];
 	}
-	void remove(T* itr) {
+	T* remove(T* itr) {
 		const size_t index = itr - begin();
   		fb_assert(index < count);
   		memmove(data + index, data + index + 1, sizeof(T) * (--count - index));
+		return &data[index];
+	}
+	T* remove(T* itrFrom, T* itrTo)
+	{
+		return removeRange(itrFrom - begin(), itrTo - begin());
 	}
 	void shrink(size_t newCount) {
 		fb_assert(newCount <= count);
@@ -189,11 +206,38 @@ public:
 		memset(data + count, 0, sizeof(T) * (newCount - count));
 		count = newCount;
 	}
+	// Resize array according to STL's vector::resize() rules
+	void resize(size_t newCount, const T& val) {
+		if (newCount > count) {
+			ensureCapacity(newCount);
+			while (count < newCount) {
+				data[count++] = val;
+			}
+		}
+		else {
+			count = newCount;
+		}
+	}
+	// Resize array according to STL's vector::resize() rules
+	void resize(size_t newCount) {
+		if (newCount > count) {
+			grow(newCount);
+		}
+		else {
+			count = newCount;
+		}
+	}
 	void join(const Array<T, Storage>& L) {
 		ensureCapacity(count + L.count);
 		memcpy(data + count, L.data, sizeof(T) * L.count);
 		count += L.count;
 	}
+	void assign(const Array<T, Storage>& L)
+	{
+		ensureCapacity(L.count);
+		memcpy(data, L.data, sizeof(T) * L.count);
+		count = L.count;
+	}
 	// NOTE: getCount method must be signal safe
 	// Used as such in GlobalRWLock::blockingAstHandler
 	size_t getCount() const { return count; }
@@ -225,7 +269,24 @@ public:
 		capacity = this->getStorageSize();
 		data = this->getStorage();
 	}
-
+	// This method only assigns "pos" if the element is found.
+	// Maybe we should modify it to iterate directy with "pos".
+	bool find(const T& item, size_t& pos) const
+	{
+		for (size_t i = 0; i < count; i++) {
+			if (data[i] == item) {
+				pos = i;
+				return true;
+			}
+		}
+		return false;
+	}
+	bool exist(const T& item) const
+	{
+		size_t pos;	// ignored
+		return find(item, pos);
+	}
+	
 protected:
 	size_t count, capacity;
 	T* data;
@@ -276,6 +337,13 @@ public:
 		return highBound != this->count &&
 			!Cmp::greaterThan(KeyOfValue::generate(this, this->data[lowBound]), item);
 	}
+
+	bool exist(const Key& item) const
+	{
+		size_t pos;	// ignored
+		return find(item, pos);
+	}
+
 	size_t add(const Value& item) {
 	    size_t pos;
   	    find(KeyOfValue::generate(this, item), pos);
@@ -296,7 +364,8 @@ public:
 		Array<T, InlineStorage<T, InlineCapacity> > (InitialCapacity) {}
 };
 
+typedef HalfStaticArray<UCHAR, 16> UCharBuffer;
+
 }	// namespace Firebird
 
 #endif // CLASSES_ARRAY_H
-
