@@ -33,11 +33,10 @@
 #include <string.h>
 #include <stdarg.h>
 
- 
+#include "firebird.h"
 #include "../include/fb_types.h"
 #include "../include/fb_exception.h"
 #include "../common/classes/alloc.h"
-#include "../include/gen/autoconfig.h"
 
 namespace Firebird
 {
@@ -193,7 +192,7 @@ namespace Firebird
 		enum TrimType {TrimLeft, TrimRight, TrimBoth};
 
 		void baseTrim(TrimType WhereTrim, const_pointer ToTrim);
-		
+
 	public:
 		inline const_pointer c_str() const {
 			return stringBuffer;
@@ -211,10 +210,10 @@ namespace Firebird
 		// in case a null ASCII was inserted in the middle of the string.
 		inline size_type recalculate_length()
 		{
-		    stringLength = strlen(stringBuffer);
+		    stringLength = static_cast<internal_size_type>(strlen(stringBuffer));
 		    return stringLength;
 		}
-		
+
 		void reserve(size_type n = 0);
 		void resize(size_type n, char_type c = ' ');
 
@@ -223,13 +222,6 @@ namespace Firebird
 			return baseAssign(l);
 		}
 
-		inline size_type copy_from(pointer s, size_type n, size_type pos = 0) const
-		{
-			AdjustRange(length(), pos, n);
-			memcpy(s, c_str() + pos, n);
-			return n;
-		}
-
 /*		inline void swap(AbstractString& str) {
 			Storage *tmp = StringData;
 			StringData = str.StringData;
@@ -363,7 +355,7 @@ namespace Firebird
 		void vprintf(const char* Format, va_list params);
 		void printf(const char* Format, ...);
 
-		inline int copyTo(pointer to, size_type toSize) const
+		inline size_type copyTo(pointer to, size_type toSize) const
 		{
 			fb_assert(to);
 			fb_assert(toSize);
@@ -376,6 +368,8 @@ namespace Firebird
 			return toSize;
 		}
 
+		void appendQuoted(const AbstractString& from, char_type quote);
+
 		inline ~AbstractString() {
 			if (stringBuffer != inlineBuffer)
 				delete[] stringBuffer;
@@ -613,7 +607,12 @@ namespace Firebird
 		int compare(const_pointer s, size_type n) const {
 			const size_type ml = length() < n ? length() : n;
 			const int rc = Comparator::compare(c_str(), s, ml);
-			return rc ? rc : length() - n;
+			if (rc)
+			{
+				return rc;
+			}
+			const difference_type dl = length() - n;
+			return (dl < 0) ? -1 : (dl > 0) ? 1 : 0;
 		}
 
 		inline bool operator< (const StringType& str) const {return compare(str) <  0;}
