@@ -30,6 +30,7 @@
  * 2003.10.05 Dmitry Yemanov: Added support for explicit cursors in PSQL
  * 2004.01.16 Vlad Horsun: Added support for default parameters and 
  *   EXECUTE BLOCK statement
+ * Adriano dos Santos Fernandes
  */
 
 #include "firebird.h"
@@ -58,11 +59,12 @@
 static void gen_aggregate(dsql_req*, const dsql_nod*);
 static void gen_cast(dsql_req*, const dsql_nod*);
 static void gen_coalesce(dsql_req*, const dsql_nod*);
-static void gen_constant(dsql_req*, dsc*, bool);
+static void gen_constant(dsql_req*, const dsc*, bool);
+static void gen_constant(dsql_req*, dsql_nod*, bool);
 static void gen_descriptor(dsql_req*, const dsc*, bool);
 static void gen_error_condition(dsql_req*, const dsql_nod*);
 static void gen_field(dsql_req*, const dsql_ctx*, const dsql_fld*, dsql_nod*);
-static void gen_for_select(dsql_req*, dsql_nod*);
+static void gen_for_select(dsql_req*, const dsql_nod*);
 static void gen_gen_id(dsql_req*, const dsql_nod*);
 static void gen_join_rse(dsql_req*, const dsql_nod*);
 static void gen_map(dsql_req*, dsql_map*);
@@ -74,11 +76,15 @@ static void gen_searched_case(dsql_req*, const dsql_nod*);
 static void gen_select(dsql_req*, dsql_nod*);
 static void gen_simple_case(dsql_req*, const dsql_nod*);
 static void gen_sort(dsql_req*, dsql_nod*);
+static void gen_statement(dsql_req*, const dsql_nod*);
+static void gen_sys_function(dsql_req*, const dsql_nod*);
 static void gen_table_lock(dsql_req*, const dsql_nod*, USHORT);
 static void gen_udf(dsql_req*, const dsql_nod*);
 static void gen_union(dsql_req*, const dsql_nod*);
 static void stuff_context(dsql_req*, const dsql_ctx*);
 static void stuff_cstring(dsql_req*, const char*);
+static void stuff_meta_string(dsql_req*, const char*);
+static void stuff_string(dsql_req*, const char*, int);
 static void stuff_word(dsql_req*, USHORT);
 
 // STUFF is defined in dsql.h for use in common with ddl.c 
@@ -117,7 +123,7 @@ void GEN_expr( dsql_req* request, dsql_nod* node)
 		return;
 
 	case nod_constant:
-		gen_constant(request, &node->nod_desc, USE_VALUE);
+		gen_constant(request, node, USE_VALUE);
 		return;
 
 	case nod_derived_field:
@@ -157,7 +163,7 @@ void GEN_expr( dsql_req* request, dsql_nod* node)
 			  ddl_node->nod_type == nod_mod_domain))
 		{
 				ERRD_post(isc_sqlerr, isc_arg_number, (SLONG) - 901,
-						  isc_arg_gds, isc_dsql_domain_err, 0);
+						  isc_arg_gds, isc_dsql_domain_err, isc_arg_end);
 		}
 		stuff(request, blr_fid);
 		stuff(request, 0);				// Context   
@@ -205,14 +211,18 @@ void GEN_expr( dsql_req* request, dsql_nod* node)
 		stuff(request, blr_current_date);
 		return;
 
-    case nod_current_role:
-        stuff(request, blr_current_role);
-        return;
+	case nod_current_role:
+		stuff(request, blr_current_role);
+		return;
 
 	case nod_udf:
 		gen_udf(request, node);
 		return;
 
+	case nod_sys_function:
+		gen_sys_function(request, node);
+		return;
+
 	case nod_variable:
 		variable = (dsql_var*) node->nod_arg[e_var_variable];
 		if (variable->var_flags & VAR_input) {
@@ -300,6 +310,11 @@ void GEN_expr( dsql_req* request, dsql_nod* node)
 			blr_agg_total_distinct : blr_agg_total;
 		break;
 
+	case nod_agg_list:
+		blr_operator = (node->nod_flags & NOD_AGG_DISTINCT) ?
+			blr_agg_list_distinct : blr_agg_list;
+		break;
+
 	case nod_and:
 		blr_operator = blr_and;
 		break;
@@ -375,7 +390,7 @@ void GEN_expr( dsql_req* request, dsql_nod* node)
 			if (child->nod_type == nod_constant &&
 				DTYPE_IS_NUMERIC(child->nod_desc.dsc_dtype))
 			{
-				gen_constant(request, &child->nod_desc, NEGATE_VALUE);
+				gen_constant(request, child, NEGATE_VALUE);
 				return;
 			}
 		}
@@ -407,7 +422,7 @@ void GEN_expr( dsql_req* request, dsql_nod* node)
 		blr_operator = blr_any;
 		break;
 	case nod_ansi_any:
-			blr_operator = blr_ansi_any;
+		blr_operator = blr_ansi_any;
 		break;
 	case nod_ansi_all:
 		blr_operator = blr_ansi_all;
@@ -445,7 +460,7 @@ void GEN_expr( dsql_req* request, dsql_nod* node)
 		gen_searched_case(request, node);
 		return;
 	case nod_average:
-	case nod_count:
+	//case nod_count:
 	case nod_from:
 	case nod_max:
 	case nod_min:
@@ -454,12 +469,12 @@ void GEN_expr( dsql_req* request, dsql_nod* node)
 		case nod_average:
 			blr_operator = blr_average;
 			break;
-		case nod_count:
-			blr_operator = blr_count;
-/* count2
-		blr_operator = node->nod_arg[0]->nod_arg[e_rse_items] ? blr_count2 : blr_count;
-*/
-			break;
+		//case nod_count:
+		//	blr_operator = blr_count;
+		// count2
+		//	blr_operator = node->nod_arg[0]->nod_arg[e_rse_items] ? blr_count2 : blr_count;
+
+		//	break;
 		case nod_from:
 			blr_operator = blr_from;
 			break;
@@ -503,7 +518,7 @@ void GEN_expr( dsql_req* request, dsql_nod* node)
 				  isc_arg_gds, isc_dsql_internal_err,
 				  isc_arg_gds, isc_expression_eval_err,
 				  // expression evaluation not supported 
-				  0);
+				  isc_arg_end);
 	}
 
 	stuff(request, blr_operator);
@@ -596,7 +611,12 @@ void GEN_port( dsql_req* request, dsql_msg* message)
 	{
 		parameter->par_parameter = number++;
 
-		if (parameter->par_desc.dsc_dtype <= dtype_any_text && request->req_dbb->dbb_att_charset != CS_NONE &&
+		const USHORT fromCharSet = parameter->par_desc.getCharSet();
+		const USHORT toCharSet = (fromCharSet == CS_NONE || fromCharSet == CS_BINARY) ?
+			fromCharSet : request->req_dbb->dbb_att_charset;
+
+		if (parameter->par_desc.dsc_dtype <= dtype_any_text &&
+			request->req_dbb->dbb_att_charset != CS_NONE &&
 			request->req_dbb->dbb_att_charset != CS_BINARY)
 		{
 			USHORT adjust = 0;
@@ -607,10 +627,6 @@ void GEN_port( dsql_req* request, dsql_msg* message)
 
 			parameter->par_desc.dsc_length -= adjust;
 
-			USHORT fromCharSet = INTL_GET_CHARSET(&parameter->par_desc);
-			USHORT toCharSet = (fromCharSet == CS_NONE || fromCharSet == CS_BINARY) ?
-				fromCharSet : request->req_dbb->dbb_att_charset;
-
 			USHORT fromCharSetBPC = METD_get_charset_bpc(request, fromCharSet);
 			USHORT toCharSetBPC = METD_get_charset_bpc(request, toCharSet);
 
@@ -622,6 +638,16 @@ void GEN_port( dsql_req* request, dsql_msg* message)
 
 			parameter->par_desc.dsc_length += adjust;
 		}
+		else if (ENCODE_ODS(request->req_dbb->dbb_ods_version,
+					request->req_dbb->dbb_minor_version) >= ODS_11_1 &&
+			parameter->par_desc.dsc_dtype == dtype_blob &&
+			parameter->par_desc.dsc_sub_type == isc_blob_text &&
+			request->req_dbb->dbb_att_charset != CS_NONE &&
+			request->req_dbb->dbb_att_charset != CS_BINARY)
+		{
+			if (fromCharSet != toCharSet)
+				parameter->par_desc.setTextType(toCharSet);
+		}
 
 		/* For older clients - generate an error should they try and
 		   access data types which did not exist in the older dialect */
@@ -640,7 +666,7 @@ void GEN_port( dsql_req* request, dsql_msg* message)
 							  isc_arg_number, (SLONG) request->req_client_dialect,
 							  isc_arg_string,
 							  DSC_dtype_tostring(parameter->par_desc.dsc_dtype),
-							  0);
+							  isc_arg_end);
 					break;
 				default:
 					// No special action for other data types 
@@ -659,7 +685,7 @@ void GEN_port( dsql_req* request, dsql_msg* message)
 		ERRD_post(isc_sqlerr, isc_arg_number, (SLONG) -204,
 				  isc_arg_gds, isc_imp_exc,
 				  isc_arg_gds, isc_blktoobig,
-				  0);
+				  isc_arg_end);
 	}
 
 	message->msg_length = (USHORT) offset;
@@ -813,7 +839,7 @@ void GEN_start_transaction( dsql_req* request, const dsql_nod* tran_node)
 		case nod_access:
 			if (sw_access)
 				ERRD_post(isc_sqlerr, isc_arg_number, (SLONG) - 104,
-						  isc_arg_gds, isc_dsql_dup_option, 0);
+						  isc_arg_gds, isc_dsql_dup_option, isc_arg_end);
 
 			sw_access = true;
 			if (ptr->nod_flags & NOD_READ_ONLY)
@@ -825,7 +851,7 @@ void GEN_start_transaction( dsql_req* request, const dsql_nod* tran_node)
 		case nod_wait:
 			if (sw_wait)
 				ERRD_post(isc_sqlerr, isc_arg_number, (SLONG) - 104,
-						  isc_arg_gds, isc_dsql_dup_option, 0);
+						  isc_arg_gds, isc_dsql_dup_option, isc_arg_end);
 
 			sw_wait = true;
 			if (ptr->nod_flags & NOD_NO_WAIT)
@@ -837,7 +863,7 @@ void GEN_start_transaction( dsql_req* request, const dsql_nod* tran_node)
 		case nod_isolation:
 			if (sw_isolation)
 				ERRD_post(isc_sqlerr, isc_arg_number, (SLONG) - 104,
-						  isc_arg_gds, isc_dsql_dup_option, 0);
+						  isc_arg_gds, isc_dsql_dup_option, isc_arg_end);
 
 			sw_isolation = true;
 
@@ -866,7 +892,7 @@ void GEN_start_transaction( dsql_req* request, const dsql_nod* tran_node)
 			{
 				if (sw_reserve)
 					ERRD_post(isc_sqlerr, isc_arg_number, (SLONG) - 104,
-							  isc_arg_gds, isc_dsql_dup_option, 0);
+							  isc_arg_gds, isc_dsql_dup_option, isc_arg_end);
 
 				sw_reserve = true;
 				const dsql_nod* reserve = ptr->nod_arg[0];
@@ -885,7 +911,7 @@ void GEN_start_transaction( dsql_req* request, const dsql_nod* tran_node)
 		case nod_tra_misc:
 			if (misc_flags & ptr->nod_flags)
 				ERRD_post(isc_sqlerr, isc_arg_number, (SLONG) - 104,
-						  isc_arg_gds, isc_dsql_dup_option, 0);
+						  isc_arg_gds, isc_dsql_dup_option, isc_arg_end);
 						  
 			misc_flags |= ptr->nod_flags;
 			if (ptr->nod_flags & NOD_NO_AUTO_UNDO)
@@ -899,7 +925,7 @@ void GEN_start_transaction( dsql_req* request, const dsql_nod* tran_node)
 		case nod_lock_timeout:
 			if (sw_lock_timeout)
 				ERRD_post(isc_sqlerr, isc_arg_number, (SLONG) - 104,
-						  isc_arg_gds, isc_dsql_dup_option, 0);
+						  isc_arg_gds, isc_dsql_dup_option, isc_arg_end);
 
 			sw_lock_timeout = true;
 			if (ptr->nod_count == 1 && ptr->nod_arg[0]->nod_type == nod_constant)
@@ -913,7 +939,7 @@ void GEN_start_transaction( dsql_req* request, const dsql_nod* tran_node)
 
 		default:
 			ERRD_post(isc_sqlerr, isc_arg_number, (SLONG) - 104,
-					  isc_arg_gds, isc_dsql_tran_err, 0);
+					  isc_arg_gds, isc_dsql_tran_err, isc_arg_end);
 		}
 	}
 }
@@ -924,7 +950,7 @@ void GEN_start_transaction( dsql_req* request, const dsql_nod* tran_node)
  	GEN_statement
   
     @brief	Generate blr for an arbitrary expression.
- 
+
 
     @param request
     @param node
@@ -935,9 +961,6 @@ void GEN_statement( dsql_req* request, dsql_nod* node)
 	dsql_nod* temp;
 	dsql_nod** ptr;
 	const dsql_nod* const* end;
-	dsql_ctx* context;
-	dsql_msg* message;
-	dsql_str* name;
 	dsql_str* string;
 	ULONG id_length;
 
@@ -962,69 +985,10 @@ void GEN_statement( dsql_req* request, dsql_nod* node)
 		stuff(request, blr_end);
 		return;
 
-	case nod_erase:
-		if ((temp = node->nod_arg[e_era_rse]) != NULL) {
-			stuff(request, blr_for);
-			GEN_expr(request, temp);
-		}
-		temp = node->nod_arg[e_era_relation];
-		context = (dsql_ctx*) temp->nod_arg[e_rel_context];
-		stuff(request, blr_erase);
-		stuff_context(request, context);
-		return;
-
-	case nod_erase_current:
-		stuff(request, blr_erase);
-		context = (dsql_ctx*) node->nod_arg[e_erc_context];
-		stuff_context(request, context);
-		return;
-
 	case nod_exec_block:
 		DDL_gen_block(request, node);
 		return;
 
-	case nod_exec_procedure:
-		if (request->req_type == REQ_EXEC_PROCEDURE) {
-			if ( (message = request->req_receive) ) {
-				stuff(request, blr_begin);
-				stuff(request, blr_send);
-				stuff(request, message->msg_number);
-			}
-		}
-		else {
-			message = NULL;
-		}
-		stuff(request, blr_exec_proc);
-		name = (dsql_str*) node->nod_arg[e_exe_procedure];
-		stuff_cstring(request, name->str_data);
-
-		if (temp = node->nod_arg[e_exe_inputs]) {
-			stuff_word(request, temp->nod_count);
-			for (ptr = temp->nod_arg, end = ptr + temp->nod_count;
-				 ptr < end; ptr++)
- 			{
- 				GEN_expr(request, *ptr);
- 			}
-		}
-		else {
-			stuff_word(request, 0);
-		}
-		if (temp = node->nod_arg[e_exe_outputs]) {
-			stuff_word(request, temp->nod_count);
-			for (ptr = temp->nod_arg, end = ptr + temp->nod_count;
-				 ptr < end; ptr++)
-			{
-				GEN_expr(request, *ptr);
-			}
-		}
-		else {
-			stuff_word(request, 0);
-		}
-		if (message) {
-			stuff(request, blr_end);
-		}
-		return;
-
 	case nod_for_select:
 		gen_for_select(request, node);
 		return;
@@ -1057,29 +1021,13 @@ void GEN_statement( dsql_req* request, dsql_nod* node)
 		stuff(request, blr_end);
 		return;
 
+	case nod_erase:
+	case nod_erase_current:
 	case nod_modify:
-		if ((temp = node->nod_arg[e_mod_rse]) != NULL) {
-			stuff(request, blr_for);
-			GEN_expr(request, temp);
-		}
-		stuff(request, blr_modify);
-		temp = node->nod_arg[e_mod_source];
-		context = (dsql_ctx*) temp->nod_arg[e_rel_context];
-		stuff_context(request, context);
-		temp = node->nod_arg[e_mod_update];
-		context = (dsql_ctx*) temp->nod_arg[e_rel_context];
-		stuff_context(request, context);
-		GEN_statement(request, node->nod_arg[e_mod_statement]);
-		return;
-
 	case nod_modify_current:
-		stuff(request, blr_modify);
-		context = (dsql_ctx*) node->nod_arg[e_mdc_context];
-		stuff_context(request, context);
-		temp = node->nod_arg[e_mdc_update];
-		context = (dsql_ctx*) temp->nod_arg[e_rel_context];
-		stuff_context(request, context);
-		GEN_statement(request, node->nod_arg[e_mdc_statement]);
+	case nod_store:
+	case nod_exec_procedure:
+		gen_statement(request, node);
 		return;
 
 	case nod_on_error:
@@ -1148,40 +1096,9 @@ void GEN_statement( dsql_req* request, dsql_nod* node)
 		stuff(request, 0);
 		return;
 
-    case nod_breakleave:
-        stuff(request, blr_leave);
-        stuff(request, (int) (IPTR) node->nod_arg[e_breakleave_label]->nod_arg[e_label_number]);
-        return;
-
-	case nod_store:
-		if (request->req_type == REQ_EXEC_PROCEDURE) {
-			if ( (message = request->req_receive) ) {
-				stuff(request, blr_begin);
-				stuff(request, blr_send);
-				stuff(request, message->msg_number);
-			}
-		}
-		else {
-			message = NULL;
-		}
-		if ( (temp = node->nod_arg[e_sto_rse]) ) {
-			stuff(request, blr_for);
-			GEN_expr(request, temp);
-		}
-		if (node->nod_arg[e_sto_return]) {
-			stuff(request, blr_store2);
-			GEN_expr(request, node->nod_arg[e_sto_relation]);
-			GEN_statement(request, node->nod_arg[e_sto_statement]);
-			GEN_statement(request, node->nod_arg[e_sto_return]);
-		}
-		else {
-			stuff(request, blr_store);
-			GEN_expr(request, node->nod_arg[e_sto_relation]);
-			GEN_statement(request, node->nod_arg[e_sto_statement]);
-		}
-		if (message) {
-			stuff(request, blr_end);
-		}
+	case nod_breakleave:
+		stuff(request, blr_leave);
+		stuff(request, (int) (IPTR) node->nod_arg[e_breakleave_label]->nod_arg[e_label_number]);
 		return;
 
 	case nod_abort:
@@ -1324,7 +1241,7 @@ void GEN_statement( dsql_req* request, dsql_nod* node)
 			dsql_nod* list = cursor->nod_arg[e_cur_rse]->nod_arg[e_rse_items];
 			if (list->nod_count != list_into->nod_count)
 				ERRD_post(isc_sqlerr, isc_arg_number, (SLONG) - 313,
-						  isc_arg_gds, isc_dsql_count_mismatch, 0);
+						  isc_arg_gds, isc_dsql_count_mismatch, isc_arg_end);
 			stuff(request, blr_begin);
 			ptr = list->nod_arg;
 			end = ptr + list->nod_count;
@@ -1339,11 +1256,16 @@ void GEN_statement( dsql_req* request, dsql_nod* node)
 		}
 		return;
 
+	case nod_src_info:
+		request->put_debug_src_info(node->nod_line, node->nod_column);
+		GEN_statement(request, node->nod_arg[e_src_info_stmt]);
+		return;
+
 	default:
 		ERRD_post(isc_sqlerr, isc_arg_number, (SLONG) - 901,
 				  isc_arg_gds, isc_dsql_internal_err,
 				  isc_arg_gds, isc_node_err, // gen.c: node not supported
-				  0);
+				  isc_arg_end);
 	}
 }
 
@@ -1469,7 +1391,7 @@ static void gen_coalesce( dsql_req* request, const dsql_nod* node)
     @param negate_value
 
  **/
-static void gen_constant( dsql_req* request, dsc* desc, bool negate_value)
+static void gen_constant( dsql_req* request, const dsc* desc, bool negate_value)
 {
 	SLONG value;
 	SINT64 i64value;
@@ -1495,6 +1417,7 @@ static void gen_constant( dsql_req* request, dsc* desc, bool negate_value)
 		value = *(SLONG *) p;
 		if (negate_value)
 			value = -value;
+		//printf("gen.cpp = %p %d\n", *((void**)p), value);
 		stuff_word(request, value);
 		stuff_word(request, value >> 16);
 		break;
@@ -1544,7 +1467,7 @@ static void gen_constant( dsql_req* request, dsc* desc, bool negate_value)
 			 */
 			ERRD_post(isc_sqlerr,
 					  isc_arg_number, (SLONG) - 104,
-					  isc_arg_gds, isc_arith_except, 0);
+					  isc_arg_gds, isc_arith_except, isc_arg_end);
 		}
 
 		/* We and the lexer both agree that this is an SINT64 constant,
@@ -1587,22 +1510,7 @@ static void gen_constant( dsql_req* request, dsc* desc, bool negate_value)
 
 	case dtype_text:
 		{
-			USHORT length;
-
-			ISC_STATUS_ARRAY status_vector = {0};
-
-			THREAD_EXIT();
-			const ISC_STATUS s =
-				gds__intl_function(status_vector, &request->req_dbb->dbb_database_handle,
-					INTL_FUNCTION_OCTET_LENGTH, INTL_GET_CHARSET(desc),
-					desc->dsc_length, desc->dsc_address, &length);
-			THREAD_ENTER();
-
-			if (s) {
-				ERRD_punt(status_vector);
-			}
-
-			desc->dsc_length = length;
+			USHORT length = desc->dsc_length;
 
 			gen_descriptor(request, desc, true);
 			if (length)
@@ -1615,11 +1523,32 @@ static void gen_constant( dsql_req* request, dsc* desc, bool negate_value)
 	default:
 		// gen_constant: datatype not understood 
 		ERRD_post(isc_sqlerr, isc_arg_number, (SLONG) - 103,
-				  isc_arg_gds, isc_dsql_constant_err, 0);
+				  isc_arg_gds, isc_dsql_constant_err, isc_arg_end);
 	}
 }
 
 
+/**
+  
+ 	gen_constant
+  
+    @brief	Generate BLR for a constant.
+ 
+
+    @param request
+    @param node
+    @param negate_value
+
+ **/
+static void gen_constant( dsql_req* request, dsql_nod* node, bool negate_value)
+{
+	if (node->nod_desc.dsc_dtype == dtype_text)
+		node->nod_desc.dsc_length = ((dsql_str*) node->nod_arg[0])->str_length;
+
+	gen_constant(request, &node->nod_desc, negate_value);
+}
+
+
 /**
   
  	gen_descriptor
@@ -1700,16 +1629,21 @@ static void gen_descriptor( dsql_req* request, const dsc* desc, bool texttype)
 		stuff(request, blr_timestamp);
 		break;
 
-	case dtype_blob:
 	case dtype_array:
 		stuff(request, blr_quad);
 		stuff(request, 0);
 		break;
 
+	case dtype_blob:
+		stuff(request, blr_blob2);
+		stuff_word(request, desc->dsc_sub_type);
+		stuff_word(request, desc->getTextType());
+		break;
+
 	default:
 		// don't understand dtype 
 		ERRD_post(isc_sqlerr, isc_arg_number, (SLONG) - 804,
-				  isc_arg_gds, isc_dsql_datatype_err, 0);
+				  isc_arg_gds, isc_dsql_datatype_err, isc_arg_end);
 	}
 }
 
@@ -1788,7 +1722,7 @@ static void gen_field( dsql_req* request, const dsql_ctx* context,
 					  isc_arg_number, (SLONG) request->req_client_dialect,
 					  isc_arg_string,
 					  DSC_dtype_tostring(static_cast < UCHAR >
-										 (field->fld_dtype)), 0);
+										 (field->fld_dtype)), isc_arg_end);
 			break;
 		default:
 			// No special action for other data types 
@@ -1807,7 +1741,7 @@ static void gen_field( dsql_req* request, const dsql_ctx* context,
 	else {
 		stuff(request, blr_field);
 		stuff_context(request, context);
-		stuff_cstring(request, field->fld_name);
+		stuff_meta_string(request, field->fld_name);
 	}
 
 	if (indices) {
@@ -1833,16 +1767,16 @@ static void gen_field( dsql_req* request, const dsql_ctx* context,
     @param for_select
 
  **/
-static void gen_for_select( dsql_req* request, dsql_nod* for_select)
+static void gen_for_select( dsql_req* request, const dsql_nod* for_select)
 {
 	dsql_nod* rse = for_select->nod_arg[e_flp_select];
 
-    /* CVC: Only put a label if this is not singular; otherwise,
-       what loop is the user trying to abandon? */
-    if (for_select->nod_arg[e_flp_action]) {
-        stuff(request, blr_label);
-        stuff(request, (int) (IPTR) for_select->nod_arg[e_flp_label]->nod_arg[e_label_number]);
-    }
+	// CVC: Only put a label if this is not singular; otherwise,
+	// what loop is the user trying to abandon?
+	if (for_select->nod_arg[e_flp_action]) {
+		stuff(request, blr_label);
+		stuff(request, (int) (IPTR) for_select->nod_arg[e_flp_label]->nod_arg[e_label_number]);
+	}
 
 // Generate FOR loop 
 
@@ -1869,18 +1803,23 @@ static void gen_for_select( dsql_req* request, dsql_nod* for_select)
 	
 	dsql_nod* list = rse->nod_arg[e_rse_items];
 	dsql_nod* list_to = for_select->nod_arg[e_flp_into];
-	if (list->nod_count != list_to->nod_count)
-		ERRD_post(isc_sqlerr, isc_arg_number, (SLONG) - 313,
-				  isc_arg_gds, isc_dsql_count_mismatch, 0);
-	dsql_nod** ptr = list->nod_arg;
-	dsql_nod** ptr_to = list_to->nod_arg;
-	for (const dsql_nod* const* const end = ptr + list->nod_count; ptr < end;
-		ptr++, ptr_to++) 
+
+	if (list_to)
 	{
-		stuff(request, blr_assignment);
-		GEN_expr(request, *ptr);
-		GEN_expr(request, *ptr_to);
+		if (list->nod_count != list_to->nod_count)
+			ERRD_post(isc_sqlerr, isc_arg_number, (SLONG) - 313,
+					isc_arg_gds, isc_dsql_count_mismatch, isc_arg_end);
+		dsql_nod** ptr = list->nod_arg;
+		dsql_nod** ptr_to = list_to->nod_arg;
+		for (const dsql_nod* const* const end = ptr + list->nod_count; ptr < end;
+			ptr++, ptr_to++) 
+		{
+			stuff(request, blr_assignment);
+			GEN_expr(request, *ptr);
+			GEN_expr(request, *ptr_to);
+		}
 	}
+
 	if (for_select->nod_arg[e_flp_action])
 		GEN_statement(request, for_select->nod_arg[e_flp_action]);
 	stuff(request, blr_end);
@@ -2112,7 +2051,7 @@ static void gen_relation( dsql_req* request, dsql_ctx* context)
 	const dsql_rel* relation = context->ctx_relation;
 	const dsql_prc* procedure = context->ctx_procedure;
 
-// if this is a trigger or procedure , don't want relation id used 
+	// if this is a trigger or procedure, don't want relation id used 
 	if (relation) {
 		if (DDL_ids(request)) {
 			if (context->ctx_alias)
@@ -2126,11 +2065,12 @@ static void gen_relation( dsql_req* request, dsql_ctx* context)
 				stuff(request, blr_relation2);
 			else
 				stuff(request, blr_relation);
-			stuff_cstring(request, relation->rel_name);
+			stuff_meta_string(request, relation->rel_name);
 		}
 
 		if (context->ctx_alias)
-			stuff_cstring(request, context->ctx_alias);
+			stuff_meta_string(request, context->ctx_alias);
+
 		stuff_context(request, context);
 	}
 	else if (procedure) {
@@ -2140,7 +2080,7 @@ static void gen_relation( dsql_req* request, dsql_ctx* context)
 		}
 		else {
 			stuff(request, blr_procedure);
-			stuff_cstring(request, procedure->prc_name);
+			stuff_meta_string(request, procedure->prc_name);
 		}
 		stuff_context(request, context);
 
@@ -2233,7 +2173,7 @@ static void gen_rse( dsql_req* request, const dsql_nod* rse)
 {
 	if (rse->nod_flags & NOD_SELECT_EXPR_SINGLETON)
 	{
-			stuff(request, blr_singular);
+		stuff(request, blr_singular);
 	}
 
 	stuff(request, blr_rse);
@@ -2279,10 +2219,10 @@ static void gen_rse( dsql_req* request, const dsql_nod* rse)
 		GEN_expr(request, node);
 	}
 
-    if ((node = rse->nod_arg[e_rse_skip]) != NULL) {
-        stuff(request, blr_skip);
-        GEN_expr (request, node);
-    }
+	if ((node = rse->nod_arg[e_rse_skip]) != NULL) {
+		stuff(request, blr_skip);
+		GEN_expr (request, node);
+	}
 
 	if ((node = rse->nod_arg[e_rse_boolean]) != NULL) {
 		stuff(request, blr_boolean);
@@ -2412,11 +2352,11 @@ static void gen_select( dsql_req* request, dsql_nod* rse)
 	list = rse->nod_arg[e_rse_streams];
 
 	if (!rse->nod_arg[e_rse_reduced]) {
-		dsql_nod* const* ptr = list->nod_arg;
-		for (const dsql_nod* const* const end2 = ptr + list->nod_count; 
-			ptr < end2; ptr++) 
+		dsql_nod* const* ptr2 = list->nod_arg;
+		for (const dsql_nod* const* const end2 = ptr2 + list->nod_count; 
+			ptr2 < end2; ptr2++) 
 		{
-			dsql_nod* item = *ptr;
+			dsql_nod* item = *ptr2;
 			if (item && item->nod_type == nod_relation) {
 				context = (dsql_ctx*) item->nod_arg[e_rel_context];
 				if (relation = context->ctx_relation) {
@@ -2630,6 +2570,213 @@ static void gen_sort( dsql_req* request, dsql_nod* list)
 }
 
 
+/**
+  
+ 	gen_statement
+  
+    @brief	Generate BLR for DML statements.
+ 
+
+    @param request
+    @param node
+
+ **/
+static void gen_statement(dsql_req* request, const dsql_nod* node)
+{
+	dsql_nod* rse = NULL;
+	const dsql_msg* message = NULL;
+	bool send_before_for = !(request->req_flags & REQ_dsql_upd_or_ins);
+
+	switch (node->nod_type) {
+	case nod_store:
+		rse = node->nod_arg[e_sto_rse];
+		break;
+	case nod_modify:
+		rse = node->nod_arg[e_mod_rse];
+		break;
+	case nod_erase:
+		rse = node->nod_arg[e_era_rse];
+		break;
+	default:
+		send_before_for = false;
+		break;
+	}
+
+	if (request->req_type == REQ_EXEC_PROCEDURE && send_before_for)
+	{
+		if ((message = request->req_receive))
+		{
+			stuff(request, blr_send);
+			stuff(request, message->msg_number);
+		}
+	}
+
+	if (rse) {
+		stuff(request, blr_for);
+		GEN_expr(request, rse);
+	}
+
+	if (request->req_type == REQ_EXEC_PROCEDURE)
+	{
+		if ((message = request->req_receive))
+		{
+			stuff(request, blr_begin);
+
+			if (!send_before_for)
+			{
+				stuff(request, blr_send);
+				stuff(request, message->msg_number);
+			}
+		}
+	}
+
+	dsql_nod* temp;
+	dsql_ctx* context;
+	dsql_str* name;
+
+	switch (node->nod_type) {
+	case nod_store:
+		stuff(request, node->nod_arg[e_sto_return] ? blr_store2 : blr_store);
+		GEN_expr(request, node->nod_arg[e_sto_relation]);
+		GEN_statement(request, node->nod_arg[e_sto_statement]);
+		if (node->nod_arg[e_sto_return]) {
+			GEN_statement(request, node->nod_arg[e_sto_return]);
+		}
+		break;
+
+	case nod_modify:
+		stuff(request, node->nod_arg[e_mod_return] ? blr_modify2 : blr_modify);
+		temp = node->nod_arg[e_mod_source];
+		context = (dsql_ctx*) temp->nod_arg[e_rel_context];
+		stuff_context(request, context);
+		temp = node->nod_arg[e_mod_update];
+		context = (dsql_ctx*) temp->nod_arg[e_rel_context];
+		stuff_context(request, context);
+		GEN_statement(request, node->nod_arg[e_mod_statement]);
+		if (node->nod_arg[e_mod_return]) {
+			GEN_statement(request, node->nod_arg[e_mod_return]);
+		}
+		break;
+
+	case nod_modify_current:
+		stuff(request, node->nod_arg[e_mdc_return] ? blr_modify2 : blr_modify);
+		context = (dsql_ctx*) node->nod_arg[e_mdc_context];
+		stuff_context(request, context);
+		temp = node->nod_arg[e_mdc_update];
+		context = (dsql_ctx*) temp->nod_arg[e_rel_context];
+		stuff_context(request, context);
+		GEN_statement(request, node->nod_arg[e_mdc_statement]);
+		if (node->nod_arg[e_mdc_return]) {
+			GEN_statement(request, node->nod_arg[e_mdc_return]);
+		}
+		break;
+
+	case nod_erase:
+		temp = node->nod_arg[e_era_relation];
+		context = (dsql_ctx*) temp->nod_arg[e_rel_context];
+		if (node->nod_arg[e_era_return]) {
+			stuff(request, blr_begin);
+			GEN_statement(request, node->nod_arg[e_era_return]);
+			stuff(request, blr_erase);
+			stuff_context(request, context);
+			stuff(request, blr_end);
+		}
+		else {
+			stuff(request, blr_erase);
+			stuff_context(request, context);
+		}
+		break;
+
+	case nod_erase_current:
+		context = (dsql_ctx*) node->nod_arg[e_erc_context];
+		if (node->nod_arg[e_erc_return]) {
+			stuff(request, blr_begin);
+			GEN_statement(request, node->nod_arg[e_erc_return]);
+			stuff(request, blr_erase);
+			stuff_context(request, context);
+			stuff(request, blr_end);
+		}
+		else {
+			stuff(request, blr_erase);
+			stuff_context(request, context);
+		}
+		break;
+
+	case nod_exec_procedure:
+		stuff(request, blr_exec_proc);
+		name = (dsql_str*) node->nod_arg[e_exe_procedure];
+		stuff_meta_string(request, name->str_data);
+
+		// Input parameters
+		if ( (temp = node->nod_arg[e_exe_inputs]) ) {
+			stuff_word(request, temp->nod_count);
+			dsql_nod** ptr = temp->nod_arg;
+			const dsql_nod* const* end = ptr + temp->nod_count;
+			while (ptr < end)
+			{
+				GEN_expr(request, *ptr++);
+			}
+		}
+		else {
+			stuff_word(request, 0);
+		}
+		// Output parameters
+		if ( ( temp = node->nod_arg[e_exe_outputs]) ) {
+			stuff_word(request, temp->nod_count);
+			dsql_nod** ptr = temp->nod_arg;
+			const dsql_nod* const* end = ptr + temp->nod_count;
+			while (ptr < end)
+			{
+				GEN_expr(request, *ptr++);
+			}
+		}
+		else {
+			stuff_word(request, 0);
+		}
+		break;
+
+	default:
+		fb_assert(false);
+	}
+
+	if (message) {
+		stuff(request, blr_end);
+	}
+}
+
+
+/**
+  
+ 	gen_sys_function
+  
+    @brief	Generate a system defined function.
+ 
+
+    @param request
+    @param node
+
+ **/
+static void gen_sys_function(dsql_req* request, const dsql_nod* node)
+{
+	stuff(request, blr_sys_function);
+	stuff_cstring(request, ((dsql_str*) node->nod_arg[e_sysfunc_name])->str_data);
+
+	const dsql_nod* list;
+	if ((node->nod_count == e_sysfunc_args + 1) && (list = node->nod_arg[e_sysfunc_args]))
+	{
+		stuff(request, list->nod_count);
+		dsql_nod* const* ptr = list->nod_arg;
+		for (const dsql_nod* const* const end = ptr + list->nod_count;
+			ptr < end; ptr++)
+		{
+			GEN_expr(request, *ptr);
+		}
+	}
+	else
+		stuff(request, 0);
+}
+
+
 /**
   
  	gen_table_lock
@@ -2663,7 +2810,7 @@ static void gen_table_lock( dsql_req* request, const dsql_nod* tbl_lock,
 	const USHORT lock_mode = (flags & NOD_WRITE) ? 
 		isc_tpb_lock_write : isc_tpb_lock_read;
 
-    const dsql_nod* const* ptr = tbl_names->nod_arg;
+	const dsql_nod* const* ptr = tbl_names->nod_arg;
 	for (const dsql_nod* const* const end = ptr + tbl_names->nod_count;
 		 ptr < end; ptr++)
 	{
@@ -2726,7 +2873,12 @@ static void gen_udf( dsql_req* request, const dsql_nod* node)
  **/
 static void gen_union( dsql_req* request, const dsql_nod* union_node)
 {
-	stuff(request, blr_union);
+	if (union_node->nod_arg[0]->nod_flags & NOD_UNION_RECURSIVE) {
+		stuff(request, blr_recurse);
+	}
+	else {
+		stuff(request, blr_union);
+	}
 
 // Obtain the context for UNION from the first dsql_map* node
 	dsql_nod* items = union_node->nod_arg[e_rse_items];
@@ -2737,11 +2889,13 @@ static void gen_union( dsql_req* request, const dsql_nod* union_node)
 	}
 	dsql_ctx* union_context = (dsql_ctx*) map_item->nod_arg[e_map_context];
 	stuff_context(request, union_context);
+	// secondary context number must be present once in generated blr 
+	union_context->ctx_flags &= ~CTX_recursive;
 
 	dsql_nod* streams = union_node->nod_arg[e_rse_streams];
 	stuff(request, streams->nod_count);	// number of substreams 
 
-    dsql_nod** ptr = streams->nod_arg;
+	dsql_nod** ptr = streams->nod_arg;
 	for (const dsql_nod* const* const end = ptr + streams->nod_count; ptr < end;
 		 ptr++)
 	{
@@ -2778,9 +2932,17 @@ static void gen_union( dsql_req* request, const dsql_nod* union_node)
 static void stuff_context(dsql_req* request, const dsql_ctx* context)
 {
 	if (context->ctx_context > MAX_UCHAR) {
-		ERRD_post(isc_too_many_contexts, 0);
+		ERRD_post(isc_too_many_contexts, isc_arg_end);
 	}
 	stuff(request, context->ctx_context);
+
+	if (context->ctx_flags & CTX_recursive)
+	{
+		if (context->ctx_recursive > MAX_UCHAR) {
+			ERRD_post(isc_too_many_contexts, isc_arg_end);
+		}
+		stuff(request, context->ctx_recursive);
+	}
 }
 
 
@@ -2797,11 +2959,46 @@ static void stuff_context(dsql_req* request, const dsql_ctx* context)
  **/
 static void stuff_cstring( dsql_req* request, const char* string)
 {
-	UCHAR c;
+	stuff_string(request, string, strlen(string));
+}
 
-	stuff(request, strlen(string));
-	while ((c = *string++))
-		stuff(request, c);
+
+/**
+  
+ 	stuff_meta_string
+  
+    @brief	Write out a string in metadata charset with one byte of length.
+ 
+
+    @param request
+    @param string
+
+ **/
+static void stuff_meta_string(dsql_req* request, const char* string)
+{
+	request->append_meta_string(string);
+}
+
+
+/**
+  
+ 	stuff_string
+  
+    @brief	Write out a string with one byte of length.
+ 
+
+    @param request
+    @param string
+
+ **/
+static void stuff_string(dsql_req* request, const char* string, int len)
+{
+	fb_assert(len >= 0 && len <= 255);
+
+	stuff(request, len);
+
+	while (len--)
+		stuff(request, *string++);
 }
 
 
