@@ -41,6 +41,10 @@
 #include "../dsql/utld_proto.h"
 #include "../jrd/gds_proto.h"
 
+#if !defined(REQUESTER) && !defined(SUPERCLIENT)
+#include "../dsql/metd_proto.h"
+#endif
+
 
 static void cleanup(void *);
 static ISC_STATUS error_dsql_804(ISC_STATUS *, ISC_STATUS);
@@ -51,8 +55,8 @@ static SLONG get_string_info(const SCHAR**, SCHAR*, int);
 static void print_xsqlda(XSQLDA *);
 #endif
 #endif
-static void sqlvar_to_xsqlvar(SQLVAR *, XSQLVAR *);
-static void xsqlvar_to_sqlvar(XSQLVAR *, SQLVAR *);
+static void sqlvar_to_xsqlvar(const SQLVAR*, XSQLVAR*);
+static void xsqlvar_to_sqlvar(const XSQLVAR*, SQLVAR*);
 
 static inline void ch_stuff(BLOB_PTR*& p, const SCHAR value, bool& same_flag)
 {
@@ -77,6 +81,69 @@ static TEXT *DSQL_failures, *DSQL_failures_ptr;
 
 const int  DSQL_FAILURE_SPACE = 2048;
 
+/**
+	Parse response on isc_info_sql_select or isc_info_sql_bind 
+	request. Return pointer to the next byte after successfully
+	parsed info or NULL if error is encountered or info is truncated
+**/
+SCHAR* UTLD_skip_sql_info(SCHAR* info)
+{
+	if (*info != isc_info_sql_select &&
+		*info != isc_info_sql_bind)
+	{
+		return 0;
+	}
+
+	info++;
+
+	if (*info++ != isc_info_sql_describe_vars)
+		return 0;
+
+	get_numeric_info((const SCHAR**) &info);
+
+	// Loop over the variables being described
+	while (true)
+	{
+		SCHAR str[256]; // must be big enough to hold metadata name
+		SCHAR item = *info++;
+
+		switch (item)
+		{
+		case isc_info_end:
+			return info;
+
+		case isc_info_truncated:
+			return 0;
+
+		case isc_info_sql_select:
+		case isc_info_sql_bind:
+			return --info;
+
+		case isc_info_sql_describe_end:
+			break;
+
+		case isc_info_sql_sqlda_seq:
+		case isc_info_sql_type:
+		case isc_info_sql_sub_type:
+		case isc_info_sql_scale:
+		case isc_info_sql_length:
+			get_numeric_info((const SCHAR**) &info);
+			break;
+
+		case isc_info_sql_field:
+		case isc_info_sql_relation:
+		case isc_info_sql_owner:
+		case isc_info_sql_alias:
+			get_string_info((const SCHAR**) &info, str, sizeof(str));
+			break;
+
+		default:
+			return 0;
+		}
+	}
+
+	return 0;
+}
 
 
 /**
@@ -920,7 +987,7 @@ static void print_xsqlda( XSQLDA * xsqlda)
     @param xsqlvar
 
  **/
-static void sqlvar_to_xsqlvar( SQLVAR * sqlvar, XSQLVAR * xsqlvar)
+static void sqlvar_to_xsqlvar(const SQLVAR* sqlvar, XSQLVAR* xsqlvar)
 {
 
 	xsqlvar->sqltype = sqlvar->sqltype;
@@ -930,21 +997,24 @@ static void sqlvar_to_xsqlvar( SQLVAR * sqlvar, XSQLVAR * xsqlvar)
 	xsqlvar->sqlsubtype = 0;
 	xsqlvar->sqlscale = 0;
 	xsqlvar->sqllen = sqlvar->sqllen;
-	if ((xsqlvar->sqltype & ~1) == SQL_LONG) {
+	switch (xsqlvar->sqltype & ~1)
+	{
+	case SQL_LONG:
 		xsqlvar->sqlscale = xsqlvar->sqllen >> 8;
 		xsqlvar->sqllen = sizeof(SLONG);
-	}
-	else if ((xsqlvar->sqltype & ~1) == SQL_SHORT) {
+		break;
+	case SQL_SHORT:
 		xsqlvar->sqlscale = xsqlvar->sqllen >> 8;
 		xsqlvar->sqllen = sizeof(SSHORT);
-	}
-	else if ((xsqlvar->sqltype & ~1) == SQL_INT64) {
+		break;
+	case SQL_INT64:
 		xsqlvar->sqlscale = xsqlvar->sqllen >> 8;
 		xsqlvar->sqllen = sizeof(SINT64);
-	}
-	else if ((xsqlvar->sqltype & ~1) == SQL_QUAD) {
+		break;
+	case SQL_QUAD:
 		xsqlvar->sqlscale = xsqlvar->sqllen >> 8;
 		xsqlvar->sqllen = sizeof(ISC_QUAD);
+		break;
 	}
 }
 
@@ -960,7 +1030,7 @@ static void sqlvar_to_xsqlvar( SQLVAR * sqlvar, XSQLVAR * xsqlvar)
     @param sqlvar
 
  **/
-static void xsqlvar_to_sqlvar( XSQLVAR * xsqlvar, SQLVAR * sqlvar)
+static void xsqlvar_to_sqlvar(const XSQLVAR* xsqlvar, SQLVAR* sqlvar)
 {
 
 	sqlvar->sqltype = xsqlvar->sqltype;
@@ -971,13 +1041,35 @@ static void xsqlvar_to_sqlvar( XSQLVAR * xsqlvar, SQLVAR * sqlvar)
 	memcpy(sqlvar->sqlname, xsqlvar->aliasname, sizeof(sqlvar->sqlname));
 
 	sqlvar->sqllen = xsqlvar->sqllen;
-	if ((sqlvar->sqltype & ~1) == SQL_LONG)
-		sqlvar->sqllen = sizeof(SLONG) | (xsqlvar->sqlscale << 8);
-	else if ((sqlvar->sqltype & ~1) == SQL_SHORT)
-		sqlvar->sqllen = sizeof(SSHORT) | (xsqlvar->sqlscale << 8);
-	else if ((sqlvar->sqltype & ~1) == SQL_INT64)
-		sqlvar->sqllen = sizeof(SINT64) | (xsqlvar->sqlscale << 8);
-	else if ((sqlvar->sqltype & ~1) == SQL_QUAD)
-		sqlvar->sqllen = sizeof(ISC_QUAD) | (xsqlvar->sqlscale << 8);
+	const USHORT scale = xsqlvar->sqlscale << 8;
+	switch (sqlvar->sqltype & ~1)
+	{
+	case SQL_LONG:
+		sqlvar->sqllen = sizeof(SLONG) | scale;
+		break;
+	case SQL_SHORT:
+		sqlvar->sqllen = sizeof(SSHORT) | scale;
+		break;
+	case SQL_INT64:
+		sqlvar->sqllen = sizeof(SINT64) | scale;
+		break;
+	case SQL_QUAD:
+		sqlvar->sqllen = sizeof(ISC_QUAD) | scale;
+		break;
+	}
 }
 
+
+#if !defined(REQUESTER) && !defined(SUPERCLIENT)
+
+UCHAR DSqlDataTypeUtil::maxBytesPerChar(UCHAR charSet)
+{
+	return METD_get_charset_bpc(request, charSet);
+}
+
+USHORT DSqlDataTypeUtil::getDialect() const
+{
+	return request->req_client_dialect;
+}
+
+#endif
