@@ -739,7 +739,7 @@ static void cmp_field( gpre_req* request, const gpre_fld* field,
 
 	switch (field->fld_dtype) {
 	case dtype_cstring:
-		if (!(field->fld_flags & FLD_charset)) {
+		if (!(field->fld_flags & FLD_charset) && field->fld_ttype) {
 			request->add_byte(blr_cstring);
 			request->add_word(field->fld_length);
 		}
@@ -753,7 +753,7 @@ static void cmp_field( gpre_req* request, const gpre_fld* field,
 		break;
 
 	case dtype_text:
-		if (!(field->fld_flags & FLD_charset)) {
+		if (!(field->fld_flags & FLD_charset) && field->fld_ttype) {
 			request->add_byte(blr_text);
 			request->add_word(field->fld_length);
 		}
@@ -767,7 +767,7 @@ static void cmp_field( gpre_req* request, const gpre_fld* field,
 		break;
 
 	case dtype_varying:
-		if (!(field->fld_flags & FLD_charset)) {
+		if (!(field->fld_flags & FLD_charset) && field->fld_ttype) {
 			request->add_byte(blr_varying);
 			request->add_word(field->fld_length);
 		}
@@ -1503,11 +1503,12 @@ static void cmp_slice( gpre_req* request)
 	request->add_byte(isc_sdl_field);
 	CMP_stuff_symbol(request, field->fld_symbol);
 
-	bool loop_flags[16];
+	bool loop_flags[MAX_ARRAY_DIMENSIONS];
 	{ // scope block
-	USHORT n = 0;
-	for (bool* p = loop_flags; n < slice->slc_dimensions; n++, p++)
-		*p = cmp_sdl_loop(request, n, slice, array);
+		fb_assert(slice->slc_dimensions <= MAX_ARRAY_DIMENSIONS);
+		USHORT n = 0;
+		for (bool* p = loop_flags; n < slice->slc_dimensions; n++, p++)
+			*p = cmp_sdl_loop(request, n, slice, array);
 	} // end scope block
 	
 	request->add_byte(isc_sdl_element);
