@@ -54,7 +54,18 @@
  *
  * 2002.10.30 Sean Leyne - Removed support for obsolete "PC_PLATFORM" define
  *
- */
+ * 2006.08.31 Stephen W. Boyd        - Added support for RM/Cobol
+ * 2006.10.12 Stephen W. Boyd        - Added support for FOR UPDATE WITH LOCK
+ *
+ * Added -noqli switch to suppress parsing of QLI keywords.  This was
+ * causing problems with Cobol programs since the QLI and Cobol reserved word
+ * lists intersect.
+ * Stephen W. Boyd 21.Mar.2007
+ *
+ * 2007.05.23 Stephen W. Boyd		- Added support for SQL "first n" and "skip n" clauses
+ * 2007.06.15 Stephen W. Boyd		- Added support for CURRENT_CONNECTION, CURRENT_ROLE,
+ *									  CURRENT_TRANSACTION and CURRENT_USER context variables.
+*/
 
 #ifndef GPRE_GPRE_H
 #define GPRE_GPRE_H
@@ -64,6 +75,7 @@
 #include <stdio.h>
 #include "../jrd/common.h"
 #include "../jrd/ibase.h"
+#include "../jrd/constants.h"
 
 #ifdef GPRE_FORTRAN
 #if defined AIX || defined AIX_PPC || defined sun
@@ -103,9 +115,9 @@ const int MAX_EVENT_SIZE = 16; // event names use 15 chars from old docs.
 // Values for SQL dialects.
 #include "../dsql/sqlda_pub.h"
 
-/* Language options */
+// Language options
 
-typedef enum lang_t
+enum lang_t
 {
 	lang_undef,
 	lang_internal,
@@ -120,7 +132,16 @@ typedef enum lang_t
 	lang_cplusplus,
 	lang_cpp,
 	lang_internal_cxx
-} LANG_T;
+};
+
+// Cobol dialect options
+enum cob_t
+{
+	cob_vms,					// VMS
+	cob_ansi,					// ANSI-85
+	cob_rmc						// RM/Cobol
+};
+
 
 //___________________________________________________________________
 // Test if input language is cpp based.
@@ -136,7 +157,8 @@ typedef enum lang_t
 // and lang_internal.
 //
 
-bool isLangCpp(LANG_T lang);
+bool isLangCpp(lang_t lang);
+bool isAnsiCobol(cob_t dialect);
 
 
 /* Structure used by Fortran and Basic to determine whether or not
@@ -228,7 +250,7 @@ typedef enum nod_t {
 	nod_label, nod_leave, nod_loop,
 	nod_max, nod_min, nod_count,
 	nod_total, nod_average, nod_list,
-	nod_defered, nod_missing, nod_between,
+	nod_deferred, nod_missing, nod_between,
 	nod_union, nod_map, nod_starting,
 	nod_like, nod_agg_count, nod_agg_max,
 	nod_agg_min, nod_agg_total, nod_agg_average,
@@ -244,7 +266,7 @@ typedef enum nod_t {
 	nod_natural, nod_index_order, nod_ansi_all,
 	nod_extract, nod_current_date,
 	nod_current_time, nod_current_timestamp,
-	nod_lowcase,
+	nod_lowcase, nod_current_connection, nod_current_role, nod_current_transaction,
 	nod_LASTNOD					/* Leave this debugging GPRE_NOD last */
 } NOD_T;
 
@@ -344,8 +366,6 @@ const size_t VAL_LEN = sizeof(val);
    Note: the dimension (dim) block used to hold dimension information.
    The preferred mechanism is the repeating tail on the array block. */
 
-const int MAX_ARRAY_DIMENSIONS = 16;
-
 struct ary {
 	USHORT ary_dtype;			/* data type of array */
 	int ary_dimension_count;	/* Number of dimensions in this array */
@@ -898,6 +918,8 @@ struct gpre_rse {
 	USHORT rse_type;			/* node type */
 	gpre_nod* rse_boolean;		/* boolean expression, if present */
 	gpre_nod* rse_first;		/* "first n" clause, if present */
+	gpre_nod* rse_sqlfirst;		/* SQL "first n" clause if present */
+	gpre_nod* rse_sqlskip;		/* SQL "skip n" clause if present */
 	gpre_nod* rse_reduced;		/* projection clause, if present */
 	gpre_nod* rse_sort;			/* sort clause, if present */
 	gpre_nod* rse_fields;		/* list of fields */
@@ -923,7 +945,9 @@ inline size_t RSE_LEN(const size_t cnt)
 }
 
 enum rse_flags_vals {
-	RSE_singleton = 1
+	RSE_singleton = 1,
+	RSE_for_update = 2,
+	RSE_with_lock = 4
 };
 
 
@@ -1450,7 +1474,7 @@ enum udf_flags_vals {
 };
 
 
-/* Update block -- used for (at least) MODIFY */
+// Update block -- used for (at least) MODIFY
 
 struct upd {
 	USHORT upd_level;			/* reference level */
@@ -1469,11 +1493,12 @@ const size_t UPD_LEN = sizeof(upd);
 #include "../jrd/dsc.h"
 #include "parse.h"
 
-/* GPRE wide globals */
+// GPRE wide globals
 
 struct GpreGlobals
 {
-	bool sw_ansi;
+	cob_t sw_cob_dialect;
+	const TEXT* sw_cob_dformat;
 	bool sw_auto;
 	bool sw_sql;
 	bool sw_raw;
@@ -1484,12 +1509,14 @@ struct GpreGlobals
 	bool sw_external;
 	bool sw_version;
 	bool sw_d_float;
+	bool sw_no_qli;
 	USHORT sw_sql_dialect;
 	USHORT sw_know_interp;
 	USHORT sw_server_version;
 	USHORT sw_ods_version;
 	bool override_case;
 	bool dialect_specified;
+	bool trusted_auth;
 
 	SSHORT sw_interp;
 	USHORT compiletime_db_dialect;
@@ -1498,25 +1525,25 @@ struct GpreGlobals
 	const TEXT* ada_null_address;
 	DBB isc_databases;
 	const TEXT* default_user;
-	TEXT* default_password;
+	const TEXT* default_password;
 	const TEXT* default_lc_ctype;
 	gpre_req* requests;
 	gpre_lls* events;
 	FILE *out_file;
-	LANG_T sw_language;
+	lang_t sw_language;
 	int errors_global;
 	act* global_functions;
 	dbd global_db_list[MAX_DATABASES];
 	USHORT global_db_count;
 	INTLSYM text_subtypes;
 
-/* ada_flags fields definition */
+	// ada_flags fields definition
 
 	int ADA_create_database;	// the flag is set when there is a
-									// create database SQL statement in
-									// user program, and is used to
-									// generate additional "with" and
-									// "function" declarations
+								// create database SQL statement in
+								// user program, and is used to
+								// generate additional "with" and
+								// "function" declarations
 
 	USHORT ada_flags;
 	// from gpre.cpp
@@ -1539,6 +1566,7 @@ struct GpreGlobals
 
 extern GpreGlobals gpreGlob;
 
+
 #ifndef fb_assert
 #ifdef DEV_BUILD
 #undef fb_assert
@@ -1555,7 +1583,7 @@ extern GpreGlobals gpreGlob;
 #define assert_IS_ACT(x) fb_assert(!(x) || ((x)->act_type >= 0 && (x)->act_type < ACT_LASTACT))
 
 
-class gpre_exception: public std::exception
+class gpre_exception: public Firebird::LongJump
 {
 	char msg[MAXPATHLEN << 1];
 public:
@@ -1570,7 +1598,7 @@ public:
 	}
 	const char* what() const throw()
 	{
-		return msg;
+		return msg[0] ? msg : "gpre_exception";
 	}
 };
 
