@@ -1,7 +1,7 @@
 /*
  *	PROGRAM:		Firebird exceptions classes
  *	MODULE:			fb_exception.h
- *	DESCRIPTION:	Specific exception classes derived from std::exception
+ *	DESCRIPTION:	Firebird's exception classes 
  *
  *  The contents of this file are subject to the Initial
  *  Developer's Public License Version 1.0 (the "License");
@@ -30,17 +30,15 @@
 #ifndef FB_EXCEPTION_H
 #define FB_EXCEPTION_H
 
-#include <exception>
 #include <stddef.h>
 #include <string.h>
 #include "fb_types.h"
 
-
 namespace Firebird {
 
 class StringsBuffer {
 public:
-	virtual char* alloc(const char* string, size_t length) = 0;
+	virtual char* alloc(const char* string, size_t& length) = 0;
 	virtual ~StringsBuffer() {}
 
 	void makePermanentVector(ISC_STATUS* perm, const ISC_STATUS* temp);
@@ -54,11 +52,15 @@ public:
 		memset(buffer, 0, BUFFER_SIZE); 
 		buffer_ptr = buffer;
 	}
-	virtual char* alloc(const char* string, size_t length) {
-		// fb_assert(length + 1 < BUFFER_SIZE);
+	virtual char* alloc(const char* string, size_t& length) {
+		// truncate too long strings
+		if (length > BUFFER_SIZE / 4)
+			length = BUFFER_SIZE / 4;
+
 		// If there isn't any more room in the buffer, start at the beginning again
 		if (buffer_ptr + length + 1 > buffer + BUFFER_SIZE)
 			buffer_ptr = buffer;
+
 		char* new_string = buffer_ptr;
 		memcpy(new_string, string, length);
 		new_string[length] = 0;
@@ -70,15 +72,48 @@ private:
 	char *buffer_ptr;
 };
 
-class status_exception : public std::exception
+class Exception
+{
+protected:
+	Exception() throw() { }
+public:
+	virtual ~Exception() throw() { }
+	virtual ISC_STATUS stuff_exception(ISC_STATUS* const status_vector, StringsBuffer* sb = NULL) const throw() = 0;
+	virtual const char* what() const throw() = 0;
+};
+
+// Used as jmpbuf to unwind when needed
+class LongJump : public Exception
+{
+public:
+	virtual ISC_STATUS stuff_exception(ISC_STATUS* const status_vector, StringsBuffer* sb = NULL) const throw();
+	virtual const char* what() const throw() { return "Firebird::LongJump"; }
+	static void raise();
+	LongJump() throw() : Exception() { }
+};
+
+// Used in MemoryPool
+class BadAlloc : public Exception
+{
+public:
+	virtual ISC_STATUS stuff_exception(ISC_STATUS* const status_vector, StringsBuffer* sb = NULL) const throw();
+	virtual const char* what() const throw() { return "Firebird::BadAlloc"; }
+	static void raise();
+	BadAlloc() throw() : Exception() { }
+};
+
+// Main exception class in firebird
+class status_exception : public Exception
 {
 public:
 	// This version of constructor receives status vector pointing to permanent or 
 	// temp strings, depending upon second parameter.
 	status_exception(const ISC_STATUS *status_vector, bool permanent) throw();
 	virtual ~status_exception() throw();
-	
+
+	virtual ISC_STATUS stuff_exception(ISC_STATUS* const status_vector, StringsBuffer* sb = NULL) const throw();
 	virtual const char* what() const throw() { return "Firebird::status_exception"; }
+
 	const ISC_STATUS* value() const throw() { return m_status_vector; }
 
 	// Returns true if strings contained in status vector are located in magical 
@@ -86,17 +121,9 @@ public:
 	// and is about to deallocate them in its destructor
 	bool strings_permanent() const throw() { return m_strings_permanent; }
 
-	// Returns true if exception class holds status vector for the error.
-	// Returns false when status vector is passed "by magic", probably 
-	// somewhere in tdbb_status_vector
-	bool status_known() const throw() { return m_status_known; }
-
 	// Takes permanent strings
 	static void raise(const ISC_STATUS *status_vector);
 	
-	// Used as jmpbuf to unwind when needed
-	static void raise();
-	
 	// Take transient strings
 	static void raise(ISC_STATUS status, ...);
 	
@@ -110,7 +137,6 @@ protected:
 private:
 	ISC_STATUS_ARRAY m_status_vector;
 	bool m_strings_permanent;
-	bool m_status_known;
 	void release_vector() throw();
 };
 
@@ -147,7 +173,7 @@ public:
 
 
 // Serialize exception into status_vector, put transient strings from exception into given StringsBuffer
-ISC_STATUS stuff_exception(ISC_STATUS *status_vector, const std::exception& ex, StringsBuffer* sb = NULL) throw();
+ISC_STATUS stuff_exception(ISC_STATUS *status_vector, const Firebird::Exception& ex, StringsBuffer* sb = NULL) throw();
 
 // These routines put strings into process-level circular buffer
 // They are obsolete, use transient version of status_exception::raise in combination with
