@@ -33,15 +33,15 @@ ULONG CVJIS_eucj_to_unicode(csconvert* obj,
 							ULONG src_len,
 							const UCHAR* src_ptr,
 							ULONG dest_len,
-							USHORT *dest_ptr,
-							USHORT *err_code,
-							ULONG *err_position)
+							UCHAR* p_dest_ptr,
+							USHORT* err_code,
+							ULONG* err_position)
 {
-	fb_assert(src_ptr != NULL || dest_ptr == NULL);
+	fb_assert(src_ptr != NULL || p_dest_ptr == NULL);
 	fb_assert(err_code != NULL);
 	fb_assert(err_position != NULL);
 	fb_assert(obj != NULL);
-	fb_assert(obj->csconvert_fn_convert == reinterpret_cast<pfn_INTL_convert>(CVJIS_eucj_to_unicode));
+	fb_assert(obj->csconvert_fn_convert == CVJIS_eucj_to_unicode);
 	fb_assert(obj->csconvert_impl->csconvert_datatable != NULL);
 	fb_assert(obj->csconvert_impl->csconvert_misc != NULL);
 
@@ -49,9 +49,12 @@ ULONG CVJIS_eucj_to_unicode(csconvert* obj,
 	*err_code = 0;
 
 /* See if we're only after a length estimate */
-	if (dest_ptr == NULL)
+	if (p_dest_ptr == NULL)
 		return (src_len);
 
+	Firebird::OutAligner<USHORT> d(p_dest_ptr, dest_len);
+	USHORT* dest_ptr = d;
+
 	USHORT ch;
 	USHORT wide;
 	USHORT this_len;
@@ -128,15 +131,15 @@ ULONG CVJIS_sjis_to_unicode(csconvert* obj,
 							ULONG sjis_len,
 							const UCHAR* sjis_str,
 							ULONG dest_len,
-							USHORT *dest_ptr,
-							USHORT *err_code,
-							ULONG *err_position)
+							UCHAR* p_dest_ptr,
+							USHORT* err_code,
+							ULONG* err_position)
 {
-	fb_assert(sjis_str != NULL || dest_ptr == NULL);
+	fb_assert(sjis_str != NULL || p_dest_ptr == NULL);
 	fb_assert(err_code != NULL);
 	fb_assert(err_position != NULL);
 	fb_assert(obj != NULL);
-	fb_assert(obj->csconvert_fn_convert == reinterpret_cast<pfn_INTL_convert>(CVJIS_sjis_to_unicode));
+	fb_assert(obj->csconvert_fn_convert == CVJIS_sjis_to_unicode);
 	fb_assert(obj->csconvert_impl->csconvert_datatable != NULL);
 	fb_assert(obj->csconvert_impl->csconvert_misc != NULL);
 
@@ -144,9 +147,12 @@ ULONG CVJIS_sjis_to_unicode(csconvert* obj,
 	*err_code = 0;
 
 /* See if we're only after a length estimate */
-	if (dest_ptr == NULL)
+	if (p_dest_ptr == NULL)
 		return (sjis_len * 2);	/* worst case - all ascii input */
 
+	Firebird::OutAligner<USHORT> d(p_dest_ptr, dest_len);
+	USHORT* dest_ptr = d;
+
 	USHORT table;
 	USHORT this_len;
 	USHORT wide;
@@ -381,17 +387,17 @@ I hope this helps in the discussion.
 
 ULONG CVJIS_unicode_to_sjis(csconvert* obj,
 							ULONG unicode_len,
-							const USHORT* unicode_str,
+							const UCHAR* p_unicode_str,
 							ULONG sjis_len,
 							UCHAR* sjis_str,
-							USHORT *err_code,
-							ULONG *err_position)
+							USHORT* err_code,
+							ULONG* err_position)
 {
-	fb_assert(unicode_str != NULL || sjis_str == NULL);
+	fb_assert(p_unicode_str != NULL || sjis_str == NULL);
 	fb_assert(err_code != NULL);
 	fb_assert(err_position != NULL);
 	fb_assert(obj != NULL);
-	fb_assert(obj->csconvert_fn_convert == reinterpret_cast<pfn_INTL_convert>(CVJIS_unicode_to_sjis));
+	fb_assert(obj->csconvert_fn_convert == CVJIS_unicode_to_sjis);
 	fb_assert(obj->csconvert_impl->csconvert_datatable != NULL);
 	fb_assert(obj->csconvert_impl->csconvert_misc != NULL);
 
@@ -402,6 +408,9 @@ ULONG CVJIS_unicode_to_sjis(csconvert* obj,
 	if (sjis_str == NULL)
 		return (unicode_len);	/* worst case - all han character input */
 
+	Firebird::Aligner<USHORT> s(p_unicode_str, unicode_len);
+	const USHORT* unicode_str = s;
+
 	const UCHAR* const start = sjis_str;
 	while ((sjis_len) && (unicode_len > 1)) {
 		/* Step 1: Convert from UNICODE to JIS code */
@@ -458,25 +467,28 @@ ULONG CVJIS_unicode_to_sjis(csconvert* obj,
 }
 
 
-ULONG CVJIS_unicode_to_eucj(csconvert* obj, ULONG unicode_len, const USHORT* unicode_str,
-							ULONG eucj_len, UCHAR *eucj_str,
-							USHORT *err_code, ULONG *err_position)
+ULONG CVJIS_unicode_to_eucj(csconvert* obj, ULONG unicode_len, const UCHAR* p_unicode_str,
+							ULONG eucj_len, UCHAR* eucj_str,
+							USHORT* err_code, ULONG* err_position)
 {
-	fb_assert(unicode_str != NULL || eucj_str == NULL);
+	fb_assert(p_unicode_str != NULL || eucj_str == NULL);
 	fb_assert(err_code != NULL);
 	fb_assert(err_position != NULL);
 	fb_assert(obj != NULL);
-	fb_assert(obj->csconvert_fn_convert == reinterpret_cast<pfn_INTL_convert>(CVJIS_unicode_to_eucj));
+	fb_assert(obj->csconvert_fn_convert == CVJIS_unicode_to_eucj);
 	fb_assert(obj->csconvert_impl->csconvert_datatable != NULL);
 	fb_assert(obj->csconvert_impl->csconvert_misc != NULL);
 
 	const ULONG src_start = unicode_len;
 	*err_code = 0;
 
-/* See if we're only after a length estimate */
+	// See if we're only after a length estimate
 	if (eucj_str == NULL)
 		return (unicode_len);	/* worst case - all han character input */
 
+	Firebird::Aligner<USHORT> s(p_unicode_str, unicode_len);
+	const USHORT* unicode_str = s;
+
 	const UCHAR* const start = eucj_str;
 	while ((eucj_len) && (unicode_len > 1)) {
 		/* Step 1: Convert from UNICODE to JIS code */
@@ -770,11 +782,11 @@ CONVERT_ENTRY(CS_SJIS, CS_EUCJ, CVJIS_sjis_x_eucj)
 {
 	if (dest_cs == CS_EUCJ)
 		CV_convert_init(csptr, dest_cs, source_cs,
-						reinterpret_cast<pfn_INTL_convert>(CVJIS_sjis2euc),
+						CVJIS_sjis2euc,
 						NULL, NULL);
 	else
 		CV_convert_init(csptr, dest_cs, source_cs,
-						reinterpret_cast<pfn_INTL_convert>(CVJIS_euc2sjis),
+						CVJIS_euc2sjis,
 						NULL, NULL);
 
 	CONVERT_RETURN;
