@@ -24,12 +24,10 @@
 #ifndef INTL_LDCOMMON_H
 #define INTL_LDCOMMON_H
 
-/* #include "../jrd/gdsassert.h" */
-/* Put the assert in here */
-
 #include "../common/classes/alloc.h"
 #include "../jrd/intlobj_new.h"
 #include "../jrd/constants.h"
+#include "../jrd/gdsassert.h"
 #include "../intl/charsets.h"
 #include "../intl/country_codes.h"
 #include "../intl/ld.h"
@@ -39,20 +37,6 @@
 
 typedef USHORT UNICODE;
 
-/* Redirect the assertion code defined by gdsassert.h to a local routine */
-#ifdef fb_assert
-#undef fb_assert
-#endif
-#ifndef DEV_BUILD
-#define ERR_assert				/* nothing */
-#define fb_assert(ex)				/* nothing */
-#else
-#include <stdlib.h> /* prototype for abort() */
-#define ERR_assert	LD_assert
-#define fb_assert(ex)	{if (!(ex)) {LD_assert (__FILE__, __LINE__); abort();}}
-
-#endif
-
 #ifndef MIN
 #define MIN(x, y) ((x) < (y) ? (x) : (y))
 #endif
@@ -62,9 +46,11 @@ typedef USHORT UNICODE;
 
 
 
-#define	TEXTTYPE_ENTRY(name)	INTL_BOOL name (TEXTTYPE cache, const ASCII* tt_name, const ASCII* cs_name, \
-												USHORT attributes, const UCHAR* specific_attributes, \
-												ULONG specific_attributes_length)
+#define	TEXTTYPE_ENTRY(name)	INTL_BOOL name (texttype* cache, charset* cs,										\
+												const ASCII* tt_name, const ASCII* cs_name,							\
+												USHORT attributes,													\
+												const UCHAR* specific_attributes, ULONG specific_attributes_length,	\
+												const ASCII* config_info)
 
 
 
@@ -91,4 +77,3 @@ inline void put(UCHAR*& dest, const C src)
 }
 
 #endif /* INTL_LDCOMMON_H */
-
