@@ -23,6 +23,8 @@
  *  All Rights Reserved.
  *  Contributor(s): ______________________________________.
  *
+ *
+ *
  */
 
 #ifndef JRD_RECORDNUMBER_H
@@ -36,7 +38,6 @@ const SINT64 BOF_NUMBER = QUADCONST(-1);
 // compiler check errors on our migration path from 32-bit to 64-bit record 
 // numbers.
 class RecordNumber {
-
 public:
 	// Packed record number represents common layout of RDB$DB_KEY and BLOB ID.
 	// To ensure binary compatibility with old (pre-ODS11) databases it differs 
@@ -89,15 +90,14 @@ public:
 		}
 	};
 	
-public:
 	// Default constructor.
-	inline RecordNumber() : value(EMPTY_NUMBER) {}
+	inline RecordNumber() : value(EMPTY_NUMBER), valid(false) {}
 
 	// Copy constructor
-	inline RecordNumber(const RecordNumber& from) : value(from.value) {}
+	inline RecordNumber(const RecordNumber& from) : value(from.value), valid(from.valid) {}
 
 	// Explicit constructor from 64-bit record number value
-	inline explicit RecordNumber(SINT64 number) : value(number) {}
+	inline explicit RecordNumber(SINT64 number) : value(number), valid(true) {}
 
 	// Assignment operator
 	inline RecordNumber& operator =(const RecordNumber& from) { 
@@ -141,6 +141,8 @@ public:
 
 	bool isEmpty() const { return value == EMPTY_NUMBER; }
 
+	bool isValid() const { return valid; }
+
 	inline bool checkNumber(
 		USHORT records_per_page, // ~400 (8k page)
 		USHORT data_pages_per_pointer_page  // ~2000 (8k page)
@@ -194,11 +196,17 @@ public:
 		value = recno->bid_decode();
 	}
 
+	inline void setValid(bool to_value)
+	{
+		valid = to_value;
+	}
+
 private:
 	// Use signed value because negative values are widely used as flags in the 
 	// engine. Since page number is the signed 32-bit integer and it is also may
 	// be stored in this structure we want sign extension to take place.
 	SINT64 value;
+	bool valid;
 };
 
 #endif // JRD_RECORDNUMBER_H
