@@ -19,8 +19,6 @@
  *
  * All Rights Reserved.
  * Contributor(s): ______________________________________.
- * Added TCP_NO_DELAY option for superserver on Linux
- * FSG 16.03.2001
  *
  * 2001.07.06 Sean Leyne - Code Cleanup, removed "#ifdef READONLY_DATABASE"
  *                         conditionals, as the engine now fully supports
@@ -84,6 +82,7 @@
 # endif
 #endif
 
+
 #ifdef SUPERSERVER
 #define GOVERNOR
 #define CANCEL_OPERATION
@@ -94,7 +93,7 @@
 
 
 /*****************************************************
-* Linux for Intel platforms 
+* Linux platforms 
 *****************************************************/
 #ifdef LINUX
 #define QUADFORMAT "ll"
@@ -112,17 +111,13 @@
 //format for __LINE__
 #define LINEFORMAT "d"
 
-#ifdef SUPERSERVER
-#define SET_TCP_NO_DELAY
-#endif
-
 //#define KILLER_SIGNALS
 
 #define UNIX
 #define IEEE
 
 #ifdef AMD64
-#define IMPLEMENTATION  isc_info_db_impl_linux_amd64 /* 66  next higher unique number, See you later  */
+#define IMPLEMENTATION  isc_info_db_impl_linux_amd64 /* 66 */
 #endif
 
 #ifdef PPC
@@ -131,14 +126,31 @@
 
 #ifdef i386
 #define I386
-#define IMPLEMENTATION  isc_info_db_impl_i386 /* 60  next higher unique number, See you later  */
+#define IMPLEMENTATION  isc_info_db_impl_i386 /* 60 */
 #endif /* i386 */
 
+#ifdef ARM
+#define IMPLEMENTATION  isc_info_db_impl_linux_arm
+#endif /* ARM */
+
 #ifdef sparc
 #define IMPLEMENTATION  isc_info_db_impl_linux_sparc /* 65  */
 #define RISC_ALIGNMENT
 #endif /* sparc */
 
+#ifdef MIPSEL
+#define IMPLEMENTATION  isc_info_db_impl_linux_mipsel /* 71  */
+#endif /* mipsel */
+
+#ifdef MIPS
+#define IMPLEMENTATION  isc_info_db_impl_linux_mips /* 72  */
+#endif /* mips */
+
+#ifdef IA64
+#define IMPLEMENTATION  isc_info_db_impl_linux_ia64	// 76
+#define RISC_ALIGNMENT
+#endif // IA64
+
 #define MEMMOVE(from, to, length)		memmove ((void *)to, (void *)from, (size_t) length)
 #define MOVE_FAST(from, to, length)       memcpy (to, from, (int) (length))
 #define MOVE_FASTER(from, to, length)     memcpy (to, from, (int) (length))
@@ -146,118 +158,6 @@
 
 #endif /* LINUX */
 
-/*****************************************************
-* SINIX-Z 5.42
-*****************************************************/
-#ifdef SINIXZ
-#include <sys/types.h>
-#include <sys/time.h>
-#include <sys/mman.h>
-#include <sys/socket.h>
-#include <string.h>
-
-#ifdef __cplusplus
-extern "C" {
-#endif
-
-/* These prototypes are missing in the system header files :-( */
-int gettimeofday (struct timeval *tp);
-int munmap(void * addr, size_t len);
-int gethostname(char *name, size_t len);
-int socket(int domain, int type, int protocol);
-int connect(int s, struct sockaddr *name, int namelen);
-int send(int s, void *msg, int len, int flags);
-int recv(int s, void *buf, int len, int flags);
-int strcasecmp(const char *s1, const char *s2);
-int strncasecmp(const char *s1, const char *s2, size_t n);
-int select(int nfds, fd_set *readfds, fd_set *writefds, fd_set *execptfds, struct timeval *timeout);
-int getsockopt(int s, int level, int optname, char *optval, int *optlen);
-int setsockopt(int s, int level, int optname, char *optval, int optlen);
-int bind(int s, struct sockaddr *name, int namelen);
-int listen(int s, int backlog);
-int accept(int s, struct sockaddr *ddr, int *addrlen);
-int getsockname(int s, struct sockaddr *name, int *namelen);
-int setsockname(int s, struct sockaddr *name, int *namelen);
-int getpeername(int s, struct sockaddr *name, int *namelen);
-int shutdown(int s, int how);
-int syslog(int pri, char *fmt, ...);
-
-#ifdef __cplusplus
-    }
-#endif
-
-#include <dlfcn.h>
-#define dlopen(a, b)		dlopen((char *)(a),(b))
-#define dlsym(a, b)		dlsym((a), (char *)(b))
-
-#include <signal.h>
-#include <sys/siginfo.h>
-
-struct sinixz_sigaction
-  {
-    int sa_flags;
-    union
-      {
-        /* Used if SA_SIGINFO is not set.  */
-        void (*sa_handler)(int);
-        /* Used if SA_SIGINFO is set.  */
-        void (*sa_sigaction) (int, siginfo_t *, void *);
-      }
-    __sigaction_handler;
-#define sa_handler		__sigaction_handler.sa_handler
-#define sa_sigaction		__sigaction_handler.sa_sigaction
-    sigset_t sa_mask;
-    int sa_resv[2];
-  };
-
-static inline int sinixz_sigaction(int sig, const struct sinixz_sigaction *act,
-                                   struct sinixz_sigaction *oact)
-{
-  return sigaction(sig, (struct sigaction*)act, (struct sigaction*)oact);
-}
-
-// Re-define things actually
-#define sigaction		sinixz_sigaction
-
-#define QUADFORMAT "ll"
-#define QUADCONST(n) (n##LL)
-
-//#define ALIGNMENT	4
-//#define DOUBLE_ALIGN	8
-
-#ifdef SUPERSERVER
-#define SET_TCP_NO_DELAY
-#endif
-
-//#define KILLER_SIGNALS
-
-#define UNIX
-#define IEEE
-
-#ifdef i386
-#define I386
-/* Change version string into SINIXZ */
-#define IMPLEMENTATION  isc_info_db_impl_sinixz  /* 64 */
-#endif /* i386 */
-
-#define setreuid(ruid, euid)     setuid(euid)
-#define setregid(rgid, egid)     setgid(egid)
-
-#define MEMMOVE(from, to, length)		memmove ((void *)to, (void *)from, (size_t) length)
-#define MOVE_FAST(from, to, length)       memcpy (to, from, (int) (length))
-#define MOVE_FASTER(from, to, length)     memcpy (to, from, (int) (length))
-#define MOVE_CLEAR(to, length)           memset (to, 0, (int) (length))
-
-//format for __LINE__
-#define LINEFORMAT "d"
-
-//#define SLONGFORMAT "ld"
-//#define ULONGFORMAT "lu"
-//#define XLONGFORMAT "lX"
-//#define xLONGFORMAT "lx"
-
-#define O_BINARY 0
-#endif /* SINIXZ */
 
 /*****************************************************
 * Darwin Platforms 
@@ -280,12 +180,21 @@ static inline int sinixz_sigaction(int sig, const struct sinixz_sigaction *act,
 //#define DOUBLE_ALIGN    4
 //#define BSD_UNIX
 #define UNIX
+#ifdef  __ppc__
+#define powerpc
+#define IMPLEMENTATION isc_info_db_impl_darwin_ppc /* 63 */
+#endif
 #ifdef i386
 #define I386
 #define IMPLEMENTATION isc_info_db_impl_darwin_x86 /* 70 */
 #endif
-#ifdef __ppc__
-#define IMPLEMENTATION isc_info_db_impl_darwin_ppc /* 63 */
+#ifdef __x86_64__
+#define DARWIN64
+#define IMPLEMENTATION isc_info_db_impl_darwin_x64 /* 73 */
+#endif
+#ifdef __ppc64__
+#define DARWINPPC64
+#define IMPLEMENTATION isc_info_db_impl_darwin_ppc64 /* 77 */
 #endif
 #define IEEE
 #define QUADCONST(n) (n##LL)
@@ -297,6 +206,11 @@ static inline int sinixz_sigaction(int sig, const struct sinixz_sigaction *act,
 #define MOVE_FASTER(from, to, length)	memcpy (to, from, (int) (length))
 #define MOVE_CLEAR(to, length)		memset (to, 0, (int) (length))
 
+#define API_ROUTINE __attribute__((visibility("default")))
+#define API_ROUTINE_VARARG API_ROUTINE
+#define INTERNAL_API_ROUTINE API_ROUTINE
+#define FB_EXPORTED __attribute__((visibility("default")))
+
 #endif /* Darwin Platforms */
 
 
@@ -387,11 +301,8 @@ static inline int sinixz_sigaction(int sig, const struct sinixz_sigaction *act,
  * in Solaris
  */
 #define SOLARIS_MT
-
-#ifdef SOLARIS_MT
-#define ANY_THREADING
 #define MULTI_THREAD
-#endif
+
 /*  Define the following only on platforms whose standard I/O
  *  implementation is so weak that we wouldn't be able to fopen
  *  a file whose underlying file descriptor would be > 255.
@@ -459,16 +370,16 @@ static inline int sinixz_sigaction(int sig, const struct sinixz_sigaction *act,
 
 #define IMPLEMENTATION  isc_info_db_impl_isc_sun4 /* 30 */
 #define RISC_ALIGNMENT
-#else /* sparc */
+#endif /* sparc */
 
 #ifdef i386
 #define I386
 #define IMPLEMENTATION  isc_info_db_impl_isc_sun_386i  /* 32 */
-#else /* i386 */
-#define IMPLEMENTATION  isc_info_db_impl_isc_sun_68k /* 28 */
-#endif /* i386 */
+#endif
 
-#endif /* sparc */
+#ifdef AMD64
+#define IMPLEMENTATION  isc_info_db_impl_sun_amd64 /* 74 */
+#endif /* AMD64 */
 
 #define MOVE_FAST(from, to, length)       memcpy (to, from, (int) (length))
 #define MOVE_FASTER(from, to, length)     memcpy (to, from, (int) (length))
@@ -621,13 +532,15 @@ typedef unsigned __int64 FB_UINT64;
    not permit the LL suffix which some other platforms require, but it
    handles numbers up to the largest 64-bit integer correctly without such
    a suffix, so the macro definition is trivial. */
-#ifdef MINGW // needed for gcc 3.3.1
+#ifdef __GNUC__ // needed for gcc 3.3.1
 #define QUADCONST(n) (n##LL)
 #else
 #define QUADCONST(n) (n)
 #endif
 
-#ifdef _X86_
+#ifdef AMD64
+#define IMPLEMENTATION  isc_info_db_impl_winnt_amd64 /* 68 */
+#else
 #ifndef I386
 #define I386
 #endif
@@ -717,18 +630,22 @@ typedef unsigned __int64 FB_UINT64;
 #define H_ERRNO		h_errno
 #endif
 
+
 /* various declaration modifiers */
 
 #ifndef API_ROUTINE
 #define API_ROUTINE
 #define API_ROUTINE_VARARG
-#define INTERNAL_API_ROUTINE	API_ROUTINE
+#define INTERNAL_API_ROUTINE
 #endif
 
 #ifndef CLIB_ROUTINE
 #define CLIB_ROUTINE
 #endif
 
+#ifndef FB_EXPORTED
+#define FB_EXPORTED
+#endif
 
 
 /* alignment macros */
@@ -1050,6 +967,16 @@ void GDS_breakpoint(int);
 #define DOUBLE_DIVIDE(a, b)      (((double) (a)) / ((double) (b)))
 #endif
 
+#define STRINGIZE_AUX(x)	#x
+#define STRINGIZE(x)		STRINGIZE_AUX(x)
+
+#ifdef _MSC_VER
+#define CONST64(a) (a##i64)
+#else
+#define CONST64(a) (a##LL)
+#endif
+
+
 /* switch name and state table.  This structure should be used in all
  * command line tools to facilitate parsing options.*/
 struct in_sw_tab_t {
