@@ -27,12 +27,16 @@
 USHORT DFW_assign_index_type(Jrd::DeferredWork*, SSHORT, SSHORT);
 void DFW_delete_deferred(Jrd::jrd_tra*, SLONG);
 void DFW_merge_work(Jrd::jrd_tra*, SLONG, SLONG);
-void DFW_perform_system_work(void);
+void DFW_perform_system_work(Jrd::thread_db*);
 void DFW_perform_work(Jrd::jrd_tra*);
 void DFW_perform_post_commit_work(Jrd::jrd_tra*);
+Jrd::DeferredWork* DFW_post_system_work(Jrd::thread_db*, enum Jrd::dfw_t, const dsc*, USHORT);
 Jrd::DeferredWork* DFW_post_work(Jrd::jrd_tra*, enum Jrd::dfw_t, const dsc*, USHORT);
 Jrd::DeferredWork* DFW_post_work_arg(Jrd::jrd_tra*, Jrd::DeferredWork*, const dsc*, USHORT);
 void DFW_update_index(const TEXT*, USHORT, const Jrd::SelectivityList&);
 
+Jrd::Lock* DFW_protect_relation(Jrd::thread_db*, Jrd::jrd_tra*, Jrd::jrd_rel*, bool&);
+void DFW_release_protect_lock(Jrd::thread_db*, Jrd::jrd_tra*, Jrd::Lock*);
+
 #endif // JRD_DFW_PROTO_H
 
