@@ -34,7 +34,7 @@
 #include "../jrd/pag.h"
 #include "../jrd/val.h"
 #include "../jrd/btr.h"
-//#include "../common/classes/timestamp.h"
+#include "../common/classes/timestamp.h"
 #include "../jrd/tra.h"
 #include "../jrd/cch_proto.h"
 #include "../jrd/dmp_proto.h"
@@ -521,11 +521,11 @@ static void dmp_data(const data_page* page)
 			fprintf(dbg_file, "\n\t%d - (empty)\n", i);
 			continue;
 		}
-		const rhd* header = (RHD) ((SCHAR *) page + index->dpg_offset);
-		const rhdf* fragment = (RHDF) header;
+		const rhd* header = (rhd*) ((SCHAR *) page + index->dpg_offset);
+		const rhdf* fragment = (rhdf*) header;
 		if (header->rhd_flags & rhd_blob)
 		{
-			const blh* blob = (BLH) header;
+			const blh* blob = (blh*) header;
 			fprintf(dbg_file,
 					   "\n\t%d - (blob) offset: %d, length: %d, flags: %x\n",
 					   i, index->dpg_offset, index->dpg_length,
@@ -548,12 +548,12 @@ static void dmp_data(const data_page* page)
 			{
 				if (header->rhd_flags & rhd_incomplete)
 				{
-					length = index->dpg_length - OFFSETA(RHDF, rhdf_data);
-					p = (SCHAR *) ((RHDF) header)->rhdf_data;
+					length = index->dpg_length - OFFSETA(rhdf*, rhdf_data);
+					p = (SCHAR *) ((rhdf*) header)->rhdf_data;
 				}
 				else
 				{
-					length = index->dpg_length - OFFSETA(RHD, rhd_data);
+					length = index->dpg_length - OFFSETA(rhd*, rhd_data);
 					p = (SCHAR *) header->rhd_data;
 				}
 				const char* q = p;
@@ -665,8 +665,11 @@ static void dmp_header(const header_page* page)
 			   page->hdr_page_size, page->hdr_ods_version & ~ODS_FIREBIRD_FLAG, 
 			   minor_version, page->hdr_ods_minor_original, page->hdr_PAGES);
 
+	const Firebird::TimeStamp ts(*((GDS_TIMESTAMP *) page->hdr_creation_date));
+
 	struct tm time;
-	isc_decode_timestamp((GDS_TIMESTAMP *) page->hdr_creation_date, &time);
+	ts.decode(&time);
+
 	fprintf(dbg_file, "\tCreation date:\t%s %d, %d %d:%02d:%02d\n",
 			   FB_SHORT_MONTHS[time.tm_mon], time.tm_mday, time.tm_year + 1900,
 			   time.tm_hour, time.tm_min, time.tm_sec);
@@ -780,7 +783,7 @@ static void dmp_index(const btree_page* page, USHORT page_size)
 			(USHORT)((ULONG) ((page_size - OFFSETA(pointer_page*, ppg_page)) * 8) /
 						(BITS_PER_LONG + 2));
 	const USHORT max_records = (page_size - sizeof(data_page)) /
-								(sizeof(data_page::dpg_repeat) + OFFSETA(RHD, rhd_data));
+								(sizeof(data_page::dpg_repeat) + OFFSETA(rhd*, rhd_data));
 
 	const btree_nod* const end  = (btree_nod*) ((UCHAR *) page + page->btr_length);
 	btree_nod* node = (btree_nod*) page->btr_nodes;
@@ -961,7 +964,7 @@ static void dmp_root(const index_root_page* page)
 			   ((PAG) page)->pag_checksum, ((PAG) page)->pag_generation,
 			   page->irt_relation, page->irt_count);
 	const bool ods11plus =
-		(JRD_get_thread_data()->tdbb_database->dbb_ods_version >= ODS_VERSION11);
+		(JRD_get_thread_data()->getDatabase()->dbb_ods_version >= ODS_VERSION11);
 	USHORT i = 0;
 	for (const index_root_page::irt_repeat* desc = page->irt_rpt;
 		i < page->irt_count; i++, desc++)
@@ -997,16 +1000,18 @@ static void dmp_transactions(const tx_inv_page* page, ULONG sequence)
  *
  **************************************/
 	thread_db* tdbb = JRD_get_thread_data();
-	Database* dbb = tdbb->tdbb_database;
+	Database* dbb = tdbb->getDatabase();
 
 	const ULONG transactions_per_tip = dbb->dbb_pcontrol->pgc_tpt;
 
 	fprintf(dbg_file,
 			   "Transaction Inventory Page\t checksum %d\t generation %ld\n",
 			   ((PAG) page)->pag_checksum, ((PAG) page)->pag_generation);
-	if (tdbb->tdbb_transaction)
+	if (tdbb->getTransaction())
+	{
 		fprintf(dbg_file, "\tCurrent transaction %d",
-				   tdbb->tdbb_transaction->tra_number);
+				   tdbb->getTransaction()->tra_number);
+	}
 	else
 		fprintf(dbg_file, "\tCurrent transaction (NULL)");
 	if (sequence)
