@@ -44,7 +44,6 @@
 #include "../jrd/req.h"
 #include "../jrd/rse.h"
 #include "../jrd/tra.h"
-#include "../jrd/all_proto.h"
 #include "../jrd/cch_proto.h"
 #include "../jrd/met_proto.h"
 #endif
@@ -106,10 +105,9 @@ void ERR_bugcheck_msg(const TEXT* msg)
 
 	dbb->dbb_flags |= DBB_bugcheck;
 	DEBUG;
-
 	CCH_shutdown_database(dbb);
 
-	ERR_post(isc_bug_check, isc_arg_string, ERR_cstring(msg), 0);
+	ERR_post(isc_bug_check, isc_arg_string, ERR_cstring(msg), isc_arg_end);
 }
 #endif
 
@@ -192,7 +190,7 @@ void ERR_duplicate_error(IDX_E	code,
 	switch (code) {
 	case idx_e_keytoobig:
 		ERR_post(isc_imp_exc, isc_arg_gds, isc_keytoobig,
-				 isc_arg_string, index_name, 0);
+				 isc_arg_string, index_name, isc_arg_end);
 		break;
 
 	case idx_e_conversion:
@@ -201,23 +199,23 @@ void ERR_duplicate_error(IDX_E	code,
 
 	case idx_e_foreign_target_doesnt_exist:
 		ERR_post(isc_foreign_key, isc_arg_string, constraint_name,
-			 	 isc_arg_string, ERR_string(relation->rel_name), 
-			 	 isc_arg_gds, isc_foreign_key_target_doesnt_exist, 0);
+			 	 isc_arg_string, ERR_cstring(relation->rel_name), 
+			 	 isc_arg_gds, isc_foreign_key_target_doesnt_exist, isc_arg_end);
 		break;
 
 	case idx_e_foreign_references_present:
 		ERR_post(isc_foreign_key, isc_arg_string, constraint_name,
-			 	 isc_arg_string, ERR_string(relation->rel_name),
-			 	 isc_arg_gds, isc_foreign_key_references_present, 0);
+			 	 isc_arg_string, ERR_cstring(relation->rel_name),
+			 	 isc_arg_gds, isc_foreign_key_references_present, isc_arg_end);
 		break;
 
 	default:
 		if (constraint.length() > 0)
 			ERR_post(isc_unique_key_violation,
 					 isc_arg_string, constraint_name,
-					 isc_arg_string, ERR_string(relation->rel_name), 0);
+					 isc_arg_string, ERR_cstring(relation->rel_name), isc_arg_end);
 		else
-			ERR_post(isc_no_dup, isc_arg_string, index_name, 0);
+			ERR_post(isc_no_dup, isc_arg_string, index_name, isc_arg_end);
 	}
 }
 #endif
@@ -243,7 +241,7 @@ void ERR_error(int number)
 	if (gds__msg_lookup(0, JRD_BUGCHK, number, sizeof(errmsg), errmsg, NULL) < 1)
 		sprintf(errmsg, "error code %d", number);
 
-	ERR_post(isc_random, isc_arg_string, ERR_cstring(errmsg), 0);
+	ERR_post(isc_random, isc_arg_string, ERR_cstring(errmsg), isc_arg_end);
 }
 #endif
 
@@ -264,7 +262,7 @@ void ERR_error_msg(const TEXT* msg)
  **************************************/
 
 	DEBUG;
-	ERR_post(isc_random, isc_arg_string, ERR_cstring(msg), 0);
+	ERR_post(isc_random, isc_arg_string, ERR_cstring(msg), isc_arg_end);
 }
 #endif
 
@@ -297,9 +295,8 @@ void ERR_log(int facility, int number, const TEXT* message)
 	const size_t len = strlen(errmsg);
 	fb_utils::snprintf(errmsg + len, sizeof(errmsg) - len, " (%d)", number);
 
-	gds__log("Database: %s\n\t%s", (tdbb && tdbb->tdbb_attachment) ?
-		tdbb->tdbb_attachment->att_filename.c_str() : "",
-		errmsg, 0);
+	gds__log("Database: %s\n\t%s", (tdbb && tdbb->getAttachment()) ?
+		tdbb->getAttachment()->att_filename.c_str() : "", errmsg);
 }
 #endif
 
@@ -455,17 +452,16 @@ static void internal_post(ISC_STATUS status, va_list args)
  *  Used in ERR_post and ERR_post_nothrow
  *
  **************************************/
-	ISC_STATUS_ARRAY tmp_status, warning_status;
-	int i, tmp_status_len = 0, status_len = 0, err_status_len = 0;
-	int warning_count = 0, warning_indx = 0;
 
 	ISC_STATUS* status_vector = ((thread_db*) JRD_get_thread_data())->tdbb_status_vector;
 
 /* stuff the status into temp buffer */
+	ISC_STATUS_ARRAY tmp_status;
 	MOVE_CLEAR(tmp_status, sizeof(tmp_status));
 	STUFF_STATUS_function(tmp_status, status, args);
 
 /* calculate length of the status */
+	int tmp_status_len = 0, warning_indx = 0;
 	PARSE_STATUS(tmp_status, tmp_status_len, warning_indx);
 	fb_assert(warning_indx == 0);
 
@@ -479,11 +475,13 @@ static void internal_post(ISC_STATUS status, va_list args)
 		return;
 	}
 
+	int status_len = 0;
 	PARSE_STATUS(status_vector, status_len, warning_indx);
 	if (status_len)
 		--status_len;
 
-/* check for duplicated error code */
+	/* check for duplicated error code */
+	int i;
 	for (i = 0; i < ISC_STATUS_LENGTH; i++) {
 		if (status_vector[i] == isc_arg_end && i == status_len)
 			break;				/* end of argument list */
@@ -503,9 +501,12 @@ static void internal_post(ISC_STATUS status, va_list args)
 	}
 
 /* if the status_vector has only warnings then adjust err_status_len */
-	if ((err_status_len = i) == 2 && warning_indx)
+	int err_status_len = i;
+	if (err_status_len == 2 && warning_indx)
 		err_status_len = 0;
 
+	ISC_STATUS_ARRAY warning_status;
+	int warning_count = 0;
 	if (warning_indx) {
 		/* copy current warning(s) to a temp buffer */
 		MOVE_CLEAR(warning_status, sizeof(warning_status));
@@ -546,12 +547,12 @@ void ERR_punt(void)
  **************************************/
 
 	thread_db* tdbb = JRD_get_thread_data();
-	Database* dbb = tdbb->tdbb_database;
+	Database* dbb = tdbb->getDatabase();
 
 	if (dbb && (dbb->dbb_flags & DBB_bugcheck))
 	{
-		gds__log_status(tdbb->tdbb_attachment->att_filename.hasData() ?
-			tdbb->tdbb_attachment->att_filename.c_str() : "Database unknown in ERR_punt on bugcheck",
+		gds__log_status(tdbb->getAttachment()->att_filename.hasData() ?
+			tdbb->getAttachment()->att_filename.c_str() : "Database unknown in ERR_punt on bugcheck",
 			tdbb->tdbb_status_vector);
  		if (Config::getBugcheckAbort())
 		{
@@ -606,7 +607,7 @@ void ERR_warning(ISC_STATUS status, ...)
 
 	STUFF_STATUS(tdbb->tdbb_status_vector, status);
 	DEBUG;
-	tdbb->tdbb_request->req_flags |= req_warning;
+	tdbb->getRequest()->req_flags |= req_warning;
 }
 #endif
 
@@ -649,7 +650,7 @@ static void internal_error(ISC_STATUS status, int number,
 		fb_utils::snprintf(errmsg + len, sizeof(errmsg) - len, " (%d)", number);
 	}
 
-	ERR_post(status, isc_arg_string, ERR_cstring(errmsg), 0);
+	ERR_post(status, isc_arg_string, ERR_cstring(errmsg), isc_arg_end);
 }
 #endif
 
