@@ -65,10 +65,6 @@
 #define MUTEX		event_mutex
 #endif
 
-#ifdef SERVER
-#undef MULTI_THREAD
-#endif
-
 #ifndef AST_TYPE
 #define AST_TYPE	void
 #endif
@@ -126,7 +122,6 @@ static SH_MEM_T EVENT_data;
 static MTX_T event_mutex[1];
 #endif
 
-
 void EVENT_cancel(SLONG request_id)
 {
 /**************************************
@@ -139,8 +134,6 @@ void EVENT_cancel(SLONG request_id)
  *	Cancel an outstanding event.
  *
  **************************************/
-	srq *event_srq, *que2;
-
 	if (!EVENT_header)
 		return;
 
@@ -148,8 +141,10 @@ void EVENT_cancel(SLONG request_id)
 
 	PRB process = (PRB) SRQ_ABS_PTR(EVENT_process_offset);
 
+	srq* que2;
 	SRQ_LOOP(process->prb_sessions, que2) {
 		SES session = (SES) ((UCHAR *) que2 - OFFSET(SES, ses_sessions));
+		srq* event_srq;
 		SRQ_LOOP(session->ses_requests, event_srq) {
 			EVT_REQ request = (EVT_REQ) ((UCHAR *) event_srq - OFFSET(EVT_REQ, req_requests));
 			if (request->req_request_id == request_id) {
@@ -164,7 +159,8 @@ void EVENT_cancel(SLONG request_id)
 }
 
 
-SLONG EVENT_create_session(ISC_STATUS * status_vector)
+#ifndef SUPERCLIENT
+SLONG EVENT_create_session(ISC_STATUS* status_vector)
 {
 /**************************************
  *
@@ -178,7 +174,7 @@ SLONG EVENT_create_session(ISC_STATUS * status_vector)
  **************************************/
 // If we're not initialized, do so now.
 
-	if (!EVENT_header && !EVENT_init(status_vector, true))
+	if (!EVENT_header && !EVENT_init(status_vector))
 		return 0;
 
 	if (!EVENT_process_offset)
@@ -200,6 +196,7 @@ SLONG EVENT_create_session(ISC_STATUS * status_vector)
 
 	return id;
 }
+#endif
 
 
 void EVENT_delete_session(SLONG session_id)
@@ -242,7 +239,6 @@ void EVENT_deliver()
  *  for delivery with EVENT_post.
  *
  **************************************/
-	srq	*event_srq;
 
 	/* If we're not initialized, do so now */
 
@@ -257,6 +253,7 @@ void EVENT_deliver()
 
 	while (flag) {
 		flag = false;
+		srq* event_srq;
 		SRQ_LOOP (EVENT_header->evh_processes, event_srq)	{
 			PRB process = (PRB) ((UCHAR*) event_srq - OFFSET (PRB, prb_processes));
 			if (process->prb_flags & PRB_wakeup) {
@@ -271,9 +268,7 @@ void EVENT_deliver()
 }
 
 
-// CVC: Contrary to the explanation, server_flag is not used!
-#pragma FB_COMPILER_MESSAGE("server_flag var is not honored: bug or deprecated?")
-EVH EVENT_init(ISC_STATUS* status_vector, bool server_flag)
+EVH EVENT_init(ISC_STATUS* status_vector)
 {
 /**************************************
  *
@@ -282,9 +277,8 @@ EVH EVENT_init(ISC_STATUS* status_vector, bool server_flag)
  **************************************
  *
  * Functional description
- *	Initialize for access to shared global region.  If "server_flag" is true,
- *	create region if it doesn't exist.  Return address of header if region
- *	exits, otherwise return NULL.
+ *	Initialize for access to shared global region.
+ *	Return address of header.
  *
  **************************************/
 	TEXT buffer[MAXPATHLEN];
@@ -296,28 +290,7 @@ EVH EVENT_init(ISC_STATUS* status_vector, bool server_flag)
 
 	EVENT_default_size = Config::getEventMemSize();
 
-#ifdef SERVER
-	EVENT_data.sh_mem_address = EVENT_header =
-		(EVH) gds__alloc((SLONG) EVENT_default_size);
-/* FREE: apparently only freed at process exit */
-	if (!EVENT_header) {		/* NOMEM: */
-		status_vector[0] = isc_arg_gds;
-		status_vector[1] = isc_virmemexh;
-		status_vector[2] = isc_arg_end;
-		return NULL;
-	}
-#ifdef DEBUG_GDS_ALLOC
-/* This structure is apparently only freed when the process exits */
-/* 1994-October-25 David Schnepper */
-	gds_alloc_flag_unfreed((void *) EVENT_header);
-#endif /* DEBUG_GDS_ALLOC */
-
-	EVENT_data.sh_mem_length_mapped = EVENT_default_size;
-	EVENT_data.sh_mem_mutex_arg = 0;
-	init((SLONG) 0, &EVENT_data, true);
-#else
-
-#if (defined UNIX)
+#ifdef UNIX
 	EVENT_data.sh_mem_semaphores = SEMAPHORES;
 #endif
 
@@ -329,14 +302,9 @@ EVH EVENT_init(ISC_STATUS* status_vector, bool server_flag)
 											init, 0, EVENT_default_size,
 											&EVENT_data))) 
 	{
-#ifdef SERVER
-		gds__free(EVENT_data.sh_mem_address);
-#endif /* SERVER */
 		return NULL;
 	}
 
-#endif
-
 	gds__register_cleanup(exit_handler, 0);
 
 #ifdef UNIX
@@ -366,11 +334,10 @@ int EVENT_post(ISC_STATUS * status_vector,
  *	Post an event.
  *
  **************************************/
-	srq *event_srq;
 
 /* If we're not initialized, do so now */
 
-	if (!EVENT_header && !EVENT_init(status_vector, false))
+	if (!EVENT_header && !EVENT_init(status_vector))
 		return status_vector[1];
 
 	acquire();
@@ -381,6 +348,7 @@ int EVENT_post(ISC_STATUS * status_vector,
 		(event = find_event(minor_length, minor_code, parent)))
 	{
 		event->evnt_count += count;
+		srq* event_srq;
 		SRQ_LOOP(event->evnt_interests, event_srq) {
 			RINT interest = (RINT) ((UCHAR *) event_srq - OFFSET(RINT, rint_interests));
 			if (interest->rint_request) {
@@ -417,6 +385,12 @@ SLONG EVENT_que(ISC_STATUS* status_vector,
  * Functional description
  *
  **************************************/
+// Sanity check
+
+	if (events[0] != EPB_version1) {
+		ERR_post(isc_random, isc_arg_string, "Invalid EPB form", isc_arg_end);
+	}
+
 // Allocate request block
 
 	acquire();
@@ -454,6 +428,13 @@ SLONG EVENT_que(ISC_STATUS* status_vector,
 
 		const USHORT count = *p++;
 
+		/* Sanity check */
+
+		if (count > end - events) {
+			release();
+			ERR_post(isc_random, isc_arg_string, "Invalid EPB form", isc_arg_end);
+		}
+
 		/* The data in the event block may have trailing blanks.  Strip them off. */
 
         const UCHAR* find_end = p + count;
@@ -536,10 +517,8 @@ static EVH acquire(void)
 	EVENT_header->evh_current_process = EVENT_process_offset;
 #else
 	if (++acquire_count == 1) {
-#ifndef SERVER
 		if (mutex_state = ISC_mutex_lock(MUTEX))
 			mutex_bugcheck("mutex lock", mutex_state);
-#endif
 		EVENT_header->evh_current_process = EVENT_process_offset;
 	}
 #endif
@@ -547,34 +526,16 @@ static EVH acquire(void)
 	if (EVENT_header->evh_length > EVENT_data.sh_mem_length_mapped) {
 		const SLONG length = EVENT_header->evh_length;
 
-#ifdef WIN_NT
-		/* Before remapping the memory, wakeup the watcher thread.
-		 * Then remap the shared memory and allow the watcher thread
-		 * to remap.
-		 */
-		/* Need to make the following code generic to all SUPERSERVER
-		 * platforms. Postponed for now. B.Sriram, 10-Jul-1997
-		 */
-
+		EVH header = NULL;
+#if (defined HAVE_MMAP || defined WIN_NT)
 		PRB process = (PRB) SRQ_ABS_PTR(EVENT_process_offset);
-		process->prb_flags |= PRB_remap;
-		event_t* event = process->prb_event;
-
-		post_process(process);
-
-		while (true) {
-			release();
-			Sleep(3);
-			acquire();
-
-			process = (PRB) SRQ_ABS_PTR(EVENT_process_offset);
-			if (!(process->prb_flags & PRB_remap))
-				break;
+		if (process->prb_flags & PRB_dont_remap)
+		{
+			ISC_event_post(process->prb_event);
+			while (process->prb_flags & PRB_dont_remap)
+				THREAD_SLEEP(10);
 		}
-#endif /* WIN_NT */
 
-	EVH header = NULL;
-#if (!(defined SUPERSERVER) && (defined HAVE_MMAP))
 		ISC_STATUS_ARRAY status_vector;
 		header = (evh*) ISC_remap_file(status_vector, &EVENT_data, length, false);
 #endif
@@ -584,11 +545,6 @@ static EVH acquire(void)
 			exit(FINI_ERROR);
 		}
 		EVENT_header = header;
-
-#ifdef WIN_NT
-		process = (PRB) SRQ_ABS_PTR(EVENT_process_offset);
-		process->prb_flags &= ~PRB_remap_over;
-#endif /* WIN_NT */
 	}
 
 	return EVENT_header;
@@ -628,33 +584,16 @@ static FRB alloc_global(UCHAR type, ULONG length, bool recurse)
 		const SLONG old_length = EVENT_data.sh_mem_length_mapped;
 		const SLONG ev_length = old_length + EVENT_EXTEND_SIZE;
 
-#ifdef WIN_NT
-
-		/* Before remapping the memory, wakeup the watcher thread.
-		 * Then remap the shared memory and allow the watcher thread
-		 * to remap.
-		 */
-
-		PRB process = (PRB) SRQ_ABS_PTR(EVENT_process_offset);
-		process->prb_flags |= PRB_remap;
-		event_t* event = process->prb_event;
-		post_process(process);
-
-		while (true) {
-			release();
-			THREAD_EXIT();
-			Sleep(3);
-			THREAD_ENTER();
-			acquire();
-
-			process = (PRB) SRQ_ABS_PTR(EVENT_process_offset);
-			if (!(process->prb_flags & PRB_remap))
-				break;
-		}
-#endif /* WIN_NT */
-
 		EVH header = 0;
-#if !((defined SUPERSERVER) && (defined HAVE_MMAP))
+#if (defined HAVE_MMAP || defined WIN_NT)
+		PRB process = (PRB) SRQ_ABS_PTR(EVENT_process_offset);
+		if (process->prb_flags & PRB_dont_remap)
+		{
+			ISC_event_post(process->prb_event);
+			while (process->prb_flags & PRB_dont_remap)
+				THREAD_SLEEP(10);
+		}
+
 		ISC_STATUS_ARRAY status_vector;
 		header =
 			reinterpret_cast<EVH>
@@ -675,11 +614,6 @@ static FRB alloc_global(UCHAR type, ULONG length, bool recurse)
 
 			free_global(free);
 
-#ifdef WIN_NT
-			process = (PRB) SRQ_ABS_PTR(EVENT_process_offset);
-			process->prb_flags &= ~PRB_remap_over;
-#endif /* WIN_NT */
-
 			return alloc_global(type, length, true);
 		}
 	}
@@ -730,16 +664,13 @@ static SLONG create_process(void)
 	EVENT_process_offset = SRQ_REL_PTR(process);
 
 #ifdef MULTI_THREAD
-
 #ifdef SOLARIS_MT
 	ISC_STATUS_ARRAY local_status;
 	ISC_event_init(process->prb_event, 0, EVENT_SIGNAL);
 	EVENT_process = (PRB) ISC_map_object(local_status, &EVENT_data,
 										 EVENT_process_offset,
 										 sizeof(prb));
-#endif
-
-#if !(defined SOLARIS_MT)
+#else
 	ISC_event_init(process->prb_event, EVENT_SIGNAL, 0);
 #endif
 #endif
@@ -750,9 +681,7 @@ static SLONG create_process(void)
 	release();
 
 #ifdef MULTI_THREAD
-	if (gds__thread_start
-		(watcher_thread, NULL,
-		 THREAD_medium, THREAD_blast, 0))
+	if (gds__thread_start(watcher_thread, NULL, THREAD_medium, THREAD_blast, 0))
 		ERR_bugcheck_msg("cannot start thread");
 #endif
 
@@ -807,37 +736,28 @@ static void delete_process(SLONG process_offset)
 		delete_session(SRQ_REL_PTR(session));
 	}
 
-/* Untangle and release process block */
-
-	remove_que(&process->prb_processes);
-	free_global((FRB) process);
-
 	if (EVENT_process_offset == process_offset) {
 #ifdef MULTI_THREAD
 		/* Terminate the event watcher thread */
-		/* When we come through the exit handler, the event semaphore might
-		   have already been released by another exit handler.  So we cannot
-		   use that semaphore to post the event.  Besides, the watcher thread 
-		   would be terminated anyway because the whole NLM is being unloaded. */
-		// CVC: NLM??? is this Novell Netware specific code???
 
 		process->prb_flags |= PRB_exiting;
-		BOOLEAN timeout = FALSE;
-		while (process->prb_flags & PRB_exiting && !timeout) {
+		while (process->prb_flags & PRB_exiting) {
 			ISC_event_post(process->prb_event);
-			SLONG value = ISC_event_clear(process->prb_event);
+
 			release();
-#ifdef SOLARIS_MT
-			event_t* events = EVENT_process->prb_event;
-#else
-			event_t* events = process->prb_event;
-#endif
-			timeout = ISC_event_wait(1, &events, &value, 5 * 1000000, 0, 0);
+			THREAD_SLEEP(10);
 			acquire();
+
+			process = (PRB) SRQ_ABS_PTR(process_offset);
 		}
 		EVENT_process_offset = 0;
 #endif
 	}
+
+/* Untangle and release process block */
+
+	remove_que(&process->prb_processes);
+	free_global((FRB) process);
 }
 
 
@@ -887,33 +807,22 @@ static void delete_session(SLONG session_id)
  *	Delete a session.
  *
  **************************************/
-#ifdef MULTI_THREAD
-/*  This code is very Netware specific, so for now I've #ifdef'ed this
- *  variable out.  In the future, other platforms may find a need to
- *  use this as well.  --  Morgan Schweers, November 16, 1994
- */
-	USHORT kill_anyway = 0;		/*  Kill session despite session delivering. */
-	/*  (Generally means session's deliver is hung. */
-#endif
-
 	SES session = (SES) SRQ_ABS_PTR(session_id);
 
 #ifdef MULTI_THREAD
-/*  delay gives up control for 250ms, so a 40 iteration timeout is a
- *  up to 10 second wait for session to finish delivering its message.
- */
-	while (session->ses_flags & SES_delivering && (++kill_anyway != 40)) {
+	// if session currently delivered events, delay its deletion until deliver ends
+	if (session->ses_flags & SES_delivering)
+	{
+		session->ses_flags |= SES_purge;
+
+		// give a chance for delivering thread to detect SES_purge flag we just set
 		release();
 		THREAD_EXIT();
-#ifdef WIN_NT
-		Sleep(250);
-#endif
-#if (defined SOLARIS_MT || defined LINUX)
-		sleep(1);
-#endif
+		THREAD_SLEEP(100);
 		THREAD_ENTER();
 		acquire();
-		session = (SES) SRQ_ABS_PTR(session_id);
+
+		return;
 	}
 #endif
 
@@ -955,8 +864,6 @@ static AST_TYPE deliver(void* arg)
  *	We've been poked -- deliver any satisfying requests.
  *
  **************************************/
-	srq *event_srq, *que2;
-
 #ifdef UNIX
 	if (acquire_count)
 		return;
@@ -966,7 +873,9 @@ static AST_TYPE deliver(void* arg)
 	PRB process = (PRB) SRQ_ABS_PTR(EVENT_process_offset);
 	process->prb_flags &= ~PRB_pending;
 
-	SRQ_LOOP(process->prb_sessions, que2) {
+	srq* que2 = SRQ_NEXT(process->prb_sessions);
+	while (que2 != &process->prb_sessions)
+	{
 		SES session = (SES) ((UCHAR *) que2 - OFFSET(SES, ses_sessions));
 #ifdef MULTI_THREAD
 		session->ses_flags |= SES_delivering;
@@ -975,6 +884,7 @@ static AST_TYPE deliver(void* arg)
 		const SLONG que2_offset = SRQ_REL_PTR(que2);
 		for (bool flag = true; flag;) {
 			flag = false;
+			srq* event_srq;
 			SRQ_LOOP(session->ses_requests, event_srq) {
 				EVT_REQ request = (EVT_REQ) ((UCHAR *) event_srq - OFFSET(EVT_REQ, req_requests));
 				if (request_completed(request)) {
@@ -982,14 +892,26 @@ static AST_TYPE deliver(void* arg)
 					process = (PRB) SRQ_ABS_PTR(EVENT_process_offset);
 					session = (SES) SRQ_ABS_PTR(session_offset);
 					que2 = (srq *) SRQ_ABS_PTR(que2_offset);
+#ifdef MULTI_THREAD
+					flag = !(session->ses_flags & SES_purge);
+#else
 					flag = true;
+#endif
 					break;
 				}
 			}
 		}
 #ifdef MULTI_THREAD
 		session->ses_flags &= ~SES_delivering;
+		if (session->ses_flags & SES_purge)
+		{
+			que2 = SRQ_NEXT((*que2));
+			delete_session(SRQ_REL_PTR(session));
+			break;
+		}
+		else
 #endif
+			que2 = SRQ_NEXT((*que2));
 	}
 
 	release();
@@ -1094,13 +1016,11 @@ static void exit_handler(void* arg)
 
 	ISC_STATUS_ARRAY local_status;
 
-#ifndef SERVER
 #ifdef SOLARIS_MT
 	ISC_unmap_object(local_status, &EVENT_data, (UCHAR**)&EVENT_process,
 					 sizeof(prb));
 #endif
 	ISC_unmap_file(local_status, &EVENT_data, 0);
-#endif
 
 	EVENT_header = NULL;
 }
@@ -1118,10 +1038,9 @@ static EVNT find_event(USHORT length, const TEXT* string, EVNT parent)
  *	Lookup an event.
  *
  **************************************/
-	srq* event_srq;
-
 	SRQ_PTR parent_offset = (parent) ? SRQ_REL_PTR(parent) : 0;
 
+	srq* event_srq;
 	SRQ_LOOP(EVENT_header->evh_events, event_srq) {
 		EVNT event = (EVNT) ((UCHAR *) event_srq - OFFSET(EVNT, evnt_events));
 		if (event->evnt_parent == parent_offset &&
@@ -1233,7 +1152,9 @@ static void init(void* arg, SH_MEM shmem_data, bool initialize)
 	int mutex_state;
 
 #if defined(WIN_NT)
-	if (mutex_state = ISC_mutex_init(MUTEX, EVENT_FILE))
+	char buffer[MAXPATHLEN];
+	gds__prefix_lock(buffer, EVENT_FILE);
+	if (mutex_state = ISC_mutex_init(MUTEX, buffer))
 		mutex_bugcheck("mutex init", mutex_state);
 #endif
 
@@ -1247,11 +1168,9 @@ static void init(void* arg, SH_MEM shmem_data, bool initialize)
 	SRQ_INIT(EVENT_header->evh_processes);
 	SRQ_INIT(EVENT_header->evh_events);
 
-#ifndef SERVER
 #if !defined(WIN_NT)
 	if (mutex_state = ISC_mutex_init(MUTEX, shmem_data->sh_mem_mutex_arg))
 		mutex_bugcheck("mutex init", mutex_state);
-#endif
 #endif
 
 	FRB free = (FRB) ((UCHAR*) EVENT_header + sizeof(evh));
@@ -1353,11 +1272,7 @@ static void post_process(PRB process)
 
 	process->prb_flags &= ~PRB_wakeup;
 	process->prb_flags |= PRB_pending;
-	release();
-
-#ifdef SERVER
-	deliver();
-#else
+//	release();
 
 #ifdef MULTI_THREAD
 	ISC_event_post(process->prb_event);
@@ -1369,8 +1284,7 @@ static void post_process(PRB process)
 #endif
 
 #endif
-	acquire();
-#endif
+//	acquire();
 }
 
 
@@ -1440,14 +1354,12 @@ static void release(void)
 #ifdef MULTI_THREAD
 	EVENT_header->evh_current_process = 0;
 	if (mutex_state = ISC_mutex_unlock(MUTEX))
-		mutex_bugcheck("mutex lock", mutex_state);
+		mutex_bugcheck("mutex unlock", mutex_state);
 #else
 	if (!--acquire_count) {
 		EVENT_header->evh_current_process = 0;
-#ifndef SERVER
 		if (mutex_state = ISC_mutex_unlock(MUTEX))
-			mutex_bugcheck("mutex lock", mutex_state);
-#endif
+			mutex_bugcheck("mutex unlock", mutex_state);
 #ifdef UNIX
 		if (EVENT_process_offset) {
 			PRB process = (PRB) SRQ_ABS_PTR(EVENT_process_offset);
@@ -1592,40 +1504,28 @@ static THREAD_ENTRY_DECLARE watcher_thread(THREAD_ENTRY_PARAM)
 
 		if (process->prb_flags & PRB_exiting) {
 			process->prb_flags &= ~PRB_exiting;
-			ISC_event_post(process->prb_event);
 			release();
 			break;
 		}
-#ifdef WIN_NT
-		if (process->prb_flags & PRB_remap) {
-			process->prb_flags |= PRB_remap_over;
-			process->prb_flags &= ~PRB_remap;
-			release();
-			while (true) {
-				Sleep(3);
-				acquire();
-				process = (PRB) SRQ_ABS_PTR(EVENT_process_offset);
-				release();
-				if (!(process->prb_flags & PRB_remap_over))
-					break;
-			}
-			acquire();
-			process = (PRB) SRQ_ABS_PTR(EVENT_process_offset);
-		}
-#endif
 
 		SLONG value = ISC_event_clear(process->prb_event);
 		release();
 		deliver(NULL);
 		acquire();
 		process = (PRB) SRQ_ABS_PTR(EVENT_process_offset);
-		release();
+		
+		// we going to wait on event, allocated in shared memory
+		// forbid shared memory remapping until waiting
+		process->prb_flags |= PRB_dont_remap;
 #ifdef SOLARIS_MT
 		event_t* events = EVENT_process->prb_event;
 #else
 		event_t* events = process->prb_event;
 #endif
+		release();
+
 		ISC_event_wait(1, &events, &value, 0, 0, 0);
+		process->prb_flags &= ~PRB_dont_remap;
 	}
 	return 0;
 }
