@@ -22,6 +22,7 @@
  *
  *  All Rights Reserved.
  *  Contributor(s): ______________________________________.
+ *  Adriano dos Santos Fernandes
  */
 
 #include "firebird.h"
@@ -34,7 +35,6 @@
 #include "../jrd/jrd.h"
 #include "../jrd/tra.h"
 #include "../jrd/dsc.h"
-#include "../jrd/y_handle.h"
 #include "../jrd/thd.h"
 #include "../jrd/err_proto.h"
 #include "../jrd/mov_proto.h"
@@ -44,8 +44,9 @@
 #include "../jrd/thread_proto.h"
 #define	WHY_NO_API 
 #include "../jrd/why_proto.h"
-
+#include "../jrd/y_handle.h"
 #include "../jrd/align.h"
+
 #include "../jrd/execute_statement.h"
 #include "../common/classes/GenericMap.h"
 #include "../common/classes/init.h"
@@ -53,11 +54,12 @@
 using namespace Jrd;
 using namespace Firebird;
 
-WHY_DBB GetWhyAttachment(ISC_STATUS* status,
-						  Attachment* jrd_attachment_handle);
+YValve::Attachment* GetWhyAttachment(ISC_STATUS*, Attachment*);
 
-static const SSHORT sqlType[] =
- {
+namespace {
+
+const SSHORT sqlType[] =
+{
 /* dtype_unknown	*/ -1,
 /* dtype_text		*/ SQL_TEXT,
 /* dtype_cstring	*/ -1,
@@ -78,10 +80,32 @@ static const SSHORT sqlType[] =
 /* dtype_blob		*/ SQL_BLOB,
 /* dtype_array		*/ SQL_ARRAY,		// Not supported for a while
 /* dtype_int64		*/ SQL_INT64
- };
+};
 
 static InitInstance<GenericMap<Pair<NonPooled<SSHORT, UCHAR> > > > sqlTypeToDscType;
 
+void startCallback(thread_db* tdbb)
+{
+#if (defined DEV_BUILD && !defined MULTI_THREAD)
+	tdbb->getDatabase()->dbb_flags |= DBB_exec_statement;
+#endif
+	if (tdbb->getTransaction())
+		tdbb->getTransaction()->tra_callback_count++;
+	THREAD_EXIT();
+}
+
+void finishCallback(thread_db* tdbb)
+{
+	THREAD_ENTER();
+#if (defined DEV_BUILD && !defined MULTI_THREAD)
+	tdbb->getDatabase()->dbb_flags &= ~DBB_exec_statement;
+#endif
+	if (tdbb->getTransaction())
+		tdbb->getTransaction()->tra_callback_count--;
+}
+
+} // anonymous namespace
+
 void ExecuteStatement::Open(thread_db* tdbb, jrd_nod* sql, SSHORT nVars, bool SingleTon)
 {
 	SET_TDBB(tdbb);
@@ -94,7 +118,7 @@ void ExecuteStatement::Open(thread_db* tdbb, jrd_nod* sql, SSHORT nVars, bool Si
 			sqlTypeToDscType().put(sqlType[i], static_cast<UCHAR>(i));
 	}
 
-	if (tdbb->tdbb_transaction->tra_callback_count >= MAX_CALLBACKS) {
+	if (tdbb->getTransaction()->tra_callback_count >= MAX_CALLBACKS) {
 		tdbb->tdbb_status_vector[0] = isc_arg_gds;
 		tdbb->tdbb_status_vector[1] = isc_exec_sql_max_call_exceeded;
 		tdbb->tdbb_status_vector[2] = isc_arg_end;
@@ -106,33 +130,25 @@ void ExecuteStatement::Open(thread_db* tdbb, jrd_nod* sql, SSHORT nVars, bool Si
 	Buffer = 0;
 	SingleMode = SingleTon;
 
-	fb_assert(tdbb->tdbb_transaction->tra_pool);
+	fb_assert(tdbb->getTransaction()->tra_pool);
 	Firebird::string SqlText;
-	getString(SqlText, EVL_expr(tdbb, sql), tdbb->tdbb_request);
+	getString(tdbb, SqlText, EVL_expr(tdbb, sql), tdbb->getRequest());
 	memcpy(StartOfSqlOperator, SqlText.c_str(), sizeof(StartOfSqlOperator) - 1);
 	StartOfSqlOperator[sizeof(StartOfSqlOperator) - 1] = 0;
 
-	WHY_DBB temp_dbb = GetWhyAttachment(tdbb->tdbb_status_vector,
-								  tdbb->tdbb_attachment);
+	YValve::Attachment* temp_dbb = GetWhyAttachment(tdbb->tdbb_status_vector,
+		tdbb->getAttachment());
 	if (!temp_dbb)
 		ERR_punt();
 
 	Attachment = temp_dbb->public_handle;
 
-	WHY_TRA temp_tra = WHY_alloc_handle(temp_dbb->implementation, HANDLE_transaction);
-	if (!temp_tra)
-		ERR_post(isc_virmemexh, 0);
-	Transaction = temp_tra->public_handle;
-	temp_tra->handle.h_tra = tdbb->tdbb_transaction;
-	temp_tra->parent = temp_dbb;
+	YValve::Transaction* temp_tra = new YValve::Transaction(tdbb->getTransaction(), &Transaction, temp_dbb);
 
 	Statement = 0;
 	Sqlda = MakeSqlda(tdbb, nVars ? nVars : 1);
-    Sqlda->sqln = nVars;
-    Sqlda->version = 1;
-
-	tdbb->tdbb_transaction->tra_callback_count++;
-	THREAD_EXIT();
+	Sqlda->sqln = nVars;
+	Sqlda->version = 1;
 
 	// For normal diagnostic
 	const int max_diag_len = 50;
@@ -141,15 +157,17 @@ void ExecuteStatement::Open(thread_db* tdbb, jrd_nod* sql, SSHORT nVars, bool Si
 	ISC_STATUS_ARRAY local;
 	ISC_STATUS *status = local;
 	memset(local, 0, sizeof(local));
+
 #	define Chk(x) if ((x) != 0) goto err_handler
+	startCallback(tdbb);
 
 	Chk(isc_dsql_allocate_statement(status, &Attachment, &Statement));
 
 	Chk(isc_dsql_prepare(status, &Transaction, &Statement,
 		SqlText.length(), SqlText.c_str(), SQL_DIALECT_CURRENT, Sqlda));
-    if (! Sqlda->sqld) {  // Non-select statement - reject for a while
+	if (! Sqlda->sqld) {  // Non-select statement - reject for a while
 		/*Chk(isc_dsql_execute(status, &Transaction,
-               &Statement, SQLDA_VERSION1, 0)); */
+			&Statement, SQLDA_VERSION1, 0)); */
 		Chk(isc_dsql_free_statement(status, &Statement, DSQL_drop));
 		Statement = 0;
 		status[0] = isc_arg_gds;
@@ -162,27 +180,27 @@ void ExecuteStatement::Open(thread_db* tdbb, jrd_nod* sql, SSHORT nVars, bool Si
 	else {
 		Chk(ReMakeSqlda(status, tdbb));
 		Chk(isc_dsql_describe(status, &Statement,
-                        SQLDA_VERSION1, Sqlda));
+				SQLDA_VERSION1, Sqlda));
 		Buffer = 0;		// Buffer is used in ParseSqlda
 						// First dummy parse - to define buffer size
-		Buffer = FB_NEW(*tdbb->tdbb_transaction->tra_pool) 
+		Buffer = FB_NEW(*tdbb->getTransaction()->tra_pool) 
 			SCHAR[XSQLDA_LENGTH(ParseSqlda())];
 		ParseSqlda();
 		Chk(isc_dsql_execute(status, &Transaction,
-                &Statement, SQLDA_VERSION1, 0));
+				&Statement, SQLDA_VERSION1, 0));
 	}
 
 #	undef Chk
 err_handler:
-	THREAD_ENTER();
-	tdbb->tdbb_transaction->tra_callback_count--;
+	finishCallback(tdbb);
+
 	if (status[0] == 1 && status[1]) {
 		memcpy(tdbb->tdbb_status_vector, status, sizeof(local));
 		Firebird::status_exception::raise(tdbb->tdbb_status_vector);
 	}
 }
 
-bool ExecuteStatement::Fetch(thread_db* tdbb, jrd_nod** JrdVar)
+bool ExecuteStatement::Fetch(thread_db* tdbb, jrd_nod** jrdVar)
 {
 	// If already bugged - we should never get here
 	fb_assert(! (tdbb->tdbb_status_vector[0] == 1 && 
@@ -194,30 +212,29 @@ bool ExecuteStatement::Fetch(thread_db* tdbb, jrd_nod** JrdVar)
 	ISC_STATUS *status = local;
 	memset(local, 0, sizeof(local));
 	status = local;
-	tdbb->tdbb_transaction->tra_callback_count++;
-	THREAD_EXIT();
-    if (isc_dsql_fetch(status, &Statement,
-                SQLDA_VERSION1, Sqlda) == 100)
+
+	startCallback(tdbb);
+	if (isc_dsql_fetch(status, &Statement,
+			SQLDA_VERSION1, Sqlda) == 100)
 	{
 		isc_dsql_free_statement(status, &Statement, DSQL_drop);
-		THREAD_ENTER();
-		tdbb->tdbb_transaction->tra_callback_count--;
+		finishCallback(tdbb);
+
 		Statement = 0;
 		return false;
-    }
-	THREAD_ENTER();
-	tdbb->tdbb_transaction->tra_callback_count--;
+	}
+	finishCallback(tdbb);
+
 	if (status[0] == 1 && status[1]) {
 		memcpy(tdbb->tdbb_status_vector, status, sizeof(local));
 		Firebird::status_exception::raise(tdbb->tdbb_status_vector);
 	}
 
 	const XSQLVAR *var = Sqlda->sqlvar;
-	for (int i = 0; i < Sqlda->sqld; i++, var++, JrdVar++) {
-		dsc* d = EVL_assign_to(tdbb, *JrdVar);
-		if (d->dsc_dtype >= FB_NELEM(sqlType))
+	for (int i = 0; i < Sqlda->sqld; i++, var++) {
+		dsc* d = EVL_assign_to(tdbb, jrdVar[i]);
+		if (d->dsc_dtype >= FB_NELEM(sqlType) || sqlType[d->dsc_dtype] < 0)
 		{
-rec_err:
 			tdbb->tdbb_status_vector[0] = isc_arg_gds;
 			tdbb->tdbb_status_vector[1] = isc_exec_sql_invalid_var;
 			tdbb->tdbb_status_vector[2] = isc_arg_number;
@@ -229,12 +246,9 @@ rec_err:
 			Firebird::status_exception::raise(tdbb->tdbb_status_vector);
 		}
 
-		if (sqlType[d->dsc_dtype] < 0)
-			goto rec_err;
-
 		// build the src descriptor
 		dsc src;
-		memset(&src, 0, sizeof(src));
+		src.clear();
 		sqlTypeToDscType().get((var->sqltype & ~1), src.dsc_dtype);
 		src.dsc_length = var->sqllen;
 		src.dsc_scale = var->sqlscale;
@@ -242,28 +256,25 @@ rec_err:
 		src.dsc_address = (UCHAR*) var->sqldata;
 
 		if ((var->sqltype & ~1) == SQL_VARYING)
-		{
 			src.dsc_length += sizeof(SSHORT);
-		}
 
 		// and assign to the target
-		EXE_assignment(tdbb, *JrdVar, &src, (var->sqltype & 1) && (*var->sqlind < 0), NULL, NULL);
-    }
+		EXE_assignment(tdbb, jrdVar[i], &src, (var->sqltype & 1) && (*var->sqlind < 0), NULL, NULL);
+	}
 
 	if (SingleMode) {
-		tdbb->tdbb_transaction->tra_callback_count++;
-		THREAD_EXIT();
+		startCallback(tdbb);
 		if (isc_dsql_fetch(status, &Statement,
-                SQLDA_VERSION1, Sqlda) == 100)
+				SQLDA_VERSION1, Sqlda) == 100)
 		{
 			isc_dsql_free_statement(status, &Statement, DSQL_drop);
-			THREAD_ENTER();
-			tdbb->tdbb_transaction->tra_callback_count--;
+			finishCallback(tdbb);
+
 			Statement = 0;
 			return false;
 		}
-		THREAD_ENTER();
-		tdbb->tdbb_transaction->tra_callback_count--;
+		finishCallback(tdbb);
+
 		if (! (status[0] == 1 && status[1])) {
 			status[0] = isc_arg_gds;
 			status[1] = isc_sing_select_err;
@@ -272,90 +283,92 @@ rec_err:
 		memcpy(tdbb->tdbb_status_vector, status, sizeof(local));
 		Firebird::status_exception::raise(tdbb->tdbb_status_vector);
 	}
-    return true;
+	return true;
 }
 
 void ExecuteStatement::Close(thread_db* tdbb)
 {
 	if (Statement) {
-		tdbb->tdbb_transaction->tra_callback_count++;
-		THREAD_EXIT();
 		// for a while don't check for errors while freeing statement
+		startCallback(tdbb);
 		isc_dsql_free_statement(0, &Statement, DSQL_drop);
-		THREAD_ENTER();
-		tdbb->tdbb_transaction->tra_callback_count--;
+		finishCallback(tdbb);
+		
 		Statement = 0;
 	}
-	char* p = reinterpret_cast<char*>(Sqlda);
-	delete[] p;
-	Sqlda = 0;
-	if (Transaction) {
-		THREAD_EXIT();
-		WHY_cleanup_transaction(WHY_translate_handle(Transaction));
-		THREAD_ENTER();
+
+	// if transaction is already missing, its pool is also deleted,
+	// i.e. impossible and no use deleting something from it
+	if (tdbb->getTransaction()) {
+		char* p = reinterpret_cast<char*>(Sqlda);
+		delete[] p;
+		delete[] Buffer;
 	}
-	WHY_free_handle(Transaction);
-	Transaction = 0;
-	delete[] Buffer;
+	Sqlda = 0;
 	Buffer = 0;
+
+	if (Transaction) {
+		delete YValve::translate<YValve::Transaction>(&Transaction);
+		Transaction = 0;
+	}
 }
 
 XSQLDA* ExecuteStatement::MakeSqlda(thread_db* tdbb, short n)
 {
 	return (XSQLDA *)
-		(FB_NEW(*tdbb->tdbb_transaction->tra_pool) char[XSQLDA_LENGTH(n)]);
+		(FB_NEW(*tdbb->getTransaction()->tra_pool) char[XSQLDA_LENGTH(n)]);
 }
 
-ISC_STATUS ExecuteStatement::ReMakeSqlda(ISC_STATUS* svector, thread_db* tdbb)
+ISC_STATUS ExecuteStatement::ReMakeSqlda(ISC_STATUS *vector, thread_db* tdbb)
 {
 	if (Sqlda->sqln != Sqlda->sqld) {
-		svector[0] = isc_arg_gds;
-		svector[1] = isc_wronumarg;
-		svector[2] = isc_arg_end;
+		vector[0] = isc_arg_gds;
+		vector[1] = isc_wronumarg;
+		vector[2] = isc_arg_end;
 	}
-	return svector[1];
+	return vector[1];
 }
 
 ULONG ExecuteStatement::ParseSqlda(void)
 {
-    ULONG offset = 0;
-    int i = 0;
-	UCHAR dscType;
-    for (XSQLVAR* var = Sqlda->sqlvar;
-                        i < Sqlda->sqld; var++, i++)
+	ULONG offset = 0;
+	int i = 0;
+
+	for (XSQLVAR* var = Sqlda->sqlvar; i < Sqlda->sqld; var++, i++)
 	{
-        USHORT length = var->sqllen;
-        const int type = var->sqltype & (~1);
+		USHORT length = var->sqllen;
+		const int type = var->sqltype & (~1);
+		UCHAR dscType;
 		sqlTypeToDscType().get(type, dscType);
-        if (type == SQL_VARYING)
-            length += sizeof (SSHORT);
+		if (type == SQL_VARYING)
+			length += sizeof (SSHORT);
 
 		const USHORT align = type_alignments[dscType];
 		if (align) {
 			offset = FB_ALIGN(offset, align);
 		}
-        var->sqldata = &Buffer[offset];
-        offset += length;
+		var->sqldata = &Buffer[offset];
+		offset += length;
 
 		offset = FB_ALIGN(offset, sizeof(short));
-        var->sqlind = (short*) (&Buffer[offset]);
-        offset += sizeof (short);
-    }
-    return offset;
+		var->sqlind = (short*) (&Buffer[offset]);
+		offset += sizeof (short);
+	}
+
+	return offset;
 }
 
-void ExecuteStatement::getString(Firebird::string& s, const dsc* d, const jrd_req* r)
+void ExecuteStatement::getString(thread_db* tdbb, Firebird::string& s, const dsc* d, const jrd_req* r)
 {
-	char buffer[BUFFER_LARGE + sizeof(vary)];
-	vary* v = reinterpret_cast<vary*>(buffer);
-	v->vary_length = BUFFER_LARGE;
+	MoveBuffer buffer;
 
 	UCHAR* p = 0;
 	const SSHORT l = (d && !(r->req_flags & req_null)) ?
-		MOV_get_string(d, &p, v, BUFFER_LARGE) : 0; // !!! How call Msgs ?
+		MOV_make_string2(tdbb, d, d->getTextType(), &p, buffer) : 0; // !!! How call Msgs ?
 	if (! p) {
-		ERR_post(isc_exec_sql_invalid_arg, 0);
+		ERR_post(isc_exec_sql_invalid_arg, isc_arg_end);
 	}
 
 	s.assign((const char*)p, l);
 }
+
