@@ -116,11 +116,12 @@ static const TEXT dtypes[][36] = {
 	"FLOAT",
 	"DOUBLE",
 	"D_FLOAT",
-	"",
-	"",
 	"DATE",
+	"TIME",
+	"TIMESTAMP",
 	"BLOB",
-	"ARRAY"
+	"ARRAY",
+	"BIGINT, scale %d"
 };
 
 
@@ -289,7 +290,7 @@ ISC_STATUS filter_format(USHORT action, BlobControl* control)
 		value = desc.dsc_length;
 	else if (desc.dsc_dtype == dtype_varying)
 		value = desc.dsc_length - sizeof(SSHORT);
-	else if (desc.dsc_dtype > dtype_array) {
+	else if (desc.dsc_dtype > dtype_int64) {
 		p = "data type %d unknown";
 		value = desc.dsc_dtype;
 	}
@@ -648,6 +649,7 @@ ISC_STATUS filter_transliterate_text(USHORT action, BlobControl* control)
  **************************************/
 	struct ctlaux {
 		CsConvert ctlaux_obj1;	/* Intl object that does tx for us */
+		USHORT ctlaux_init_action;	// isc_blob_filter_open or isc_blob_filter_create?
 		BYTE *ctlaux_buffer1;	/* Temporary buffer for transliteration */
 		BlobControl* ctlaux_subfilter;	/* For chaining transliterate filters */
 		ISC_STATUS ctlaux_source_blob_status;	/* marks when source is EOF, etc */
@@ -665,7 +667,6 @@ ISC_STATUS filter_transliterate_text(USHORT action, BlobControl* control)
 	
 	BlobControl* source;
 	ISC_STATUS status;
-	USHORT err_code;
 	ULONG err_position;
 	SSHORT source_cs, dest_cs;
 	SSHORT i;
@@ -694,65 +695,13 @@ ISC_STATUS filter_transliterate_text(USHORT action, BlobControl* control)
 #endif
 		control->ctl_data[0] = (IPTR) aux;
 
+		aux->ctlaux_init_action = action;
 		aux->ctlaux_source_blob_status = FB_SUCCESS;
 		aux->ctlaux_buffer1_unused = 0;
 		aux->ctlaux_expansion_factor = EXP_SCALE * 1;
 
 		SET_TDBB(tdbb);
 		aux->ctlaux_obj1 = INTL_convert_lookup(tdbb, dest_cs, source_cs);
-		if (aux->ctlaux_obj1 == NULL) {
-			/* Do the convert the hard way, via Unicode.
-			   In this case, we become the filter from Unicode to
-			   the destination format.  And we setup as our source
-			   a filter from <source> to <Unicode>.
-			   recursively using the same routine. */
-
-			if (action == isc_blob_filter_open) {
-				aux->ctlaux_obj1 =
-					INTL_convert_lookup(tdbb, dest_cs, CS_UTF16);
-			}
-			else {
-				aux->ctlaux_obj1 =
-					INTL_convert_lookup(tdbb, CS_UTF16, source_cs);
-			}
-
-			if (aux->ctlaux_obj1 == NULL) {
-				control->ctl_status[0] = isc_arg_gds;
-				control->ctl_status[1] = isc_text_subtype;
-				control->ctl_status[2] = isc_arg_number;
-				control->ctl_status[3] = dest_cs;
-				control->ctl_status[4] = isc_arg_end;
-				return isc_text_subtype;
-			}
-
-			// ISC_STATUS to pointer!
-			aux->ctlaux_subfilter =
-				(BlobControl*) caller(isc_blob_filter_alloc, control, 0, NULL, NULL);
-
-			/* This is freed in BLF_close_filter */
-
-			*(aux->ctlaux_subfilter) = *control;
-
-			control->ctl_handle = aux->ctlaux_subfilter;
-			control->ctl_source = filter_transliterate_text;
-			source = control->ctl_handle;
-
-			if (action == isc_blob_filter_open) {
-				control->ctl_from_sub_type = CS_UTF16;
-				aux->ctlaux_subfilter->ctl_to_sub_type = CS_UTF16;
-			}
-			else {
-				control->ctl_to_sub_type = CS_UTF16;
-				aux->ctlaux_subfilter->ctl_from_sub_type = CS_UTF16;
-			}
-
-
-			/* Now that the new filter has been inserted, tell it to open */
-
-			status = caller(action, control, 0, NULL, NULL);
-			if (status)
-				return status;
-		}
 
 		if (action == isc_blob_filter_open) {
 			// hvlad: avoid possible overflow of USHORT variables converting long
@@ -760,7 +709,7 @@ ISC_STATUS filter_transliterate_text(USHORT action, BlobControl* control)
 			// Also buffer must contain integer number of utf16 characters as we
 			// do transliteration via utf16 character set
 			// (see assert at start of UnicodeUtil::utf16ToUtf8 for example)
-			const SLONG max_seg = aux->ctlaux_obj1.convertLength(source->ctl_max_segment);
+			const ULONG max_seg = aux->ctlaux_obj1.convertLength(source->ctl_max_segment);
 			control->ctl_max_segment = MIN(MAX_USHORT - sizeof(ULONG) + 1, max_seg);
 
 			if (source->ctl_max_segment && control->ctl_max_segment)
@@ -818,16 +767,26 @@ ISC_STATUS filter_transliterate_text(USHORT action, BlobControl* control)
 		return FB_SUCCESS;
 
 	case isc_blob_filter_close:
+		// ASF: Raise error at close functions is something bad,
+		// but I know no better thing to do here.
+		if (aux->ctlaux_init_action == isc_blob_filter_create &&
+			aux->ctlaux_buffer1_unused != 0)
+		{
+			return isc_transliteration_failed;
+		}
+
 		if (aux && aux->ctlaux_buffer1) {
 			gds__free(aux->ctlaux_buffer1);
 			aux->ctlaux_buffer1 = NULL;
 			aux->ctlaux_buffer1_len = 0;
 		}
+
 		if (aux) {
 			gds__free(aux);
 			control->ctl_data[0] = 0;
 			aux = NULL;
 		}
+
 		return FB_SUCCESS;
 
 	case isc_blob_filter_get_segment:
@@ -835,47 +794,81 @@ ISC_STATUS filter_transliterate_text(USHORT action, BlobControl* control)
 		break;
 
 	case isc_blob_filter_put_segment:
-		/* Now convert from the input buffer into the temporary buffer */
+	{
+		USHORT len = control->ctl_buffer_length;
+		Firebird::HalfStaticArray<BYTE, BUFFER_MEDIUM> buffer;
+		BYTE* p;
 
-		/* How much space do we need to convert? */
-		result_length = aux->ctlaux_obj1.convertLength(control->ctl_buffer_length);
+		while (len || aux->ctlaux_buffer1_unused)
+		{
+			if (aux->ctlaux_buffer1_unused != 0)
+			{
+				p = buffer.getBuffer(aux->ctlaux_buffer1_unused + len);
+				memcpy(p, aux->ctlaux_buffer1, aux->ctlaux_buffer1_unused);
+				memcpy(p + aux->ctlaux_buffer1_unused, control->ctl_buffer, len);
+				len += aux->ctlaux_buffer1_unused;
+			}
+			else
+				p = control->ctl_buffer;
+	
+			/* Now convert from the input buffer into the temporary buffer */
+	
+			/* How much space do we need to convert? */
+			const ULONG cvt_len = aux->ctlaux_obj1.convertLength(len);
+			result_length = MIN(MAX_USHORT, cvt_len);
+	
+			/* Allocate a new buffer if we don't have enough */
+			if (result_length > aux->ctlaux_buffer1_len) {
+				gds__free(aux->ctlaux_buffer1);
+				aux->ctlaux_buffer1_len = result_length;
+				aux->ctlaux_buffer1 = (BYTE *) gds__alloc((SLONG) result_length);
+				/* FREE: above & isc_blob_filter_close in this routine */
+				if (!aux->ctlaux_buffer1)	/* NOMEM: */
+					return isc_virmemexh;
+			}
+	
+			/* convert the text */
+	
+			try
+			{
+				err_position = len;
+				result_length = aux->ctlaux_obj1.convert(len, p,
+					aux->ctlaux_buffer1_len, aux->ctlaux_buffer1, &err_position);
+			}
+			catch (const Firebird::status_exception&)
+			{
+				return isc_transliteration_failed;
+			}
+	
+			if (len > 0 && err_position == 0)
+				return isc_transliteration_failed;
+	
+			/* hand the text off to the next stage of the filter */
+	
+			status = caller(isc_blob_filter_put_segment, control, result_length,
+							aux->ctlaux_buffer1, NULL);
+	
+			if (status)
+				return status;
+	
+			aux->ctlaux_buffer1_unused = len - err_position;
+	
+			if (aux->ctlaux_buffer1_unused != 0)
+				memmove(aux->ctlaux_buffer1, p + err_position, aux->ctlaux_buffer1_unused);
+	
+			/* update local control variables for segment length */
+	
+			if (result_length > control->ctl_max_segment)
+				control->ctl_max_segment = result_length;
+	
+			control->ctl_total_length += result_length;
+			control->ctl_number_segments++;
 
-		/* Allocate a new buffer if we don't have enough */
-		if (result_length > aux->ctlaux_buffer1_len) {
-			gds__free(aux->ctlaux_buffer1);
-			aux->ctlaux_buffer1_len = result_length;
-			aux->ctlaux_buffer1 = (BYTE *) gds__alloc((SLONG) result_length);
-			/* FREE: above & isc_blob_filter_close in this routine */
-			if (!aux->ctlaux_buffer1)	/* NOMEM: */
-				return isc_virmemexh;
+			len = 0;
 		}
 
-		/* convert the text */
-
-		result_length = aux->ctlaux_obj1.convert(control->ctl_buffer_length, control->ctl_buffer,
-			aux->ctlaux_buffer1_len, aux->ctlaux_buffer1,
-			&err_code, &err_position);
-
-		if (err_code)
-			return isc_transliteration_failed;
-
-		/* hand the text off to the next stage of the filter */
-
-		status = caller(isc_blob_filter_put_segment, control, result_length,
-						aux->ctlaux_buffer1, NULL);
-
-		if (status)
-			return status;
-
-		/* update local control variables for segment length */
-
-		if (result_length > control->ctl_max_segment)
-			control->ctl_max_segment = result_length;
-
-		control->ctl_total_length += result_length;
-		control->ctl_number_segments++;
-
 		return FB_SUCCESS;
+	}
 
 	case isc_blob_filter_seek:
 		return isc_uns_ext;
@@ -927,7 +920,7 @@ ISC_STATUS filter_transliterate_text(USHORT action, BlobControl* control)
 		USHORT bytes_read_from_source = 0;
 		status = caller(isc_blob_filter_get_segment,
 						control,
-						(USHORT) (aux->ctlaux_buffer1_len - length),
+						(USHORT) MIN((aux->ctlaux_buffer1_len - length), control->ctl_buffer_length),
 						aux->ctlaux_buffer1 + length,
 						&bytes_read_from_source);
 		if (status == isc_segment)	/* source has more segment bytes */
@@ -946,14 +939,19 @@ ISC_STATUS filter_transliterate_text(USHORT action, BlobControl* control)
 
 /* Now convert from the temporary buffer into the destination buffer */
 
-	result_length = aux->ctlaux_obj1.convert(length, aux->ctlaux_buffer1,
-					control->ctl_buffer_length, control->ctl_buffer,
-					&err_code, &err_position);
-
-	if (err_code == CS_CONVERT_ERROR)
+	try
+	{
+		err_position = length;
+		result_length = aux->ctlaux_obj1.convert(length, aux->ctlaux_buffer1,
+						control->ctl_buffer_length, control->ctl_buffer,
+						&err_position);
+	}
+	catch (const Firebird::status_exception&)
+	{
 		return isc_transliteration_failed;
+	}
 
-	if (err_code == CS_BAD_INPUT) {
+	if (err_position < length) {
 		/* Bad input *might* be due to input buffer truncation in the middle
 		   of a character, so shuffle bytes, add some more data, and try again.
 		   If we already tried that then it's really some bad input */
@@ -962,7 +960,7 @@ ISC_STATUS filter_transliterate_text(USHORT action, BlobControl* control)
 			return isc_transliteration_failed;
 	}
 	
-	const USHORT unused_len = (err_code == 0) ?	0 : length - err_position;
+	const USHORT unused_len = (err_position >= length) ? 0 : length - err_position;
 	control->ctl_segment_length = result_length;
 	if (unused_len) {
 		memcpy(aux->ctlaux_buffer1, aux->ctlaux_buffer1 + err_position, unused_len);
