@@ -59,14 +59,13 @@
 
 #include "../jrd/flu_proto.h"
 #include "../jrd/gds_proto.h"
-#include "../jrd/dls_proto.h"
 #include "../jrd/err_proto.h"
 
 #include "gen/iberror.h"
 
 #include <string.h>
 
-#if (defined SOLARIS || defined SCO_EV || defined LINUX || defined AIX_PPC || defined SINIXZ || defined FREEBSD || defined NETBSD || HPUX)
+#if (defined SOLARIS || defined SCO_EV || defined LINUX || defined AIX_PPC || defined FREEBSD || defined NETBSD || HPUX)
 #define DYNAMIC_SHARED_LIBRARIES
 #endif
 
@@ -389,7 +388,7 @@ FPTR_INT ISC-lookup-entrypoint(TEXT* module,
 		}
 
 	REPLACE THIS COMPILER ERROR WITH CODE TO VERIFY THAT THE MODULE IS FOUND
-		EITHER IN $INTERBASE:UDF, or $INTERBASE:intl,
+		EITHER IN $FIREBIRD:UDF, or $FIREBIRD:intl,
 		OR IN ONE OF THE DIRECTORIES NAMED IN EXTERNAL_FUNCTION_DIRECTORY
 		LINES IN ISC_CONFIG.for (p = absolute_module; *p && *p != ' '; p++);
 
