@@ -52,6 +52,7 @@
 #include "../common/classes/locks.h"
 #include "../common/classes/timestamp.h"
 #include "../common/classes/init.h"
+#include "../common/classes/TempFile.h"
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -104,39 +105,6 @@
 
 #include "../common/config/config.h"
 
-// Turn on V4 mutex protection for gds__alloc/free 
-// 03/23/2003 BRS. Those defines don't do anything, V4_ macros are defined in thd.h 
-// but this file is included before this definition and so the macros are defined as empty
-// Those definitions are commented to allow V4_ macros to be inside V4_THREADING ifdefs
-// The include chain is
-// gdsassert.h -> gds_proto.h -> fil.h -> thd.h
-//
-//#ifdef WIN_NT
-//#define V4_THREADING
-//#endif
-//
-//#ifdef SOLARIS_MT
-//#define V4_THREADING
-//#endif
-//
-//#ifdef SUPERSERVER
-//#define V4_THREADING			// RFM: 9/22/2000 fix from Inprise tree,
-//								// Inprise bug 114840
-//#endif
-//
-// The following ifdef was added to build thread safe gds shared
-//  library on linux platform. It seems the gdslib works now (20020220)
-//  with thread enabled applications. Anyway, more tests should be 
-//  done as I don't have deep knowledge of the interbase/firebird 
-//  engine and this change may imply side effect I haven't known 
-//  about yet. Tomas Nejedlik (tomas@nejedlik.cz)
-//
-//#if ((defined(LINUX) || defined(FREEBSD)) && defined(SUPERCLIENT))
-//#define V4_THREADING
-//#endif
-//
-
-
 #ifdef SUPERSERVER
 static const TEXT gdslogid[] = " (Server)";
 #else
@@ -170,19 +138,16 @@ static const char* FB_PID_FILE = "fb_%d";
 #undef leave
 #endif /* WIN_NT */
 
-// Number of times to try to generate new name for temporary file
-const int MAX_TMPFILE_TRIES		= 256;
-
-static char ib_prefix_val[MAXPATHLEN];
-static char ib_prefix_lock_val[MAXPATHLEN];
-static char ib_prefix_msg_val[MAXPATHLEN];
+static char fb_prefix_val[MAXPATHLEN];
+static char fb_prefix_lock_val[MAXPATHLEN];
+static char fb_prefix_msg_val[MAXPATHLEN];
 static char fbTempDir[MAXPATHLEN];
 #ifdef EMBEDDED
 static char fbEmbeddedRoot[MAXPATHLEN];
 #endif
-static char *ib_prefix = 0;
-static char *ib_prefix_lock = 0;
-static char *ib_prefix_msg = 0;
+static char *fb_prefix = 0;
+static char *fb_prefix_lock = 0;
+static char *fb_prefix_msg = 0;
 static void gdsPrefixInit();
 
 #include "gen/msgs.h"
@@ -197,6 +162,9 @@ const SLONG GENERIC_SQLCODE		= -999;
 #include "../jrd/jrd.h"
 #include "../common/utils_proto.h"
 
+#include "../common/classes/SafeArg.h"
+#include "../common/classes/MsgPrint.h"
+
 using Firebird::TimeStamp;
 
 // This structure is used to parse the firebird.msg file.
@@ -274,7 +242,7 @@ static CLEAN	cleanup_handlers = NULL;
 static gds_msg* global_default_msg = NULL;
 static bool initialized = false;
 
-void* API_ROUTINE gds__alloc_debug(SLONG size_request,
+VoidPtr API_ROUTINE gds__alloc_debug(SLONG size_request,
                                    const TEXT* filename,
                                    ULONG lineno)
 {
@@ -389,7 +357,8 @@ static const UCHAR
 	dcl_cursor[] = { op_word, op_line, op_verb, op_indent, op_word, op_line, op_args, 0},
 	cursor_stmt[] = { op_cursor_stmt, 0 },
 	strlength[] = { op_byte, op_line, op_verb, 0},
-	trim[] = { op_byte, op_byte, op_line, op_verb, 0};
+	trim[] = { op_byte, op_byte, op_line, op_verb, 0},
+	modify2[] = { op_byte, op_byte, op_line, op_verb, op_verb, 0};
 
 #include "../jrd/blp.h"
 
@@ -455,14 +424,11 @@ ISC_STATUS API_ROUTINE gds__decode(ISC_STATUS code, USHORT* fac, USHORT*
  **************************************/
 
 	if (!code)
-	{
 		return FB_SUCCESS;
-	}
-	else if ((code & ISC_MASK) != ISC_MASK)
-	{
-		/* not an ISC error message */
+		
+	// not an ISC error message
+	if ((code & ISC_MASK) != ISC_MASK)
 		return code;
-	}
 
 	*fac = GET_FACILITY(code);
 	*code_class = GET_CLASS(code);
@@ -836,6 +802,8 @@ static SLONG safe_interpret(char* const s, const size_t bufsize,
 	const TEXT** arg = args;
 	const char* const* const argend = arg + FB_NELEM(args);
 
+	MsgFormat::SafeArg safe;
+
 	// Parse and collect any arguments that may be present
 	
 	TEXT* p = 0;
@@ -854,8 +822,15 @@ static SLONG safe_interpret(char* const s, const size_t bufsize,
 		switch (x)
 		{
 		case isc_arg_string:
+			*arg++ = (TEXT*) *v;
+			safe << (TEXT*) *v;
+			++v;
+			continue;
+
 		case isc_arg_number:
-			*arg++ = (TEXT*) *v++;
+			*arg++ = (TEXT*) *v;
+			safe << *v;
+			++v;
 			continue;
 
 		case isc_arg_cstring:
@@ -880,14 +855,18 @@ static SLONG safe_interpret(char* const s, const size_t bufsize,
 				// the loop changes "len".
 				temp_len -= len;
 				*arg++ = p;
+				safe << p;
 				// We'll silently truncate the parameter to our available space.
 				while (--len) // CVC: Decrement first to make room for the null terminator.
 					*p++ = *q++;
 					
 				*p++ = 0;
 			}
-			else // No space at all, pass the empty string.
+			else  // No space at all, pass the empty string.
+			{
 				*arg++ = "";
+				safe << "";
+			}
 				
 			continue;
 
@@ -904,22 +883,24 @@ static SLONG safe_interpret(char* const s, const size_t bufsize,
 	case isc_arg_warning:
 	case isc_arg_gds:
 		{
+			while (arg < args + 5) // could be argend, but we only use up to args[4]
+			    *arg++ = 0;
+			    
 			USHORT fac = 0, dummy_class = 0;
 			const ISC_STATUS decoded = gds__decode(code, &fac, &dummy_class);
-			if (gds__msg_format(0, fac, (USHORT) decoded,
-								bufsize, s, args[0], args[1], args[2], args[3],
-								args[4]) < 0)
+			if (fb_msg_format(0, fac, (USHORT) decoded, bufsize, s, safe) < 0)
 			{
 				bool found = false;
 
 				for (int i = 0; messages[i].code_number; ++i) {
 					if (code == messages[i].code_number) {
-						if (legacy)
+						if (legacy && strchr(messages[i].code_text, '%'))
+						{
 							sprintf(s, messages[i].code_text,
 									args[0], args[1], args[2], args[3], args[4]);
+						}
 						else
-							fb_utils::snprintf(s, bufsize, messages[i].code_text,
-									args[0], args[1], args[2], args[3], args[4]);
+							MsgFormat::MsgPrint(s, bufsize, messages[i].code_text, safe);
 						found = true;
 						break;
 					}
@@ -981,14 +962,14 @@ static SLONG safe_interpret(char* const s, const size_t bufsize,
 
 #ifdef WIN_NT
 	case isc_arg_win32:
-		if (!FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM,
+		if (!FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_MAX_WIDTH_MASK,
 										 NULL,
 										 code,
 										 GetUserDefaultLangID(),
 										 s,
 										 bufsize,
-						                 NULL)
-		  && !FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM,
+										 NULL)
+		  && !FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_MAX_WIDTH_MASK,
 										 NULL,
 										 code,
 										 0, // TMN: Fallback to system known language
@@ -1137,7 +1118,9 @@ void API_ROUTINE gds__trace_raw(const char* text, unsigned int length)
 	// Note: signal-safe code
 	gds__prefix(name, LOGFILE);
 	int file = open(name, O_CREAT | O_APPEND | O_WRONLY, 0660);
-	if (file == -1) return;
+	if (file == -1)
+		return;
+		
 	write(file, text, length);
 	close(file);
 #endif
@@ -1175,32 +1158,41 @@ void API_ROUTINE gds__trace(const TEXT * text)
 
 	char buffer[1024]; // 1K should be enough for the trace message
 	char* p = buffer;
-	gds__ulstr(p, today.tm_year + 1900, 4, '0'); p += 4;
+	gds__ulstr(p, today.tm_year + 1900, 4, '0');
+	p += 4;
 	*p++ = '-';
-	gds__ulstr(p, today.tm_mon, 2, '0'); p += 2;
+	gds__ulstr(p, today.tm_mon, 2, '0');
+	p += 2;
 	*p++ = '-';
-	gds__ulstr(p, today.tm_mday, 2, '0'); p += 2;
+	gds__ulstr(p, today.tm_mday, 2, '0');
+	p += 2;
 	*p++ = 'T';
-	gds__ulstr(p, today.tm_hour, 2, '0'); p += 2;
+	gds__ulstr(p, today.tm_hour, 2, '0');
+	p += 2;
 	*p++ = ':';
-	gds__ulstr(p, today.tm_min, 2, '0'); p += 2;
+	gds__ulstr(p, today.tm_min, 2, '0');
+	p += 2;
 	*p++ = ':';
-	gds__ulstr(p, today.tm_sec, 2, '0'); p += 2;
+	gds__ulstr(p, today.tm_sec, 2, '0');
+	p += 2;
 	*p++ = ' ';
-	gds__ulstr(p, 
+	ULONG apid =
 #ifdef WIN_NT
 #ifdef SUPERSERVER
-			     GetCurrentThreadId(),
+			     GetCurrentThreadId();
 #else
-				 GetCurrentProcessId(),				   
+				 GetCurrentProcessId();
 #endif
 #else
-				 getpid(),
+				 getpid();
 #endif
-				 5, ' '); p += 5;
+	gds__ulstr(p, apid, 5, ' ');
+	p += 5;
 	*p++ = ' ';
-	strcpy(p, text); p += strlen(p);
-	strcat(p, "\n"); p += strlen(p);
+	strcpy(p, text);
+	p += strlen(p);
+	strcat(p, "\n");
+	p += strlen(p);
 	gds__trace_raw(buffer, p - buffer);
 }
 
@@ -1427,7 +1419,7 @@ SSHORT API_ROUTINE gds__msg_format(void*       handle,
 
 	size = (size < length) ? length : size;
 
-	TEXT* formatted = (TEXT *) gds__alloc((SLONG) size);
+	TEXT* formatted = (TEXT *) gds__alloc(size);
 
 	if (!formatted)				/* NOMEM: */
 		return -1;
@@ -1450,8 +1442,9 @@ SSHORT API_ROUTINE gds__msg_format(void*       handle,
 			s += "message text not found";
 		else if (n == -2) {
 			s += "message file ";
-			gds__prefix_msg(formatted, MSG_FILE);
-			s += formatted;
+			TEXT temp[MAXPATHLEN];
+			gds__prefix_msg(temp, MSG_FILE);
+			s += temp;
 			s += " not found";
 		}
 		else {
@@ -1470,7 +1463,7 @@ SSHORT API_ROUTINE gds__msg_format(void*       handle,
 	*buffer = 0;
 
 	gds__free(formatted);
-	return ((n > 0) ? l : -l);
+	return (n > 0 ? l : -l);
 }
 
 
@@ -1478,7 +1471,8 @@ SSHORT API_ROUTINE gds__msg_lookup(void* handle,
 								   USHORT facility,
 								   USHORT number,
 								   USHORT length,
-								   TEXT* buffer, USHORT* flags)
+								   TEXT* buffer,
+								   USHORT* flags)
 {
 /**************************************
  *
@@ -1517,6 +1511,13 @@ SSHORT API_ROUTINE gds__msg_lookup(void* handle,
 			if (fb_utils::readenv("LC_MESSAGES", p))
 			{
 				sanitize(p);
+				Firebird::string::size_type pos = p.find_last_of('/');
+				if (pos == Firebird::string::npos)
+				    pos = p.find_last_of('\\');
+				    
+				if (pos != Firebird::string::npos)
+				    p.erase(0, pos + 1);
+				    
 				fb_utils::snprintf(translated_msg_file,
 					sizeof(translated_msg_file), MSG_FILE_LANG, p.c_str());
 				gds__prefix_msg(msg_file, translated_msg_file);
@@ -1526,6 +1527,7 @@ SSHORT API_ROUTINE gds__msg_lookup(void* handle,
 			}
 			else
 				status = 1;
+				
 			if (status) {
 				/* Default to standard message file */
 
@@ -1582,7 +1584,7 @@ SSHORT API_ROUTINE gds__msg_lookup(void* handle,
 				status = -1;
 				break;
 			}
-			else if (leaf->msgrec_code == code) {
+			if (leaf->msgrec_code == code) {
 				/* We found the correct message, so return it to the user */
 				const USHORT n = MIN(length - 1, leaf->msgrec_length);
 				memcpy(buffer, leaf->msgrec_text, n);
@@ -1623,11 +1625,12 @@ int API_ROUTINE gds__msg_open(void** handle, const TEXT* filename)
 		close(n);
 		return -3;
 	}
-	// Trick to silence compiler when MSG_MINOR_VERSION == 0
-	const USHORT minor = MSG_MINOR_VERSION;
 
-	if (header.msghdr_major_version != MSG_MAJOR_VERSION ||
-		header.msghdr_minor_version < minor)
+	if (header.msghdr_major_version != MSG_MAJOR_VERSION
+#if FB_MSG_MINOR_VERSION > 0
+		|| header.msghdr_minor_version < MSG_MINOR_VERSION
+#endif
+		)
 	{
 		close(n);
 		return -4;
@@ -1652,15 +1655,13 @@ int API_ROUTINE gds__msg_open(void** handle, const TEXT* filename)
 	messageL->msg_levels = header.msghdr_levels;
 	messageL->msg_top_tree = header.msghdr_top_tree;
 
-
 	*handle = messageL;
 
 	return 0;
 }
 
 
-void API_ROUTINE gds__msg_put(
-							  void* handle,
+void API_ROUTINE gds__msg_put(void* handle,
 							  USHORT facility,
 							  USHORT number,
 							  const TEXT* arg1, const TEXT* arg2,
@@ -1676,6 +1677,7 @@ void API_ROUTINE gds__msg_put(
  * Functional description
  *	Lookup and format message.  Return as much of formatted string
  *	as fits in callers buffer.
+ *  This function will misbehave with the new format and we can't solve it.
  *
  **************************************/
 	TEXT formatted[BUFFER_MEDIUM];
@@ -1695,52 +1697,57 @@ SLONG API_ROUTINE gds__get_prefix(SSHORT arg_type, const TEXT* passed_string)
  **************************************
  *
  * Functional description
- *	Find appropriate InterBase command line arguments 
+ *	Find appropriate Firebird command line arguments
  *	for Interbase file prefix.
  *
- *      arg_type is 0 for $INTERBASE, 1 for $INTERBASE_LOCK 
- *      and 2 for $INTERBASE_MSG
+ *      arg_type is 0 for $FIREBIRD, 1 for $FIREBIRD_LOCK
+ *      and 2 for $FIREBIRD_MSG
  *
  *      Function returns 0 on success and -1 on failure
+ *		it has very strange name, but to keep API as is leave it
  **************************************/
 	int count = 0;
+	
+	if (! passed_string)
+		return -1;
 
-	if (arg_type < IB_PREFIX_TYPE || arg_type > IB_PREFIX_MSG_TYPE)
-		return ((SLONG) - 1);
+	Firebird::PathName prefix(passed_string);
+	prefix.erase(MAXPATHLEN);
+	for (size_t n = 0; n < prefix.length(); ++n)
+	{
+		switch (prefix[n])
+		{
+		case ' ':
+		case '\n':	// don't know exact reason - just keep
+		case '\r':	// old API call behavior
+			prefix.erase(n);
+			break;	// will also leave for() due to length change
+		}
+	}
+
+	if (arg_type == IB_PREFIX_TYPE)
+	{
+		// it's very important to do it BEFORE gdsPrefixInit()
+		Config::setRootDirectoryFromCommandLine(prefix);
+	}
 
 	gdsPrefixInit();
 
-	char* prefix_ptr;
 	switch (arg_type) {
 	case IB_PREFIX_TYPE:
-		prefix_ptr = ib_prefix = ib_prefix_val;
+		prefix.copyTo(fb_prefix_val, sizeof fb_prefix_val);
 		break;
 	case IB_PREFIX_LOCK_TYPE:
-		prefix_ptr = ib_prefix_lock = ib_prefix_lock_val;
+		prefix.copyTo(fb_prefix_lock_val, sizeof fb_prefix_lock_val);
 		break;
 	case IB_PREFIX_MSG_TYPE:
-		prefix_ptr = ib_prefix_msg = ib_prefix_msg_val;
+		prefix.copyTo(fb_prefix_msg_val, sizeof fb_prefix_msg_val);
 		break;
 	default:
-		return ((SLONG) - 1);
+		return -1;
 	}
-/* the command line argument was 'H' for interbase home */
-	while (*prefix_ptr++ = *passed_string++) {
-		/* if the next character is space, newline or carriage return OR
-		   number of characters exceeded */
-		if (*passed_string == ' ' || *passed_string == 10
-			|| *passed_string == 13 || (count == MAXPATHLEN))
-		{
-			*(prefix_ptr++) = '\0';
-			break;
-		}
-		count++;
-	}
-	if (!count) {
-		prefix_ptr = NULL;
-		return ((SLONG) - 1);
-	}
-	return ((SLONG) 0);
+
+	return 0;
 }
 
 
@@ -1761,7 +1768,7 @@ void API_ROUTINE gds__prefix(TEXT* resultString, const TEXT* file)
 
 	gdsPrefixInit();
 
-	strcpy(resultString, ib_prefix);	// safe - no BO
+	strcpy(resultString, fb_prefix);	// safe - no BO
 	safe_concat_path(resultString, file);
 }
 #endif /* !defined(VMS) */
@@ -1777,9 +1784,9 @@ void API_ROUTINE gds__prefix(TEXT* string, const TEXT* root)
  **************************************
  *
  * Functional description
- *	Find appropriate InterBase file prefix.
+ *	Find appropriate Firebird file prefix.
  *	Override conditional defines with
- *	the enviroment variable INTERBASE if it is set.
+ *	the enviroment variable FIREBIRD if it is set.
  *
  **************************************/
 	if (*root != '[') {
@@ -1787,7 +1794,7 @@ void API_ROUTINE gds__prefix(TEXT* string, const TEXT* root)
 		return;
 	}
 
-/* Check for the existence of an InterBase logical name.  If there is 
+/* Check for the existence of a Firebird logical name.  If there is
    one use it, otherwise use the system directories. */
 	TEXT temp[256];
 	if (ISC_expand_logical_once(ISC_LOGICAL, sizeof(ISC_LOGICAL) - 2, temp, sizeof(temp)))
@@ -1838,7 +1845,7 @@ void API_ROUTINE gds__prefix_lock(TEXT* string, const TEXT* root)
 	root = buf;
 #endif
 
-	strcpy(string, ib_prefix_lock);	// safe - no BO
+	strcpy(string, fb_prefix_lock);	// safe - no BO
 	safe_concat_path(string, root);
 }
 #endif
@@ -1912,7 +1919,7 @@ void API_ROUTINE gds__prefix_msg(TEXT* string, const TEXT* root)
 
 	gdsPrefixInit();
 
-	strcpy(string, ib_prefix_msg);	// safe - no BO
+	strcpy(string, fb_prefix_msg);	// safe - no BO
 	safe_concat_path(string, root);
 }
 #endif
@@ -1938,7 +1945,7 @@ void API_ROUTINE gds__prefix_msg(TEXT* string, const TEXT* root)
 	}
 
 
-/* Check for the existence of an InterBase logical name.  If there is
+/* Check for the existence of a Firebird logical name.  If there is
    one use it, otherwise use the system directories. */
 
 /* ISC_LOGICAL_MSG macro needs to be defined, check non VMS version of routine
@@ -2028,7 +2035,8 @@ USHORT API_ROUTINE gds__parse_bpb(USHORT bpb_length,
 
   /* SIGN ERROR */
 
-	return gds__parse_bpb2(bpb_length, bpb, (SSHORT*)source, (SSHORT*)target, NULL, NULL);
+	return gds__parse_bpb2(bpb_length, bpb, (SSHORT*)source, (SSHORT*)target,
+		NULL, NULL, NULL, NULL, NULL, NULL);
 }
 
 
@@ -2037,7 +2045,11 @@ USHORT API_ROUTINE gds__parse_bpb2(USHORT bpb_length,
 								   SSHORT* source,
 								   SSHORT* target,
 								   USHORT* source_interp,
-								   USHORT* target_interp)
+								   USHORT* target_interp,
+								   bool* source_type_specified,
+								   bool* source_interp_specified,
+								   bool* target_type_specified,
+								   bool* target_interp_specified)
 {
 /**************************************
  *
@@ -2059,6 +2071,14 @@ USHORT API_ROUTINE gds__parse_bpb2(USHORT bpb_length,
 		*source_interp = 0;
 	if (target_interp)
 		*target_interp = 0;
+	if (source_type_specified)
+		*source_type_specified = false;
+	if (source_interp_specified)
+		*source_interp_specified = false;
+	if (target_type_specified)
+		*target_type_specified = false;
+	if (target_interp_specified)
+		*target_interp_specified = false;
 
 	if (!bpb_length || !bpb)
 		return type;
@@ -2075,24 +2095,33 @@ USHORT API_ROUTINE gds__parse_bpb2(USHORT bpb_length,
 		switch (op) {
 		case isc_bpb_source_type:
 			*source = (USHORT) gds__vax_integer(p, length);
+			if (source_type_specified)
+				*source_type_specified = true;
 			break;
 
 		case isc_bpb_target_type:
 			*target = (USHORT) gds__vax_integer(p, length);
+			if (target_type_specified)
+				*target_type_specified = true;
 			break;
 
 		case isc_bpb_type:
-			type = (USHORT) gds__vax_integer(p, length);
+		case isc_bpb_storage:
+			type |= (USHORT) gds__vax_integer(p, length);
 			break;
 
 		case isc_bpb_source_interp:
 			if (source_interp)
 				*source_interp = (USHORT) gds__vax_integer(p, length);
+			if (source_interp_specified)
+				*source_interp_specified = true;
 			break;
 
 		case isc_bpb_target_interp:
 			if (target_interp)
 				*target_interp = (USHORT) gds__vax_integer(p, length);
+			if (target_interp_specified)
+				*target_interp_specified = true;
 			break;
 
 		default:
@@ -2195,7 +2224,7 @@ int API_ROUTINE gds__print_blr(
 	blr_print_line(control, (SSHORT) offset);
 
 	}	// try
-	catch (const std::exception&) {
+	catch (const Firebird::LongJump&) {
 		return -1;
 	}
 
@@ -2382,27 +2411,7 @@ void API_ROUTINE gds__sqlcode_s(const ISC_STATUS* status_vector, ULONG* sqlcode)
 }
 
 
-void API_ROUTINE gds__temp_dir(TEXT* buffer)
-{
-/**************************************
- *
- *      g d s _ _ t e m p _ d i r
- *
- **************************************
- *
- * Functional description
- *      Return temporary directory.
- *
- **************************************/
-	buffer[0] = 0;
-
-	gdsPrefixInit();
-
-	strcpy(buffer, fbTempDir);	// safe - no BO
-}
-
-
-void* API_ROUTINE gds__temp_file(
+VoidPtr API_ROUTINE gds__temp_file(
 					 BOOLEAN stdio_flag, const TEXT* string,
 					 TEXT* expanded_string, TEXT* dir, BOOLEAN unlink_flag)
 {
@@ -2424,88 +2433,34 @@ void* API_ROUTINE gds__temp_file(
  *      via introducing two functions with different return types.
  *
  **************************************/
-	TEXT temp_dir[MAXPATHLEN];
+	try {
 
-	const TEXT* directory = dir;
-	if (!directory) {
-		gds__temp_dir(temp_dir);
-		directory = temp_dir;
-	}
-	if (strlen(directory) >= MAXPATHLEN - strlen(string) - strlen(TEMP_PATTERN) - 2)
-		return (void *)-1;
+	// This legacy wrapper cannot process these parameters.
+	// Fortunately, utilities never pass non-default values.
+	fb_assert(!dir && !unlink_flag);
 
-	void* result;
-
-#ifdef WIN_NT
-	/* These are the characters used in temporary filenames.  */
-	static const char letters[] = "abcdefghijklmnopqrstuvwxyz0123456789";
-
-	_timeb t;
-	_ftime(&t);
-	__int64 randomness = t.time;
-	randomness *= 1000;
-	randomness += t.millitm;
-	for (int tryCount = 0; tryCount < MAX_TMPFILE_TRIES; tryCount++) {
-		char file_name[MAXPATHLEN];
-		strcpy(file_name, directory);
-		if (file_name[strlen(file_name) - 1] != '\\')
-			strcat(file_name, "\\");
-		strcat(file_name, string);
-		char suffix[] = TEMP_PATTERN;
-		__int64 temp = randomness;
-		for (size_t i = 0; i < sizeof(suffix) - 1; i++) {
-			suffix[i] = letters[temp % (sizeof(letters) - 1)];
-			temp /= sizeof(letters) - 1;
-		}
-		strcat(file_name, suffix);
-		if (expanded_string)
-			strcpy(expanded_string, file_name);
-		result = (void*)_sopen(file_name, _O_CREAT | _O_TRUNC | _O_RDWR | 
-			_O_BINARY | _O_SHORT_LIVED | _O_NOINHERIT | _O_EXCL |
-			(expanded_string && !unlink_flag ? 0 : _O_TEMPORARY),
-			_SH_DENYRW, _S_IREAD | _S_IWRITE);
-		if ((int)result != -1 || (errno != EACCES && errno != EEXIST))
-			break;
-		randomness++;
-	}
-	if ((int)result == -1) return result;
-	if (stdio_flag) {
-		if (!(result = fdopen((int) result, "w+b")))
-			return (void *)-1;
-	}
-#else
-	TEXT file_name[MAXPATHLEN];
-	strcpy(file_name, directory);
-	if (file_name[strlen(file_name) - 1] != '/')
-		strcat(file_name, "/");
-	strcat(file_name, string);
-	strcat(file_name, TEMP_PATTERN);
-	
-#ifdef HAVE_MKSTEMP
-	result = (void *)(IPTR)mkstemp(file_name);
-#else
-	if (mktemp(file_name) == (char *)0)
-		return (void *)-1;
-
-	do {
-		result = (void *)open(file_name, O_RDWR | O_EXCL | O_CREAT);
-	} while (result == (void *)-1 && errno == EINTR);
-#endif
-	if (result == (void *)-1)
-		return result;
-
-	if (stdio_flag)
-		if (!(result = fdopen((int)(IPTR)result, "w+")))
-			return (void *)-1;
+	Firebird::PathName filename = TempFile::create(string);
 
 	if (expanded_string)
-		strcpy(expanded_string, file_name);
+	{
+		strcpy(expanded_string, filename.c_str());
+	}
 
-	if (!expanded_string || unlink_flag)
-		unlink(file_name);
-#endif
+	if (stdio_flag)
+	{
+		FILE* result = fopen(filename.c_str(), "w+b");
+		return result ? result : (void*) (IPTR) (-1);
+	}
+	else
+	{
+		return (void*) (IPTR) open(filename.c_str(), O_RDWR | O_EXCL | O_TRUNC);
+	}
 
-	return result;
+	}
+	catch (const Firebird::Exception&)
+	{
+		return (void*) (IPTR) (-1);
+	}
 }
 
 
@@ -2622,7 +2577,7 @@ BOOLEAN API_ROUTINE gds__validate_lib_path(const TEXT* module,
  **************************************
  *
  * Functional description
- *	Find the InterBase external library path variable.
+ *	Find the Firebird external library path variable.
  *	Validate that the path to the library module name 
  *	in the path specified.  If the external lib path
  *	is not defined then accept any path, and return true.
@@ -2634,7 +2589,7 @@ BOOLEAN API_ROUTINE gds__validate_lib_path(const TEXT* module,
 	TEXT abs_module_path[MAXPATHLEN];
 	TEXT abs_module[MAXPATHLEN];
 
-/* Check for the existence of an InterBase logical name.  If there is 
+/* Check for the existence of a Firebird logical name.  If there is
    one use it, otherwise use the system directories. */
 
 	COMPILER ERROR ! BEFORE DOING A VMS POST PLEASE
@@ -2802,16 +2757,14 @@ void API_ROUTINE isc_sql_interprete(
  *
  *	NOTE: As of 21-APR-1999, sqlmessages HAVE arguments hence use
  *	      an empty string instead of NULLs.
- *	      
+ *
  **************************************/
-	const TEXT* str = "";
+	static const MsgFormat::SafeArg arg = MsgFormat::SafeArg() << "" << "" << "" << "" << "";
 
 	if (sqlcode < 0)
-		gds__msg_format(0, 13, (USHORT) (1000 + sqlcode), length, buffer,
-						str, str, str, str, str);
+		fb_msg_format(0, 13, (USHORT) (1000 + sqlcode), length, buffer, arg);
 	else
-		gds__msg_format(0, 14, sqlcode, length, buffer,
-						str, str, str, str, str);
+		fb_msg_format(0, 14, sqlcode, length, buffer, arg);
 }
 
 
@@ -2839,12 +2792,10 @@ int unlink(const SCHAR* file)
 			break;
 	}
 
-	if (!status)
-		return 0;
-	else if (status != RMS$_FNF)
-		return -1;
-	else
+	if (!status || status == RMS$_FNF)
 		return 0;
+
+	return -1;
 }
 #endif
 
@@ -2869,7 +2820,7 @@ static void blr_error(gds_ctl* control, const TEXT* string, ...)
 	va_end(args);
 	offset = 0;
 	blr_print_line(control, (SSHORT) offset);
-	Firebird::status_exception::raise();
+	Firebird::LongJump::raise();
 }
 
 
@@ -3024,6 +2975,14 @@ static void blr_print_cond(gds_ctl* control)
 			blr_print_char(control);
 		break;
 
+	case blr_exception_msg:
+		blr_format(control, "blr_exception_msg, ");
+		n = blr_print_byte(control);
+		while (--n >= 0)
+			blr_print_char(control);
+		blr_print_verb(control, 0);
+		break;
+
 	case blr_sql_code:
 		blr_format(control, "blr_sql_code, ");
 		blr_print_word(control);
@@ -3062,7 +3021,8 @@ static int blr_print_dtype(gds_ctl* control)
    jump table */
 	const TEXT* string;
 
-	switch (dtype) {
+	switch (dtype)
+	{
 	case blr_short:
 		string = "short";
 		length = 2;
@@ -3148,6 +3108,24 @@ static int blr_print_dtype(gds_ctl* control)
 		length = 8;
 		break;
 
+	case blr_domain_name:
+		string = "domain_name";
+		// Don't bother with this length.
+		// It will not be used for blr_domain_name.
+		length = 0;
+		break;
+
+	case blr_domain_name2:
+		string = "domain_name2";
+		// Don't bother with this length.
+		// It will not be used for blr_domain_name2.
+		length = 0;
+		break;
+
+	case blr_not_nullable:
+		string = "not_nullable";
+		break;
+
 	default:
 		blr_error(control, "*** invalid data type ***");
 		break;
@@ -3155,7 +3133,12 @@ static int blr_print_dtype(gds_ctl* control)
 
 	blr_format(control, "blr_%s, ", string);
 
-	switch (dtype) {
+	switch (dtype)
+	{
+	case blr_not_nullable:
+		length = blr_print_dtype(control);
+		break;
+
 	case blr_text:
 		length = blr_print_word(control);
 		break;
@@ -3164,6 +3147,10 @@ static int blr_print_dtype(gds_ctl* control)
 		length = blr_print_word(control) + 2;
 		break;
 
+	case blr_cstring:
+		length = blr_print_word(control);
+		break;
+
 	case blr_short:
 	case blr_long:
 	case blr_quad:
@@ -3191,10 +3178,20 @@ static int blr_print_dtype(gds_ctl* control)
 		blr_print_word(control);
 		break;
 
-	default:
-		if (dtype == blr_cstring)
-			length = blr_print_word(control);
-		break;
+	case blr_domain_name:
+	case blr_domain_name2:
+		{
+			// 0 = blr_domain_type_of; 1 = blr_domain_full
+			blr_print_byte(control);
+
+			for (UCHAR n = blr_print_byte(control); n > 0; --n)
+				blr_print_char(control);
+
+			if (dtype == blr_domain_name2)
+				blr_print_word(control);
+
+			break;
+		}
 	}
 
 	return length;
@@ -3292,6 +3289,7 @@ static void blr_print_verb(gds_ctl* control, SSHORT level)
 	SSHORT n;
 
 	while (*ops)
+	{
 		switch (*ops++) {
 		case op_verb:
 			blr_print_verb(control, level);
@@ -3440,6 +3438,7 @@ static void blr_print_verb(gds_ctl* control, SSHORT level)
 			fb_assert(false);
 			break;
 		}
+	}
 }
 
 
@@ -3519,13 +3518,6 @@ static void init(void)
 	if (initialized)
 		return;
 
-	/* V4_INIT; */
-	/* V4_GLOBAL_MUTEX_LOCK; */
-	if (initialized) {
-		/*  V4_GLOBAL_MUTEX_UNLOCK; */
-		return;
-	}
-
 #ifdef VMS
 	exit_description.exit_handler = cleanup;
 	exit_description.args = 1;
@@ -3564,17 +3556,15 @@ static void init(void)
 #endif /* SUPERSERVER */
 #endif /* UNIX */
 
-	initialized = true;
-
 	atexit(gds__cleanup);
 
+	initialized = true;
+
 	gdsPrefixInit();
 
 #ifndef REQUESTER
 	ISC_signal_init();
 #endif
-
-	/* V4_GLOBAL_MUTEX_UNLOCK; */
 }
 
 static void sanitize(Firebird::string& locale)
@@ -3627,7 +3617,7 @@ static void safe_concat_path(TEXT *resultString, const TEXT *appendString)
 }
 
 
-void gds__default_printer(void* arg, SSHORT offset, const TEXT* line)
+void FB_EXPORTED gds__default_printer(void* arg, SSHORT offset, const TEXT* line)
 {
 	printf("%4d %s\n", offset, line);
 }
@@ -3641,7 +3631,8 @@ void gds__trace_printer(void* arg, SSHORT offset, const TEXT* line)
 	gds__ulstr(p, offset, 4, ' ');
 	p += strlen(p);
 	*p++ = ' ';
-	strcpy(p, line); p += strlen(p);
+	strcpy(p, line);
+	p += strlen(p);
 	*p++ = '\n';
 	*p = 0;
 	gds__trace_raw(buffer);
@@ -3649,7 +3640,7 @@ void gds__trace_printer(void* arg, SSHORT offset, const TEXT* line)
 
 #undef gds__alloc
 
-void* API_ROUTINE gds__alloc(SLONG size_request)
+VoidPtr API_ROUTINE gds__alloc(SLONG size_request)
 {
 	return getDefaultMemoryPool()->allocate_nothrow(size_request, 0
 #ifdef DEBUG_GDS_ALLOC
@@ -3664,7 +3655,7 @@ class InitPrefix
 public:
 	static void init()
 	{
-		// Get ib_prefix value from config file
+		// Get fb_prefix value from config file
 		// CVC: I put this protection block because we can't raise exceptions
 		// if exceptions are already raised due to the same reason:
 		// config file not found.
@@ -3681,8 +3672,8 @@ public:
 			if (!GetProgramFilesDir(prefix))
 				prefix = FB_PREFIX;
 		}
-		prefix.copyTo(ib_prefix_val, sizeof(ib_prefix_val));
-		ib_prefix = ib_prefix_val;
+		prefix.copyTo(fb_prefix_val, sizeof(fb_prefix_val));
+		fb_prefix = fb_prefix_val;
 
 		// Find appropiate temp directory
 		Firebird::PathName tempDir;
@@ -3724,8 +3715,8 @@ public:
 			lockPrefix = prefix;
 #endif
 		}
-		lockPrefix.copyTo(ib_prefix_lock_val, sizeof(ib_prefix_lock_val));
-		ib_prefix_lock = ib_prefix_lock_val;
+		lockPrefix.copyTo(fb_prefix_lock_val, sizeof(fb_prefix_lock_val));
+		fb_prefix_lock = fb_prefix_lock_val;
 
 		// Find appropriate Firebird message file prefix.
 		Firebird::PathName msgPrefix;
@@ -3733,8 +3724,8 @@ public:
 		{
 			msgPrefix = prefix;
 		}
-		msgPrefix.copyTo(ib_prefix_msg_val, sizeof(ib_prefix_msg_val));
-		ib_prefix_msg = ib_prefix_msg_val;
+		msgPrefix.copyTo(fb_prefix_msg_val, sizeof(fb_prefix_msg_val));
+		fb_prefix_msg = fb_prefix_msg_val;
 	}
 	static void cleanup()
 	{
@@ -3752,7 +3743,7 @@ static void gdsPrefixInit()
  **************************************
  *
  * Functional description
- *	Initialize all data in various ib_prefixes.
+ *	Initialize all data in various fb_prefixes.
  *	Calling it before any signal can be caught (from init())
  *	makes gds__prefix* family of functions signal-safe.
  *	In order not to break external API, call to gdsPrefixInit
