@@ -24,11 +24,11 @@
 #ifndef JRD_GRANT_PROTO_H
 #define JRD_GRANT_PROTO_H
 
+#include "../common/classes/array.h"
+
 class Jrd::DeferredWork;
-typedef Firebird::Array<UCHAR> UCharBuffer;
 
 bool GRANT_privileges(Jrd::thread_db*, SSHORT, Jrd::DeferredWork*, Jrd::jrd_tra*);
-void GRANT_realloc_acl(UCharBuffer&, UCHAR**, ULONG *);
 
 #endif // JRD_GRANT_PROTO_H
 
