@@ -28,7 +28,6 @@
 #define JRD_PWD_H
 
 #include "../jrd/ibase.h"
-#include "../jrd/smp_impl.h"
 #include "../jrd/thd.h"
 #include "../jrd/sha.h"
 #ifdef HAVE_STDLIB_H
@@ -97,7 +96,7 @@ private:
 	static const UCHAR PWD_REQUEST[256];
 	static const UCHAR TPB[4];
 
-	V4Mutex mutex;
+	Firebird::Mutex mutex;
 
 	ISC_STATUS_ARRAY status;
 
@@ -121,27 +120,16 @@ private:
 	}
 };
 
-namespace Jrd {
-
-class DelayFailedLogin : public Firebird::status_exception
+class DelayFailedLogin : public Firebird::Exception
 {
 private:
 	int seconds;
 public:
-	explicit DelayFailedLogin(int sec) throw()
-		: Firebird::status_exception(), seconds(sec) 
-	{
-		ISC_STATUS temp[] = {isc_arg_gds, 
-							isc_login,
-							isc_arg_end};
-		set_status(temp, false);
-	}
+	virtual ISC_STATUS stuff_exception(ISC_STATUS* const status_vector, Firebird::StringsBuffer* sb = NULL) const throw();
 	virtual const char* what() const throw() { return "Jrd::DelayFailedLogin"; }
 	static void raise(int sec);
+	DelayFailedLogin(int sec) throw() : Exception(), seconds(sec) { }
 	void sleep() const;
 };
 
-
-} // namespace Jrd
-
 #endif /* JRD_PWD_H */
