@@ -33,7 +33,6 @@
 #include "../jrd/lck.h"
 #include "gen/iberror.h"
 #include "../jrd/iberr.h"
-#include "../jrd/all_proto.h"
 #include "../jrd/err_proto.h"
 #include "../jrd/gds_proto.h"
 #include "../jrd/jrd_proto.h"
@@ -63,11 +62,11 @@
 
 using namespace Jrd;
 
-static void bug_lck(TEXT*);
+static void bug_lck(const TEXT*);
 #ifdef MULTI_THREAD
 static void check_lock(Lock*, USHORT);
 #endif
-static bool compatible(Lock*, Lock*, USHORT);
+static bool compatible(const Lock*, const Lock*, USHORT);
 static void enqueue(thread_db*, Lock*, USHORT, SSHORT);
 static int external_ast(void*);
 #ifdef MULTI_THREAD
@@ -75,11 +74,11 @@ static Lock* find_block(Lock*, USHORT);
 #endif
 static USHORT hash_func(const UCHAR*, USHORT);
 static void hash_allocate(Lock*);
-static Lock* hash_get_lock(Lock*, USHORT *, Lock***);
+static Lock* hash_get_lock(Lock*, USHORT*, Lock***);
 static void hash_insert_lock(Lock*);
 static bool hash_remove_lock(Lock*, Lock**);
 static void internal_ast(Lock*);
-static bool internal_compatible(Lock*, Lock*, USHORT);
+static bool internal_compatible(Lock*, const Lock*, USHORT);
 static void internal_dequeue(thread_db*, Lock*);
 static USHORT internal_downgrade(thread_db*, Lock*);
 static bool internal_enqueue(thread_db*, Lock*, USHORT, SSHORT, bool);
@@ -150,16 +149,15 @@ inline SLONG* LCK_OWNER_HANDLE_ATT(Attachment* attachment) {
 
 static const UCHAR compatibility[] = {
 
-/*				Shared	Prot	Shared	Prot
-		none	null	 Read	Read	Write	Write	Exclusive */
-
-/* none */ 1, 1, 1, 1, 1, 1, 1,
-/* null */ 1, 1, 1, 1, 1, 1, 1,
-/* SR */ 1, 1, 1, 1, 1, 1, 0,
-/* PR */ 1, 1, 1, 1, 0, 0, 0,
-/* SW */ 1, 1, 1, 0, 1, 0, 0,
-/* PW */ 1, 1, 1, 0, 0, 0, 0,
-/* EX */ 1, 1, 0, 0, 0, 0, 0
+/*							Shared	Prot	Shared	Prot
+			none	null	Read	Read	Write	Write	Exclusive */
+/* none */	1,		1,		1,		1,		1,		1,		1,
+/* null */	1,		1,		1,		1,		1,		1,		1,
+/* SR	*/	1,		1,		1,		1,		1,		1,		0,
+/* PR	*/	1,		1,		1,		1,		0,		0,		0,
+/* SW	*/	1,		1,		1,		0,		1,		0,		0,
+/* PW	*/	1,		1,		1,		0,		0,		0,		0,
+/* EX	*/	1,		1,		0,		0,		0,		0,		0
 };
 
 #define COMPATIBLE(st1, st2)	compatibility [st1 * LCK_max + st2]
@@ -211,10 +209,16 @@ JMB: As part of the c++ conversion I removed the check for Lock block type.
  Here is the line I removed from the macro:
 				 (l->blk_type == type_lck) && \
  */
-#define LCK_CHECK_LOCK(l)	(((l) != NULL) && \
-				 (l->lck_dbb != NULL) && \
-				 (l->lck_test_field == 666) && \
-				 (l->lck_id || (l->lck_physical == LCK_none)))
+#define LCK_CHECK_LOCK checkLock
+
+inline bool checkLock(const Lock* l)
+{
+	return (l != NULL &&
+			l->lck_length <= MAX_UCHAR &&
+			l->lck_dbb != NULL &&
+			l->lck_test_field == 666 &&
+			(l->lck_id || l->lck_physical == LCK_none));
+}
 
 /* The following check should be part of LCK_CHECK_LOCK, but it fails
    when the exclusive attachment to a database is changed to a shared
@@ -254,7 +258,7 @@ void LCK_assert(thread_db* tdbb, Lock* lock)
 	}
 
 	if (!LCK_lock(tdbb, lock, lock->lck_logical, LCK_WAIT))
-		BUGCHECK(159);			/* msg 159 cannot fb_assert logical lock */
+		BUGCHECK(159);			// msg 159 cannot assert logical lock
 
 	fb_assert(LCK_CHECK_LOCK(lock));
 }
@@ -279,7 +283,7 @@ bool LCK_convert(thread_db* tdbb, Lock* lock, USHORT level, SSHORT wait)
 	ISC_STATUS* status = tdbb->tdbb_status_vector;
 
 	Attachment *old_attachment = lock->lck_attachment;
-	set_lock_attachment(lock, tdbb->tdbb_attachment);
+	set_lock_attachment(lock, tdbb->getAttachment());
 
 	const bool result = CONVERT(tdbb, lock, level, wait, status);
 
@@ -326,7 +330,7 @@ int LCK_convert_non_blocking(thread_db* tdbb, Lock* lock, USHORT level, SSHORT w
 		return LCK_convert(tdbb, lock, level, wait);
 
 	Attachment* old_attachment = lock->lck_attachment;
-	set_lock_attachment(lock, tdbb->tdbb_attachment);
+	set_lock_attachment(lock, tdbb->getAttachment());
 
 /* Save context and checkout from the scheduler */
 
@@ -477,8 +481,8 @@ void LCK_fini(thread_db* tdbb, enum lck_owner_t owner_type)
 	SLONG* owner_handle_ptr = 0;
 
 	SET_TDBB(tdbb);
-	Database* dbb = tdbb->tdbb_database;
-	Attachment* attachment = tdbb->tdbb_attachment;
+	Database* dbb = tdbb->getDatabase();
+	Attachment* attachment = tdbb->getAttachment();
 
 	switch (owner_type) {
 	case LCK_OWNER_process:
@@ -517,8 +521,8 @@ SLONG LCK_get_owner_handle(thread_db* tdbb, enum lck_t lock_type)
 
 	SET_TDBB(tdbb);
 #ifdef SUPERSERVER
-	Database* dbb = tdbb->tdbb_database;
-	Attachment* attachment = tdbb->tdbb_attachment;
+	Database* dbb = tdbb->getDatabase();
+	Attachment* attachment = tdbb->getAttachment();
 #else
 	Database* dbb = NULL;
 	Attachment* attachment = NULL;
@@ -537,15 +541,21 @@ SLONG LCK_get_owner_handle(thread_db* tdbb, enum lck_t lock_type)
 	//case LCK_range_relation: // PC_ENGINE
 	case LCK_backup_alloc:
 	case LCK_backup_database:
+	case LCK_monitor:
+	case LCK_tt_exist:
 		return *LCK_OWNER_HANDLE_DBB(dbb);
 	case LCK_attachment:
+	case LCK_page_space:
 	case LCK_relation:
 	case LCK_file_extend:
 	case LCK_tra:
 	case LCK_sweep:
 	case LCK_record:
 	case LCK_update_shadow:
+	case LCK_dsql_cache:
 	case LCK_backup_end:
+	case LCK_cancel:
+	case LCK_btr_dont_gc:
 		return *LCK_OWNER_HANDLE_ATT(attachment);
 	default:
 		bug_lck("Invalid lock type in LCK_get_owner_handle ()");
@@ -554,6 +564,7 @@ SLONG LCK_get_owner_handle(thread_db* tdbb, enum lck_t lock_type)
 	}
 }
 
+
 SLONG LCK_get_owner_handle_by_type(thread_db* tdbb, lck_owner_t lck_owner_type)
 {
 /**********************************************************
@@ -573,15 +584,16 @@ SLONG LCK_get_owner_handle_by_type(thread_db* tdbb, lck_owner_t lck_owner_type)
 		case LCK_OWNER_process:
 			return *LCK_OWNER_HANDLE_PROCESS();
 		case LCK_OWNER_database:
-			return *LCK_OWNER_HANDLE_DBB(tdbb->tdbb_database);;
+			return *LCK_OWNER_HANDLE_DBB(tdbb->getDatabase());;
 		case LCK_OWNER_attachment:
-			return *LCK_OWNER_HANDLE_ATT(tdbb->tdbb_attachment);
+			return *LCK_OWNER_HANDLE_ATT(tdbb->getAttachment());
 		default:
 			bug_lck("Invalid lock owner type in LCK_get_owner_handle_by_type ()");
 			return 0;
 	}
 }
 
+
 bool LCK_set_owner_handle(Jrd::thread_db* tdbb, Jrd::Lock* lock, SLONG owner_handle)
 {
 /**************************************
@@ -605,6 +617,7 @@ bool LCK_set_owner_handle(Jrd::thread_db* tdbb, Jrd::Lock* lock, SLONG owner_han
 	return result;
 }
 
+
 void LCK_init(thread_db* tdbb, enum lck_owner_t owner_type)
 {
 /**************************************
@@ -621,8 +634,8 @@ void LCK_init(thread_db* tdbb, enum lck_owner_t owner_type)
 	SLONG* owner_handle_ptr = 0;
 
 	SET_TDBB(tdbb);
-	Database* dbb = tdbb->tdbb_database;
-	Attachment* attachment = tdbb->tdbb_attachment;
+	Database* dbb = tdbb->getDatabase();
+	Attachment* attachment = tdbb->getAttachment();
 
 	switch (owner_type) {
 	case LCK_OWNER_process:
@@ -673,7 +686,7 @@ int LCK_lock(thread_db* tdbb, Lock* lock, USHORT level, SSHORT wait)
 	Database* dbb = lock->lck_dbb;
 	ISC_STATUS* status = tdbb->tdbb_status_vector;
 	lock->lck_blocked_threads = NULL;
-    set_lock_attachment(lock, tdbb->tdbb_attachment);
+    set_lock_attachment(lock, tdbb->getAttachment());
 
 	ENQUEUE(tdbb, lock, level, wait);
 	fb_assert(LCK_CHECK_LOCK(lock));
@@ -725,7 +738,7 @@ int LCK_lock_non_blocking(thread_db* tdbb, Lock* lock, USHORT level, SSHORT wait
 	if (!wait || !gds__thread_enable(FALSE))
 		return LCK_lock(tdbb, lock, level, wait);
 
-	set_lock_attachment(lock, tdbb->tdbb_attachment);
+	set_lock_attachment(lock, tdbb->getAttachment());
 
 /* Make sure we're not about to wait for ourselves */
 
@@ -962,7 +975,7 @@ void LCK_write_data(Lock* lock, SLONG data)
 }
 
 
-static void bug_lck(TEXT* string)
+static void bug_lck(const TEXT* string)
 {
 /**************************************
  *
@@ -977,9 +990,9 @@ static void bug_lck(TEXT* string)
  **************************************/
 	TEXT s[128];
 
-	sprintf(s, "Fatal lock interface error: %s", string);
+	sprintf(s, "Fatal lock interface error: %.96s", string);
 	gds__log(s);
-	ERR_post(isc_db_corrupt, isc_arg_string, string, 0);
+	ERR_post(isc_db_corrupt, isc_arg_string, string, isc_arg_end);
 }
 
 
@@ -1007,7 +1020,7 @@ static void check_lock(Lock* lock, USHORT level)
 #endif
 
 
-static bool compatible(Lock* lock1, Lock* lock2, USHORT level2)
+static bool compatible(const Lock* lock1, const Lock* lock2, USHORT level2)
 {
 /**************************************
  *
@@ -1211,7 +1224,7 @@ static void hash_allocate(Lock* lock)
 }
 
 
-static Lock* hash_get_lock(Lock* lock, USHORT * hash_slot, Lock*** prior)
+static Lock* hash_get_lock(Lock* lock, USHORT* hash_slot, Lock*** prior)
 {
 /**************************************
  *
@@ -1410,7 +1423,7 @@ static void internal_ast(Lock* lock)
 
 
 
-static bool internal_compatible(Lock* match, Lock* lock, USHORT level)
+static bool internal_compatible(Lock* match, const Lock* lock, USHORT level)
 {
 /**************************************
  *
@@ -1441,7 +1454,6 @@ static bool internal_compatible(Lock* match, Lock* lock, USHORT level)
 
 /* now deliver the blocking asts, attempting to gain
    compatibility by getting everybody to downgrade */
-
 	internal_ast(match);
 
 /* make one more pass to see if all locks were downgraded */
