@@ -19,6 +19,7 @@
  *
  * All Rights Reserved.
  * Contributor(s): ______________________________________.
+ *
  * Revision 1.5  2000/12/08 16:18:21  fsg
  * Preliminary changes to get IB_BUILD_NO automatically
  * increased on commits.
@@ -71,7 +72,7 @@
 #define FB_PLATFORM	"HU"
 #endif /* HP10 */
 #ifdef HP11 
-#define FB_PLATFORM "HU"
+#define FB_PLATFORM     "HU"
 #endif /* HP11 */
 #endif
 
@@ -86,6 +87,9 @@
 #ifdef i386
 #define FB_PLATFORM     "SI"
 #endif /* i386 */
+#ifdef AMD64
+#define FB_PLATFORM	"SI"
+#endif
 #ifndef FB_PLATFORM
 #define FB_PLATFORM	"S3"
 #endif
@@ -119,10 +123,6 @@
 #define FB_PLATFORM     "LI"	/* Linux on Intel */
 #endif
 
-#ifdef SINIXZ
-#define FB_PLATFORM     "SZ"	/* SINIX for PC  */
-#endif
-
 #ifdef FREEBSD
 #define FB_PLATFORM     "FB"	/* FreeBSD/i386 */
 #endif
@@ -132,13 +132,13 @@
 #endif
 
 #ifdef DARWIN
-#ifdef i386
+#if defined(i386) || defined(__x86_64__)
 #define FB_PLATFORM	"UI"	/* Darwin/Intel */
 #endif
-#endif
-#if defined (DARWIN) && defined (__ppc__)
+#if defined(__ppc__) || defined(__ppc64__)
 #define FB_PLATFORM     "UP"	/* Darwin/PowerPC */
 #endif
+#endif  // DARWIN
 
 #ifndef FB_VERSION
 #define FB_VERSION      FB_PLATFORM "-" FB_BUILD_TYPE FB_MAJOR_VER "." FB_MINOR_VER "." FB_REV_NO "." FB_BUILD_NO " " FB_BUILD_SUFFIX
