@@ -31,6 +31,8 @@
 
 #include "../jrd/jrd_blks.h"
 #include "../include/fb_blk.h"
+#include "../jrd/thread_proto.h"
+#include "../common/classes/rwlock.h"
 
 namespace Jrd {
 
@@ -45,8 +47,8 @@ class jrd_file : public pool_alloc_rpt<SCHAR, type_fil>
 	USHORT fil_sequence;		/* Sequence number of file */
 	USHORT fil_fudge;			/* Fudge factor for page relocation */
 	int fil_desc;
-	int *fil_trace;				/* Trace file, if any */
-	Firebird::Mutex fil_mutex[1];
+	//int *fil_trace;				/* Trace file, if any */
+	Firebird::Mutex fil_mutex;
 	USHORT fil_flags;
 	USHORT fil_length;			/* Length of expanded file name */
 	SCHAR fil_string[1];		/* Expanded file name */
@@ -67,7 +69,7 @@ class jrd_file : public pool_alloc_rpt<SCHAR, type_fil>
 	USHORT fil_fudge;			/* Fudge factor for page relocation */
 	int fil_desc;
 	int fil_trace;				/* Trace file, if any */
-	Firebird::Mutex fil_mutex[1];
+	Firebird::Mutex fil_mutex;
 	USHORT fil_length;			/* Length of expanded file name */
 	USHORT fil_fid[3];			/* File id */
 	USHORT fil_did[3];			/* Directory id */
@@ -85,15 +87,20 @@ const int MAX_FILE_IO	= 32;			/* Maximum "allocated" overlapped I/O events */
 class jrd_file : public pool_alloc_rpt<SCHAR, type_fil>
 {
     public:
+
+	~jrd_file() {
+		delete fil_ext_lock;
+	}
+
 	jrd_file*	fil_next;		/* Next file in database */
 	ULONG fil_min_page;			/* Minimum page number in file */
 	ULONG fil_max_page;			/* Maximum page number in file */
 	USHORT fil_sequence;		/* Sequence number of file */
 	USHORT fil_fudge;			/* Fudge factor for page relocation */
-	SLONG fil_desc;
-	SLONG fil_force_write_desc;	/* Handle of force write open */
-	int *fil_trace;				/* Trace file, if any */
-	Firebird::Mutex fil_mutex[1];
+	HANDLE fil_desc;			// File descriptor
+	//int *fil_trace;				/* Trace file, if any */
+	Firebird::Mutex fil_mutex;
+	Firebird::RWLock* fil_ext_lock;	// file extend lock
 #ifdef SUPERSERVER_V2
 	void* fil_io_events[MAX_FILE_IO];	/* Overlapped I/O events */
 #endif
@@ -106,8 +113,8 @@ class jrd_file : public pool_alloc_rpt<SCHAR, type_fil>
 
 
 const USHORT FIL_force_write		= 1;
-const USHORT FIL_force_write_init	= 2;
-const USHORT FIL_readonly			= 4;
+const USHORT FIL_no_fs_cache		= 2;	// not using file system cache
+const USHORT FIL_readonly			= 4;	// file opened in readonly mode
 
 /* Physical IO trace events */
 
@@ -120,6 +127,7 @@ const SSHORT trace_close	= 6;
 
 // Physical I/O status block, used only in SS v2 for Win32
 
+#ifdef SUPERSERVER_V2
 struct phys_io_blk {
 	jrd_file* piob_file;				/* File being read/written */
 	SLONG piob_desc;			/* File descriptor */
@@ -134,6 +142,33 @@ struct phys_io_blk {
 const UCHAR PIOB_error		= 1;	/* I/O error occurred */
 const UCHAR PIOB_success	= 2;	/* I/O successfully completed */
 const UCHAR PIOB_pending	= 4;	/* Asynchronous I/O not yet completed */
+#endif
+
+// This class ensures that all actions in the appropriate scope
+// are performed outside the engine. Used in many PIO routines.
+
+class ThreadExit
+{
+public:
+	ThreadExit()
+	{
+#ifdef SUPERSERVER
+		THREAD_EXIT();
+#endif
+	}
+
+	~ThreadExit()
+	{
+#ifdef SUPERSERVER
+		THREAD_ENTER();
+#endif
+	}
+
+private:
+	// prohibited
+	ThreadExit(const ThreadExit&);
+	ThreadExit& operator=(const ThreadExit&);
+};
 
 } //namespace Jrd
 
