@@ -32,7 +32,6 @@
 #include "../jrd/tra.h"
 #include "../jrd/ods.h"
 #include "../jrd/btr.h"
-#include "../jrd/all_proto.h"
 #include "../jrd/err_proto.h"
 #include "../jrd/evl_proto.h"
 #include "../jrd/ext_proto.h"
@@ -84,7 +83,7 @@ void EXT_close(RecordSource* rsb)
 
 	jrd_rel* relation = rsb->rsb_relation;
 	ExternalFile* file = relation->rel_file;
-	jrd_req* request = tdbb->tdbb_request;
+	jrd_req* request = tdbb->getRequest();
 	record_param* rpb = &request->req_rpb[rsb->rsb_stream];
 
 	if ((rab.rab$w_isi = rpb->rpb_ext_isi) &&
@@ -96,7 +95,7 @@ void EXT_close(RecordSource* rsb)
 }
 
 
-void EXT_erase(record_param* rpb, int *transaction)
+void EXT_erase(record_param* rpb, jrd_tra *transaction)
 {
 /**************************************
  *
@@ -143,7 +142,7 @@ ExternalFile* EXT_file(jrd_rel* relation, TEXT* file_name, bid* description)
 	UCHAR index_buffer[MAX_KEYS * sizeof(index_desc)];
 
 	thread_db* tdbb = JRD_get_thread_data();
-	Database* dbb = tdbb->tdbb_database;
+	Database* dbb = tdbb->getDatabase();
 
 /* Allocate and fill out an external file block.  Get the
    external format and flesh it out using the internal
@@ -293,7 +292,7 @@ bool EXT_get(RecordSource* rsb)
  **************************************/
 	thread_db* tdbb = JRD_get_thread_data();
 
-	if (tdbb->tdbb_request->req_flags & req_abort)
+	if (tdbb->getRequest()->req_flags & req_abort)
 		return false;
 
 	switch (rsb->rsb_type) {
@@ -312,7 +311,7 @@ bool EXT_get(RecordSource* rsb)
 }
 
 
-void EXT_modify(record_param* old_rpb, record_param* new_rpb, int *transaction)
+void EXT_modify(record_param* old_rpb, record_param* new_rpb, jrd_tra *transaction)
 {
 /**************************************
  *
@@ -365,7 +364,7 @@ EXT_open(RecordSource* rsb)
 	thread_db* tdbb = JRD_get_thread_data();
 
 	jrd_rel* relation = rsb->rsb_relation;
-	jrd_req* request = tdbb->tdbb_request;
+	jrd_req* request = tdbb->getRequest();
 	record_param* rpb = &request->req_rpb[rsb->rsb_stream];
 
 	const Format* format;
@@ -503,7 +502,7 @@ void EXT_ready(jrd_rel* relation)
 }
 
 
-void EXT_store(record_param* rpb, int *transaction)
+void EXT_store(record_param* rpb, jrd_tra *transaction)
 {
 /**************************************
  *
@@ -546,7 +545,7 @@ void EXT_store(record_param* rpb, int *transaction)
 }
 
 
-void EXT_trans_commit(tra* transaction)
+void EXT_trans_commit(jrd_tra* transaction)
 {
 /**************************************
  *
@@ -561,7 +560,7 @@ void EXT_trans_commit(tra* transaction)
 }
 
 
-void EXT_trans_prepare(tra* transaction)
+void EXT_trans_prepare(jrd_tra* transaction)
 {
 /**************************************
  *
@@ -576,7 +575,7 @@ void EXT_trans_prepare(tra* transaction)
 }
 
 
-void EXT_trans_rollback(tra* transaction)
+void EXT_trans_rollback(jrd_tra* transaction)
 {
 /**************************************
  *
@@ -591,7 +590,7 @@ void EXT_trans_rollback(tra* transaction)
 }
 
 
-void EXT_trans_start(tra* transaction)
+void EXT_trans_start(jrd_tra* transaction)
 {
 /**************************************
  *
@@ -840,7 +839,7 @@ static bool get_dbkey(RecordSource* rsb)
 
 	jrd_rel* relation = rsb->rsb_relation;
 	ExternalFile* file = relation->rel_file;
-	jrd_req* request = tdbb->tdbb_request;
+	jrd_req* request = tdbb->getRequest();
 
 	irsb_ext* impure = (irsb_ext*) ((UCHAR *) request + rsb->rsb_impure);
 	record_param* rpb = &request->req_rpb[rsb->rsb_stream];
@@ -904,7 +903,7 @@ static bool get_indexed(RecordSource* rsb)
 	ExternalFile* file = relation->rel_file;
 	const Format* format = file->ext_format;
 
-	jrd_req* request = tdbb->tdbb_request;
+	jrd_req* request = tdbb->getRequest();
 	record_param* rpb = &request->req_rpb[rsb->rsb_stream];
 	irsb_ext* impure = (irsb_ext*) ((UCHAR *) request + rsb->rsb_impure);
 
@@ -1016,7 +1015,7 @@ static bool get_sequential(RecordSource* rsb)
 
 	jrd_rel* relation = rsb->rsb_relation;
 	ExternalFile* file = relation->rel_file;
-	jrd_req* request = tdbb->tdbb_request;
+	jrd_req* request = tdbb->getRequest();
 
 	irsb_ext* impure = (irsb_ext*) ((UCHAR *) request + rsb->rsb_impure);
 	record_param* rpb = &request->req_rpb[rsb->rsb_stream];
@@ -1118,7 +1117,7 @@ static open_indexed(RecordSource* rsb)
 
 	jrd_rel* relation = rsb->rsb_relation;
 	ExternalFile* file = relation->rel_file;
-	jrd_req* request = tdbb->tdbb_request;
+	jrd_req* request = tdbb->getRequest();
 	record_param* rpb = &request->req_rpb[rsb->rsb_stream];
 	jrd_nod* node = (jrd_nod*) rsb->rsb_arg[0];
 	IndexRetrieval* retrieval = (IndexRetrieval*) node->nod_arg[e_idx_retrieval];
@@ -1148,7 +1147,7 @@ static open_sequential(RecordSource* rsb)
 
 	jrd_rel* relation = rsb->rsb_relation;
 	ExternalFile* file = relation->rel_file;
-	jrd_req* request = tdbb->tdbb_request;
+	jrd_req* request = tdbb->getRequest();
 	record_param* rpb = &request->req_rpb[rsb->rsb_stream];
 
 	if (file->ext_file_type == FAB$C_IDX) {
