@@ -42,7 +42,6 @@
 #include "../jrd/lck.h"
 #include "../jrd/cch.h"
 #include "gen/iberror.h"
-#include "../jrd/all_proto.h"
 #include "../jrd/cch_proto.h"
 #include "../jrd/err_proto.h"
 #include "../jrd/isc_proto.h"
@@ -51,10 +50,46 @@
 #include "../jrd/lck_proto.h"
 #include "../jrd/mov_proto.h"
 #include "../jrd/os/pio_proto.h"
-#include "../jrd/thd.h"
+#include "../common/classes/init.h"
 
 #include <windows.h>
 
+namespace Jrd {
+
+class FileExtendLockGuard
+{
+public:
+	FileExtendLockGuard(Firebird::RWLock* lock, bool exclusive) :
+	  m_lock(lock), m_exclusive(exclusive) 
+	{
+		if (m_exclusive) {
+			fb_assert(m_lock);
+		}
+		if (m_lock) {
+			if (m_exclusive)
+				m_lock->beginWrite();
+			else
+				m_lock->beginRead();
+		}
+	}
+
+	~FileExtendLockGuard()
+	{
+		if (m_lock) {
+			if (m_exclusive)
+				m_lock->endWrite();
+			else
+				m_lock->endRead();
+		}
+	}
+
+private:
+	Firebird::RWLock*	m_lock;
+	bool				m_exclusive;
+};
+
+
+} // namespace Jrd
 
 using namespace Jrd;
 
@@ -69,8 +104,7 @@ const USHORT OS_CHICAGO		= 2;
 #ifdef SUPERSERVER_V2
 static void release_io_event(jrd_file*, OVERLAPPED*);
 #endif
-static ULONG get_number_of_pages(const jrd_file*, const USHORT);
-static bool	MaybeCloseFile(SLONG*);
+static bool	maybe_close_file(HANDLE&);
 static jrd_file* seek_file(jrd_file*, BufferDesc*, ISC_STATUS*, OVERLAPPED*, OVERLAPPED**);
 static jrd_file* setup_file(Database*, const Firebird::PathName&, HANDLE);
 static bool nt_error(TEXT*, const jrd_file*, ISC_STATUS, ISC_STATUS*);
@@ -92,6 +126,9 @@ static const DWORD g_dwExtraFlags = FILE_FLAG_RANDOM_ACCESS;
 #endif
 #endif
 
+static const DWORD g_dwShareTempFlags = FILE_SHARE_READ;
+static const DWORD g_dwExtraTempFlags = FILE_ATTRIBUTE_TEMPORARY |
+										FILE_FLAG_DELETE_ON_CLOSE;
 
 
 int PIO_add_file(Database* dbb, jrd_file* main_file, const Firebird::PathName& file_name, SLONG start)
@@ -108,7 +145,7 @@ int PIO_add_file(Database* dbb, jrd_file* main_file, const Firebird::PathName& f
  *	sequence of 0.
  *
  **************************************/
-	jrd_file* new_file = PIO_create(dbb, file_name, false, false);
+	jrd_file* new_file = PIO_create(dbb, file_name, false, false, false);
 	if (!new_file) {
 		return 0;
 	}
@@ -141,8 +178,7 @@ void PIO_close(jrd_file* main_file)
  **************************************/
 	for (jrd_file* file = main_file; file; file = file->fil_next)
 	{
-		if (MaybeCloseFile(&file->fil_desc) ||
-			MaybeCloseFile(&file->fil_force_write_desc))
+		if (maybe_close_file(file->fil_desc))
 		{
 #ifdef SUPERSERVER_V2
 			for (int i = 0; i < MAX_FILE_IO; i++)
@@ -159,29 +195,8 @@ void PIO_close(jrd_file* main_file)
 }
 
 
-int PIO_connection(const Firebird::PathName& file_name)
-{
-/**************************************
- *
- *	P I O _ c o n n e c t i o n
- *
- **************************************
- *
- * Functional description
- *	Analyze a file specification and determine whether a page/lock
- *	server is required and available.  If so, return a "connection"
- *	block.  If not, return NULL.
- *
- *	Note: The file name must have been expanded prior to this call.
- *
- **************************************/
-
-	return 0;
-}
-
-
-
-jrd_file* PIO_create(Database* dbb, const Firebird::PathName& string, bool overwrite, bool share_delete)
+jrd_file* PIO_create(Database* dbb, const Firebird::PathName& string,
+					 bool overwrite, bool temporary, bool share_delete)
 {
 /**************************************
  *
@@ -198,35 +213,41 @@ jrd_file* PIO_create(Database* dbb, const Firebird::PathName& string, bool overw
 	if (!ISC_is_WinNT())
 		share_delete = false;
 
+	DWORD dwShareMode = (temporary ? g_dwShareTempFlags : g_dwShareFlags);
+	if (share_delete)
+		dwShareMode |= FILE_SHARE_DELETE;
+
+	DWORD dwFlagsAndAttributes = FILE_ATTRIBUTE_NORMAL | g_dwExtraFlags;
+	if (temporary)
+		dwFlagsAndAttributes |= g_dwExtraTempFlags;
+
 	const HANDLE desc = CreateFile(file_name,
 					  GENERIC_READ | GENERIC_WRITE,
-					  g_dwShareFlags | (share_delete ? FILE_SHARE_DELETE : 0),
+					  dwShareMode,
 					  NULL,
 					  (overwrite ? CREATE_ALWAYS : CREATE_NEW),
-					  FILE_ATTRIBUTE_NORMAL |
-					  g_dwExtraFlags,
+					  dwFlagsAndAttributes,
 					  0);
 
 	if (desc == INVALID_HANDLE_VALUE)
 	{
 		ERR_post(isc_io_error,
 				 isc_arg_string, "CreateFile (create)",
-				 isc_arg_cstring,
-				 string.length(),
-				 ERR_cstring(string),
-				 isc_arg_gds,
-				 isc_io_create_err, isc_arg_win32, GetLastError(), 0);
+				 isc_arg_cstring, string.length(), ERR_cstring(string),
+				 isc_arg_gds, isc_io_create_err, isc_arg_win32, GetLastError(),
+				 0);
 	}
 
 /* File open succeeded.  Now expand the file name. */
-/* workspace is the exapnded name here */
+/* workspace is the expanded name here */
 
 	Firebird::PathName workspace(string);
 	ISC_expand_filename(workspace, false);
 	jrd_file *file;
 	try {
 		file = setup_file(dbb, workspace, desc);
-	} catch(const std::exception&) {
+	}
+	catch (const Firebird::Exception&) {
 		CloseHandle(desc);
 		throw;
 	}
@@ -235,7 +256,7 @@ jrd_file* PIO_create(Database* dbb, const Firebird::PathName& string, bool overw
 }
 
 
-int PIO_expand(const TEXT* file_name, USHORT file_length, TEXT* expanded_name,
+bool PIO_expand(const TEXT* file_name, USHORT file_length, TEXT* expanded_name,
 	size_t len_expanded)
 {
 /**************************************
@@ -255,6 +276,76 @@ int PIO_expand(const TEXT* file_name, USHORT file_length, TEXT* expanded_name,
 }
 
 
+void PIO_extend(jrd_file* main_file, const ULONG extPages, const USHORT pageSize)
+{
+/**************************************
+ *
+ *	P I O _ e x t e n d
+ *
+ **************************************
+ *
+ * Functional description
+ *	Extend file by extPages pages of pageSize size. 
+ *
+ **************************************/
+ 
+	// hvlad: prevent other reading\writing threads from changing file pointer.
+	// As we open file without FILE_FLAG_OVERLAPPED, ReadFile\WriteFile calls
+	// will change file pointer we set here and file truncation instead of file
+	// extension will occurs.
+	// It solved issue CORE-1468 (database file corruption when file extension 
+	// and read\write activity performed simultaneously)
+
+	// if file have no extend lock it is better to not extend file than corrupt it
+	if (!main_file->fil_ext_lock)
+		return;
+
+	ThreadExit teHolder;
+	FileExtendLockGuard extLock(main_file->fil_ext_lock, true);
+
+	ULONG leftPages = extPages;
+	for (jrd_file* file = main_file; file && leftPages; file = file->fil_next)
+	{
+		const ULONG filePages = PIO_get_number_of_pages(file, pageSize);
+		const ULONG fileMaxPages = (file->fil_max_page == MAX_ULONG) ? MAX_ULONG : 
+									file->fil_max_page - file->fil_min_page + 1;
+		if (filePages < fileMaxPages)
+		{
+			const ULONG extendBy = MIN(fileMaxPages - filePages + file->fil_fudge, leftPages);
+
+			HANDLE hFile = file->fil_desc;
+
+			LARGE_INTEGER newSize; 
+			newSize.QuadPart = (ULONGLONG) (filePages + extendBy) * pageSize;
+
+			if (ostype == OS_CHICAGO) {	// WIN95
+				file->fil_mutex.enter();
+			}
+
+			const DWORD ret = SetFilePointer(hFile, newSize.LowPart, &newSize.HighPart, FILE_BEGIN);
+			if (ret == INVALID_SET_FILE_POINTER && GetLastError() != NO_ERROR) {
+				if (ostype == OS_CHICAGO) {
+					file->fil_mutex.leave();
+				}
+				nt_error("SetFilePointer", file, isc_io_write_err, 0);
+			}
+			if (!SetEndOfFile(hFile)) {
+				if (ostype == OS_CHICAGO) {
+					file->fil_mutex.leave();
+				}
+				nt_error("SetEndOfFile", file, isc_io_write_err, 0);
+			}
+
+			if (ostype == OS_CHICAGO) {
+				file->fil_mutex.leave();
+			}
+
+			leftPages -= extendBy;
+		}
+	}
+}
+
+
 void PIO_flush(jrd_file* main_file)
 {
 /**************************************
@@ -271,18 +362,18 @@ void PIO_flush(jrd_file* main_file)
 	{
 		if (ostype == OS_CHICAGO)
 		{
-			THD_MUTEX_LOCK(file->fil_mutex);
+			file->fil_mutex.enter();
 		}
-		FlushFileBuffers((HANDLE) file->fil_desc);
+		FlushFileBuffers(file->fil_desc);
 		if (ostype == OS_CHICAGO)
 		{
-			THD_MUTEX_UNLOCK(file->fil_mutex);
+			file->fil_mutex.leave();
 		}
 	}
 }
 
 
-void PIO_force_write(jrd_file* file, bool flag)
+void PIO_force_write(jrd_file* file, bool forceWrite, bool notUseFSCache)
 {
 /**************************************
  *
@@ -295,23 +386,26 @@ void PIO_force_write(jrd_file* file, bool flag)
  *
  **************************************/
 
-	const bool bOldForce = (file->fil_flags & FIL_force_write) != 0;
+	const bool oldForce = (file->fil_flags & FIL_force_write) != 0;
+	const bool oldNotUseCache = (file->fil_flags & FIL_no_fs_cache) != 0;
 
-	if ((flag && !bOldForce) || (!flag && bOldForce)) {
-		SLONG& hOld = flag ? file->fil_desc : file->fil_force_write_desc;
-		HANDLE& hNew = reinterpret_cast<HANDLE&>(flag ? file->fil_force_write_desc : file->fil_desc);
-        const int force = flag ? FILE_FLAG_WRITE_THROUGH : 0;
+	if (forceWrite != oldForce || notUseFSCache != oldNotUseCache)
+	{
+		const int force = forceWrite ? FILE_FLAG_WRITE_THROUGH : 0;
+		const int fsCache = notUseFSCache ? FILE_FLAG_NO_BUFFERING : 0;
+		const int writeMode = (file->fil_flags & FIL_readonly) ? 0 : GENERIC_WRITE;
 
-		MaybeCloseFile(&hOld);
-		hNew = CreateFile(file->fil_string,
-						  GENERIC_READ | GENERIC_WRITE,
+        HANDLE& hFile = file->fil_desc;
+		maybe_close_file(hFile);
+		hFile = CreateFile(file->fil_string,
+						  GENERIC_READ | writeMode,
 						  g_dwShareFlags,
 						  NULL,
 						  OPEN_EXISTING,
-						  FILE_ATTRIBUTE_NORMAL | force | g_dwExtraFlags,
+						  FILE_ATTRIBUTE_NORMAL | force | fsCache | g_dwExtraFlags,
 						  0);
 
-		if (hNew == INVALID_HANDLE_VALUE)
+		if (hFile == INVALID_HANDLE_VALUE)
 		{
 			ERR_post(isc_io_error,
 					 isc_arg_string,
@@ -325,12 +419,18 @@ void PIO_force_write(jrd_file* file, bool flag)
 					 0);
 		}
 		
-		if (flag) {
-			file->fil_flags |= (FIL_force_write | FIL_force_write_init);
+		if (forceWrite) {
+			file->fil_flags |= FIL_force_write;
 		}
 		else {
 			file->fil_flags &= ~FIL_force_write;
 		}
+		if (notUseFSCache) {
+			file->fil_flags |= FIL_no_fs_cache;
+		}
+		else {
+			file->fil_flags &= ~FIL_no_fs_cache;
+		}
 	}
 }
 
@@ -351,17 +451,19 @@ void PIO_header(Database* dbb, SCHAR * address, int length)
  *  callers should not rely on this behavior
  *
  **************************************/
-	jrd_file* file = dbb->dbb_file;
-	HANDLE desc = (HANDLE) ((file->fil_flags & FIL_force_write) ?
-					 file->fil_force_write_desc : file->fil_desc);
+	PageSpace* pageSpace = dbb->dbb_page_manager.findPageSpace(DB_PAGE_SPACE);
+	jrd_file* file = pageSpace->file;
+	HANDLE desc = file->fil_desc;
+
+	OVERLAPPED overlapped;
+	OVERLAPPED* overlapped_ptr;
 
-	OVERLAPPED overlapped, *overlapped_ptr;
 	if (ostype == OS_CHICAGO)
 	{
-		THD_MUTEX_LOCK(file->fil_mutex);
+		file->fil_mutex.enter();
 		if (SetFilePointer(desc, 0, NULL, FILE_BEGIN) == (DWORD) -1)
 		{
-			THD_MUTEX_UNLOCK(file->fil_mutex);
+			file->fil_mutex.leave();
 			nt_error("SetFilePointer", file, isc_io_read_err, 0);
 		}
 		overlapped_ptr = NULL;
@@ -387,7 +489,7 @@ void PIO_header(Database* dbb, SCHAR * address, int length)
 		{
 			if (ostype == OS_CHICAGO)
 			{
-				THD_MUTEX_UNLOCK(file->fil_mutex);
+				file->fil_mutex.leave();
 			}
 			nt_error("ReadFile", file, isc_io_read_err, 0);
 		}
@@ -410,7 +512,7 @@ void PIO_header(Database* dbb, SCHAR * address, int length)
 #else
 			if (ostype == OS_CHICAGO)
 			{
-				THD_MUTEX_UNLOCK(file->fil_mutex);
+				file->fil_mutex.leave();
 			}
 			nt_error("ReadFile", file, isc_io_read_err, 0);
 #endif
@@ -422,67 +524,115 @@ void PIO_header(Database* dbb, SCHAR * address, int length)
 #endif
 
 	if (ostype == OS_CHICAGO) {
-		THD_MUTEX_UNLOCK(file->fil_mutex);
+		file->fil_mutex.leave();
 	}
 }
 
+namespace {
+	// we need a class here only to return memory on shutdown and avoid
+	// false memory leak reports
+	static const int ZERO_BUF_SIZE = 1024 * 128;
 
-SLONG PIO_max_alloc(Database* dbb)
+	class HugeStaticBuffer 
+	{
+	public:
+		HugeStaticBuffer(MemoryPool& p)
+			: zeroArray(p), 
+			zeroBuff(zeroArray.getBuffer(ZERO_BUF_SIZE)) 
+		{
+			memset(zeroBuff, 0, ZERO_BUF_SIZE);
+		}
+
+		const char* get() { return zeroBuff; }
+
+	private:
+		Firebird::Array<char> zeroArray;
+		char* zeroBuff;
+	};
+
+	static Firebird::InitInstance<HugeStaticBuffer> zeros;
+}
+
+
+USHORT PIO_init_data(Database* dbb, jrd_file* main_file, ISC_STATUS* status_vector, 
+					 ULONG startPage, USHORT initPages)
 {
 /**************************************
  *
- *	P I O _ m a x _ a l l o c
+ *	P I O _ i n i t _ d a t a
  *
  **************************************
  *
  * Functional description
- *	Compute last physically allocated page of database.
+ *	Initialize tail of file with zeros
  *
  **************************************/
-	jrd_file* file = dbb->dbb_file;
 
-	while (file->fil_next) {
-		file = file->fil_next;
+	const char* zero_buff = zeros().get();
+
+	ThreadExit teHolder;
+	FileExtendLockGuard extLock(main_file->fil_ext_lock, false);
+
+	// Fake buffer, used in seek_file. Page space ID have no matter there
+	// as we already know file to work with
+	BufferDesc bdb;
+	bdb.bdb_dbb = dbb;
+	bdb.bdb_page = PageNumber(0, startPage);
+
+	OVERLAPPED overlapped;
+	OVERLAPPED* overlapped_ptr;
+	jrd_file* file = 
+		seek_file(main_file, &bdb, status_vector, &overlapped, &overlapped_ptr);
+
+	if (!file)
+		return 0;
+
+	if (ostype == OS_CHICAGO)
+		file->fil_mutex.leave();
+
+	if (file->fil_min_page + 8 > startPage)
+		return 0;
+
+	USHORT leftPages = initPages;
+	const ULONG initBy = MIN(file->fil_max_page - startPage, leftPages);
+	for (ULONG i = startPage; i < startPage + initBy; )
+	{
+		bdb.bdb_page = PageNumber(0, i);
+		USHORT write_pages = ZERO_BUF_SIZE / dbb->dbb_page_size;
+		if (write_pages > leftPages)
+			write_pages = leftPages;
+
+		seek_file(main_file, &bdb, status_vector, &overlapped, &overlapped_ptr);
+
+		DWORD to_write = (DWORD) write_pages * dbb->dbb_page_size;
+		DWORD written;
+
+		if (!WriteFile(file->fil_desc, zero_buff, to_write, &written, overlapped_ptr) ||
+			to_write != written)
+		{
+			if (ostype == OS_CHICAGO)
+				file->fil_mutex.leave();
+
+			nt_error("WriteFile", file, isc_io_write_err, status_vector);
+			break;
+		}
+
+		if (ostype == OS_CHICAGO)
+			file->fil_mutex.leave();
+
+		leftPages -= write_pages;
+		i += write_pages;
 	}
 
-	const ULONG nPages = get_number_of_pages(file, dbb->dbb_page_size);
-
-	return file->fil_min_page - file->fil_fudge + nPages;
-}
-
-
-SLONG PIO_act_alloc(Database* dbb)
-{
-/**************************************
- *
- *	P I O _ a c t _ a l l o c
- *
- **************************************
- *
- * Functional description
- *  Compute actual number of physically allocated pages of database.
- *
- **************************************/
-	SLONG tot_pages = 0;
-
-/**
- **  Traverse the linked list of files and add up the number of pages
- **  in each file
- **/
-	for (const jrd_file* file = dbb->dbb_file; file != NULL; file = file->fil_next) {
-		tot_pages += get_number_of_pages(file, dbb->dbb_page_size);
-	}
-
-	return tot_pages;
+	return (initPages - leftPages);
 }
 
 
 jrd_file* PIO_open(Database* dbb,
-			 const Firebird::PathName& string,
-			 bool trace_flag,
-			 blk* connection, 
-			 const Firebird::PathName& file_name,
-			 bool share_delete)
+				   const Firebird::PathName& string,
+				   bool trace_flag,
+				   const Firebird::PathName& file_name,
+				   bool share_delete)
 {
 /**************************************
  *
@@ -491,12 +641,11 @@ jrd_file* PIO_open(Database* dbb,
  **************************************
  *
  * Functional description
- *	Open a database file. If a "connection"
- *	block is provided, use the connection
- *	to communicate with a page/lock server.
+ *	Open a database file.
  *
  **************************************/
 	const TEXT* ptr = (string.hasData() ? string : file_name).c_str();
+	bool readOnly = false;
 
 	if (!ISC_is_WinNT())
 		share_delete = false;
@@ -539,7 +688,9 @@ jrd_file* PIO_open(Database* dbb,
 			 * the Header Page flag setting to make sure that the database is set
 			 * ReadOnly.
 			 */
-			if (!dbb->dbb_file)
+			readOnly = true;
+			PageSpace* pageSpace = dbb->dbb_page_manager.findPageSpace(DB_PAGE_SPACE);
+			if (!pageSpace->file)
 				dbb->dbb_flags |= DBB_being_opened_read_only;
 		}
 	}
@@ -547,7 +698,11 @@ jrd_file* PIO_open(Database* dbb,
 	jrd_file *file;
 	try {
 		file = setup_file(dbb, string, desc);
-	} catch(const std::exception&) {
+
+		if (readOnly)
+			file->fil_flags |= FIL_readonly;
+	}
+	catch (const Firebird::Exception&) {
 		CloseHandle(desc);
 		throw;
 	}
@@ -570,12 +725,14 @@ bool PIO_read(jrd_file* file, BufferDesc* bdb, Ods::pag* page, ISC_STATUS* statu
 	Database* dbb = bdb->bdb_dbb;
 	const DWORD size = dbb->dbb_page_size;
 
+	ThreadExit teHolder;
+	FileExtendLockGuard extLock(file->fil_ext_lock, false);
+
 	OVERLAPPED overlapped, *overlapped_ptr;
 	if (!(file = seek_file(file, bdb, status_vector, &overlapped, &overlapped_ptr)))
 		return false;
 
-	HANDLE desc = (HANDLE) ((file->fil_flags & FIL_force_write) ?
-					 file->fil_force_write_desc : file->fil_desc);
+	HANDLE desc = file->fil_desc;
 
 	if (dbb->dbb_encrypt_key.hasData())
 	{
@@ -586,7 +743,7 @@ bool PIO_read(jrd_file* file, BufferDesc* bdb, Ods::pag* page, ISC_STATUS* statu
 			|| actual_length != size)
 		{
 			if (ostype == OS_CHICAGO) {
-				THD_MUTEX_UNLOCK(file->fil_mutex);
+				file->fil_mutex.leave();
 			}
 			return nt_error("ReadFile", file, isc_io_read_err, status_vector);
 		}
@@ -610,7 +767,7 @@ bool PIO_read(jrd_file* file, BufferDesc* bdb, Ods::pag* page, ISC_STATUS* statu
 			}
 #else
 			if (ostype == OS_CHICAGO) {
-				THD_MUTEX_UNLOCK(file->fil_mutex);
+				file->fil_mutex.leave();
 			}
 			return nt_error("ReadFile", file, isc_io_read_err, status_vector);
 #endif
@@ -622,7 +779,7 @@ bool PIO_read(jrd_file* file, BufferDesc* bdb, Ods::pag* page, ISC_STATUS* statu
 #endif
 
 	if (ostype == OS_CHICAGO) {
-		THD_MUTEX_UNLOCK(file->fil_mutex);
+		file->fil_mutex.leave();
 	}
 
 	return true;
@@ -651,6 +808,8 @@ bool PIO_read_ahead(Database*		dbb,
  **************************************/
 	OVERLAPPED overlapped, *overlapped_ptr;
 
+	ThreadExit teHolder;
+
 /* If an I/O status block was passed the caller wants to
    queue an asynchronous I/O. */
 
@@ -689,8 +848,7 @@ bool PIO_read_ahead(Database*		dbb,
 			--pages;
 		}
 
-		HANDLE desc = (HANDLE) ((file->fil_flags & FIL_force_write) ?
-						 file->fil_force_write_desc : file->fil_desc);
+		HANDLE desc = file->fil_desc;
 
 		DWORD actual_length;
 		if (ReadFile(	desc,
@@ -749,6 +907,8 @@ bool PIO_status(phys_io_blk* piob, ISC_STATUS* status_vector)
  *
  **************************************/
 
+	ThreadExit teHolder;
+
 	if (!(piob->piob_flags & PIOB_success)) {
 		if (piob->piob_flags & PIOB_error) {
 			return false;
@@ -790,14 +950,16 @@ bool PIO_write(jrd_file* file, BufferDesc* bdb, Ods::pag* page, ISC_STATUS* stat
 	Database*   dbb  = bdb->bdb_dbb;
 	const DWORD size = dbb->dbb_page_size;
 
+	ThreadExit teHolder;
+	FileExtendLockGuard extLock(file->fil_ext_lock, false);
+
 	file = seek_file(file, bdb, status_vector, &overlapped,
 				   &overlapped_ptr);
 	if (!file) {
 		return false;
 	}
 
-	HANDLE desc = (HANDLE) ((file->fil_flags & FIL_force_write) ?
-					 file->fil_force_write_desc : file->fil_desc);
+	HANDLE desc = file->fil_desc;
 
 	if (dbb->dbb_encrypt_key.hasData()) {
 		SLONG spare_buffer[MAX_PAGE_SIZE / sizeof(SLONG)];
@@ -810,7 +972,7 @@ bool PIO_write(jrd_file* file, BufferDesc* bdb, Ods::pag* page, ISC_STATUS* stat
 			|| actual_length != size)
 		{
 			if (ostype == OS_CHICAGO) {
-				THD_MUTEX_UNLOCK(file->fil_mutex);
+				file->fil_mutex.leave();
 			}
 			return nt_error("WriteFile", file, isc_io_write_err, status_vector);
 		}
@@ -831,7 +993,7 @@ bool PIO_write(jrd_file* file, BufferDesc* bdb, Ods::pag* page, ISC_STATUS* stat
 			}
 #else
 			if (ostype == OS_CHICAGO) {
-				THD_MUTEX_UNLOCK(file->fil_mutex);
+				file->fil_mutex.leave();
 			}
 			return nt_error("WriteFile", file, isc_io_write_err, status_vector);
 #endif
@@ -843,19 +1005,18 @@ bool PIO_write(jrd_file* file, BufferDesc* bdb, Ods::pag* page, ISC_STATUS* stat
 #endif
 
 	if (ostype == OS_CHICAGO) {
-		THD_MUTEX_UNLOCK(file->fil_mutex);
+		file->fil_mutex.leave();
 	}
 
 	return true;
 }
 
 
-
-static ULONG get_number_of_pages(const jrd_file* file, const USHORT pagesize)
+ULONG PIO_get_number_of_pages(const jrd_file* file, const USHORT pagesize)
 {
 /**************************************
  *
- *	g e t _ n u m b e r _ o f _ p a g e s
+ *	P I O _ g e t _ n u m b e r _ o f _ p a g e s
  *
  **************************************
  *
@@ -863,8 +1024,7 @@ static ULONG get_number_of_pages(const jrd_file* file, const USHORT pagesize)
  *	Compute number of pages in file, based only on file size.
  *
  **************************************/
-	HANDLE hFile = (HANDLE) ((file->fil_flags & FIL_force_write) ?
-		file->fil_force_write_desc : file->fil_desc);
+	HANDLE hFile = file->fil_desc;
 
 	DWORD dwFileSizeHigh;
 	const DWORD dwFileSizeLow = GetFileSize(hFile, &dwFileSizeHigh);
@@ -896,14 +1056,14 @@ static void release_io_event(jrd_file* file, OVERLAPPED* overlapped)
 	if (!overlapped || !overlapped->hEvent)
 		return;
 
-	THD_MUTEX_LOCK(file->fil_mutex);
+	file->fil_mutex.enter();
 	for (int i = 0; i < MAX_FILE_IO; i++)
 		if (!file->fil_io_events[i]) {
 			file->fil_io_events[i] = overlapped->hEvent;
 			overlapped->hEvent = NULL;
 			break;
 		}
-	THD_MUTEX_UNLOCK(file->fil_mutex);
+	file->fil_mutex.leave();
 
 	if (overlapped->hEvent)
 		CloseHandle(overlapped->hEvent);
@@ -929,7 +1089,7 @@ static jrd_file* seek_file(jrd_file*			file,
  *
  **************************************/
 	Database* dbb = bdb->bdb_dbb;
-	ULONG page = bdb->bdb_page;
+	ULONG page = bdb->bdb_page.getPageNum();
 
 	for (;; file = file->fil_next) {
 		if (!file) {
@@ -947,15 +1107,14 @@ static jrd_file* seek_file(jrd_file*			file,
 		UInt32x32To64((DWORD) page, (DWORD) dbb->dbb_page_size);
 
 	if (ostype == OS_CHICAGO) {
-		THD_MUTEX_LOCK(file->fil_mutex);
-		HANDLE desc = (HANDLE) ((file->fil_flags & FIL_force_write) ?
-						 file->fil_force_write_desc : file->fil_desc);
+		file->fil_mutex.enter();
+		HANDLE desc = file->fil_desc;
 
 		if (SetFilePointer(desc,
 						   (LONG) liOffset.LowPart,
 						   &liOffset.HighPart, FILE_BEGIN) == 0xffffffff)
 		{
-			THD_MUTEX_UNLOCK(file->fil_mutex);
+			file->fil_mutex.leave();
 			nt_error("SetFilePointer", file, isc_io_access_err, status_vector);
 			return 0;
 		}
@@ -971,14 +1130,14 @@ static jrd_file* seek_file(jrd_file*			file,
 		*overlapped_ptr = overlapped;
 
 #ifdef SUPERSERVER_V2
-		THD_MUTEX_LOCK(file->fil_mutex);
+		file->fil_mutex.enter();
 		for (USHORT i = 0; i < MAX_FILE_IO; i++) {
 			if (overlapped->hEvent = (HANDLE) file->fil_io_events[i]) {
 				file->fil_io_events[i] = 0;
 				break;
 			}
 		}
-		THD_MUTEX_UNLOCK(file->fil_mutex);
+		file->fil_mutex.leave();
 		if (!overlapped->hEvent &&
 			!(overlapped->hEvent = CreateEvent(NULL, TRUE, FALSE, NULL)))
 		{
@@ -1011,20 +1170,20 @@ static jrd_file* setup_file(Database*					dbb,
 /* Allocate file block and copy file name string */
 
 	jrd_file* file = FB_NEW_RPT(*dbb->dbb_permanent, file_name.length() + 1) jrd_file;
-	file->fil_desc = reinterpret_cast<SLONG>(desc);
-	file->fil_force_write_desc =
-		reinterpret_cast<SLONG>(INVALID_HANDLE_VALUE);
+	file->fil_desc = desc;
 	file->fil_length = file_name.length();
 	file->fil_max_page = (ULONG) -1;
+	file->fil_ext_lock = NULL;
 #ifdef SUPERSERVER_V2
-	memset(file->fil_io_events, 0, MAX_FILE_IO * sizeof(SLONG));
+	memset(file->fil_io_events, 0, MAX_FILE_IO * sizeof(void*));
 #endif
 	MOVE_FAST(file_name.c_str(), file->fil_string, file_name.length());
 	file->fil_string[file_name.length()] = 0;
 
 /* If this isn't the primary file, we're done */
 
-	if (dbb->dbb_file)
+	PageSpace* pageSpace = dbb->dbb_page_manager.findPageSpace(DB_PAGE_SPACE);
+	if (pageSpace && pageSpace->file)
 		return file;
 
 /* Set a local variable that indicates whether we're running
@@ -1036,7 +1195,7 @@ static jrd_file* setup_file(Database*					dbb,
 /* Build unique lock string for file and construct lock block */
 
 	BY_HANDLE_FILE_INFORMATION file_info;
-	GetFileInformationByHandle((HANDLE) desc, &file_info);
+	GetFileInformationByHandle(desc, &file_info);
 	UCHAR lock_string[32];
 	UCHAR* p = lock_string;
 
@@ -1060,6 +1219,8 @@ static jrd_file* setup_file(Database*					dbb,
 	l = p - lock_string;
 	fb_assert(l <= sizeof(lock_string)); // In case we continue adding information.
 
+	file->fil_ext_lock = FB_NEW(*dbb->dbb_permanent) Firebird::RWLock();
+
 	Lock* lock = FB_NEW_RPT(*dbb->dbb_permanent, l) Lock;
 	dbb->dbb_lock = lock;
 	lock->lck_type = LCK_database;
@@ -1077,25 +1238,26 @@ static jrd_file* setup_file(Database*					dbb,
 	if (!LCK_lock(NULL, lock, LCK_EX, LCK_NO_WAIT)) {
 		dbb->dbb_flags &= ~DBB_exclusive;
 		thread_db* tdbb = JRD_get_thread_data();
-
+		
 		while (!LCK_lock(tdbb, lock, LCK_SW, -1)) {
 			tdbb->tdbb_status_vector[0] = 0; // Clean status vector from lock manager error code
 			// If we are in a single-threaded maintenance mode then clean up and stop waiting
 			SCHAR spare_memory[MIN_PAGE_SIZE * 2];
-			SCHAR *header_page_buffer = (SCHAR*) FB_ALIGN((IPTR) spare_memory, MIN_PAGE_SIZE);
-			Ods::header_page* header = reinterpret_cast<Ods::header_page*>(header_page_buffer);
-
+			SCHAR *header_page_buffer = (SCHAR*) FB_ALIGN((IPTR)spare_memory, MIN_PAGE_SIZE);
+		
+			pageSpace = dbb->dbb_page_manager.findPageSpace(DB_PAGE_SPACE);
 			try {
-				dbb->dbb_file = file;
+				pageSpace->file = file;
 				PIO_header(dbb, header_page_buffer, MIN_PAGE_SIZE);
-				if ((header->hdr_flags & Ods::hdr_shutdown_mask) == Ods::hdr_shutdown_single)
-					ERR_post(isc_shutdown, isc_arg_string, ERR_string(file_name), 0);
-				dbb->dbb_file = NULL; // Will be set again later by the caller				
-			} catch(const std::exception&) {
+				if ((reinterpret_cast<Ods::header_page*>(header_page_buffer)->hdr_flags & Ods::hdr_shutdown_mask) == Ods::hdr_shutdown_single)
+					ERR_post(isc_shutdown, isc_arg_cstring, file_name.length(), ERR_cstring(file_name), 0);
+				pageSpace->file = NULL; // Will be set again later by the caller				
+			}
+			catch (const Firebird::Exception&) {
 				delete dbb->dbb_lock;
 				dbb->dbb_lock = NULL;
 				delete file;
-				dbb->dbb_file = NULL; // Will be set again later by the caller
+				pageSpace->file = NULL; // Will be set again later by the caller
 				throw;
 			}
 		}
@@ -1104,7 +1266,7 @@ static jrd_file* setup_file(Database*					dbb,
 	return file;
 }
 
-static bool MaybeCloseFile(SLONG* pFile)
+static bool maybe_close_file(HANDLE& hFile)
 {
 /**************************************
  *
@@ -1117,10 +1279,10 @@ static bool MaybeCloseFile(SLONG* pFile)
  *
  **************************************/
 
-	if (pFile && (HANDLE)*pFile != INVALID_HANDLE_VALUE)
+	if (hFile != INVALID_HANDLE_VALUE)
 	{
-		CloseHandle((HANDLE)*pFile);
-		*pFile = (SLONG) INVALID_HANDLE_VALUE;
+		CloseHandle(hFile);
+		hFile = INVALID_HANDLE_VALUE;
 		return true;
 	}
 	return false;
@@ -1166,4 +1328,3 @@ static bool nt_error(TEXT*	string,
 
 	return true;
 }
-
