@@ -122,7 +122,7 @@ namespace {
 		Firebird::string login;
 		int	failCount;
 		time_t lastAttempt;
-		explicit FailedLogin(const Firebird::string& l) 
+		FailedLogin(const Firebird::string& l) 
 			: login(l), failCount(1), lastAttempt(time(0)) {}
 		FailedLogin(Firebird::MemoryPool& p, const FailedLogin& fl) 
 			: login(p, fl.login), failCount(fl.failCount), lastAttempt(fl.lastAttempt) {}
@@ -150,7 +150,7 @@ namespace {
 			const Firebird::string, FailedLogin> inherited;
 
 	public:
-		explicit FailedLogins(MemoryPool& p) : inherited(p) {}
+		FailedLogins(MemoryPool& p) : inherited(p) {}
 
 		void loginFail(const Firebird::string& login)
 		{
@@ -163,14 +163,14 @@ namespace {
 			{
 				FailedLogin& l = (*this)[pos];
 				if (t - l.lastAttempt >= FAILURE_DELAY)
-		{
+				{
 					l.failCount = 0;
 				}
 				l.lastAttempt = t;
 				if (++l.failCount >= MAX_FAILED_ATTEMPTS)
-			{
+				{
 					l.failCount = 0;
-					Jrd::DelayFailedLogin::raise(FAILURE_DELAY);
+					DelayFailedLogin::raise(FAILURE_DELAY);
 				}
 				return;
 			}
@@ -193,7 +193,7 @@ namespace {
 			if (getCount() >= MAX_CONCURRENT_FAILURES)
 			{
 				// it seems we are under attack - too many wrong logins !!!
-				Jrd::DelayFailedLogin::raise(FAILURE_DELAY);
+				DelayFailedLogin::raise(FAILURE_DELAY);
 			}
 
 			add(FailedLogin(login));
@@ -267,7 +267,7 @@ bool SecurityDatabase::lookup_user(const TEXT* user_name, int* uid, int* gid, TE
 	uname[sizeof uname - 1] = 0;
 
 	THREAD_EXIT();
-	mutex.acquire();
+	mutex.enter();
 	THREAD_ENTER();
 
 	// Attach database and compile request
@@ -281,8 +281,8 @@ bool SecurityDatabase::lookup_user(const TEXT* user_name, int* uid, int* gid, TE
 			isc_detach_database(status, &tmp);
 		}
 		THREAD_ENTER();
-		mutex.release();
-		ERR_post(isc_psw_attach, 0);
+		mutex.leave();
+		ERR_post(isc_psw_attach, isc_arg_end);
 	}
 
 	// Lookup
@@ -292,8 +292,8 @@ bool SecurityDatabase::lookup_user(const TEXT* user_name, int* uid, int* gid, TE
 	if (isc_start_transaction(status, &lookup_trans, 1, &lookup_db, sizeof(TPB), TPB))
 	{
 		THREAD_ENTER();
-		mutex.release();
-		ERR_post(isc_psw_start_trans, 0);
+		mutex.leave();
+		ERR_post(isc_psw_start_trans, isc_arg_end);
 	}
 
 	if (!isc_start_and_send(status, &lookup_req, &lookup_trans, 0, sizeof(uname), uname, 0))
@@ -323,7 +323,7 @@ bool SecurityDatabase::lookup_user(const TEXT* user_name, int* uid, int* gid, TE
 		isc_detach_database(status, &lookup_db);
 	}
 	THREAD_ENTER();
-	mutex.release();
+	mutex.leave();
 
 	return found;
 }
@@ -437,6 +437,24 @@ void SecurityDatabase::verifyUser(Firebird::string& name,
 	}
 
 #ifndef EMBEDDED
+	else
+	{
+		remoteFailedLogins().loginFail(remoteId);
+		ERR_post(isc_login, isc_arg_end);
+	}
+
+	static AmCache useNative = AM_UNKNOWN;
+	if (useNative == AM_UNKNOWN)
+	{
+		Firebird::PathName authMethod(Config::getAuthMethod());
+		useNative = (authMethod == AmNative || authMethod == AmMixed) ? 
+			AM_ENABLED : AM_DISABLED;
+	}
+	if (useNative == AM_DISABLED)
+	{
+		remoteFailedLogins().loginFail(remoteId);
+		ERR_post(isc_login, isc_arg_end);
+	}
 
 	// Look up the user name in the userinfo database and use the parameters
 	// found there. This means that another database must be accessed, and
@@ -457,7 +475,7 @@ void SecurityDatabase::verifyUser(Firebird::string& name,
 	{
 		usernameFailedLogins().loginFail(name);
 		remoteFailedLogins().loginFail(remoteId);
-		ERR_post(isc_login, 0);
+		ERR_post(isc_login, isc_arg_end);
 	}
 
 	TEXT pwt[MAX_PASSWORD_LENGTH + 2];
@@ -484,7 +502,7 @@ void SecurityDatabase::verifyUser(Firebird::string& name,
 		{
 			usernameFailedLogins().loginFail(name);
 			remoteFailedLogins().loginFail(remoteId);
-			ERR_post(isc_login, 0);
+			ERR_post(isc_login, isc_arg_end);
 		}
 	}
 
@@ -500,6 +518,17 @@ void DelayFailedLogin::raise(int sec)
 	throw DelayFailedLogin(sec);
 }
 
+ISC_STATUS DelayFailedLogin::stuff_exception(ISC_STATUS* const status_vector, Firebird::StringsBuffer*) const throw()
+{
+	ISC_STATUS* sv = status_vector;
+
+	*sv++ = isc_arg_gds;
+	*sv++ = isc_login;
+	*sv++ = isc_arg_end;
+
+	return status_vector[1];
+}
+
 void DelayFailedLogin::sleep() const
 {
 	THREAD_EXIT();
