@@ -38,7 +38,6 @@
 #include "../jrd/sdw.h"
 #include "../jrd/sbm.h"
 #include "../jrd/flags.h"
-#include "../jrd/all_proto.h"
 #include "../jrd/cch_proto.h"
 #include "../jrd/err_proto.h"
 #include "../jrd/gds_proto.h"
@@ -92,10 +91,14 @@ void SDW_add(const TEXT* file_name, USHORT shadow_number, USHORT file_flags)
 			isc_arg_end);
 	}
 
-	jrd_file* shadow_file = PIO_create(dbb, file_name, false, false);
+	jrd_file* shadow_file = PIO_create(dbb, file_name, false, false, false);
 
-	if (dbb->dbb_flags & DBB_force_write)
-		PIO_force_write(shadow_file, true);
+	if (dbb->dbb_flags & (DBB_force_write | DBB_no_fs_cache))
+	{
+		PIO_force_write(shadow_file, 
+			dbb->dbb_flags & DBB_force_write, 
+			dbb->dbb_flags & DBB_no_fs_cache);
+	}
 
 	Shadow* shadow = allocate_shadow(shadow_file, shadow_number, file_flags);
 
@@ -104,7 +107,7 @@ void SDW_add(const TEXT* file_name, USHORT shadow_number, USHORT file_flags)
 
 	if (shadow->sdw_flags & SDW_conditional)
 		shadow->sdw_flags &= ~SDW_conditional;
-	WIN window(HEADER_PAGE);
+	WIN window(HEADER_PAGE_NUMBER);
 	CCH_FETCH(tdbb, &window, LCK_write, pag_header);
 	CCH_MARK_MUST_WRITE(tdbb, &window);
 	CCH_write_all_shadows(tdbb, 0, window.win_bdb,
@@ -129,7 +132,7 @@ int SDW_add_file(const TEXT* file_name, SLONG start, USHORT shadow_number)
  *
  **************************************/
 	thread_db* tdbb = JRD_get_thread_data();
-	Database* dbb = tdbb->tdbb_database;
+	Database* dbb = tdbb->getDatabase();
 
 /* Find the file to be extended */
 
@@ -170,8 +173,12 @@ int SDW_add_file(const TEXT* file_name, SLONG start, USHORT shadow_number)
 
 	jrd_file* next = file->fil_next;
 
-	if (dbb->dbb_flags & DBB_force_write)
-		PIO_force_write(next, true);
+	if (dbb->dbb_flags & (DBB_force_write | DBB_no_fs_cache))
+	{
+		PIO_force_write(next, 
+			dbb->dbb_flags & DBB_force_write, 
+			dbb->dbb_flags & DBB_no_fs_cache);
+	}
 
 /* Always write the header page, even for a conditional
  * shadow that hasn't been activated.
@@ -276,7 +283,7 @@ else
 		delete[] spare_buffer;
 
 	}	// try
-	catch (const std::exception&) {
+	catch (const Firebird::Exception&) {
 		if (spare_buffer)
 			delete[] spare_buffer;
 		throw;
@@ -318,7 +325,7 @@ void SDW_check(void)
 			MET_delete_shadow(tdbb, shadow->sdw_number);
 			gds__log
 				("shadow %s deleted from database %s due to unavailability on write",
-				 shadow->sdw_file->fil_string, dbb->dbb_file->fil_string);
+				 shadow->sdw_file->fil_string, dbb->dbb_filename.c_str());
 		}
 
 		/* note that shutting down a shadow is destructive to 
@@ -339,7 +346,7 @@ void SDW_check(void)
 			lock->lck_owner_handle =
 				LCK_get_owner_handle(tdbb, lock->lck_type);
 			lock->lck_parent = dbb->dbb_lock;
-			lock->lck_owner = tdbb->tdbb_attachment;
+			lock->lck_owner = tdbb->getAttachment();
 
 			LCK_lock(tdbb, lock, LCK_EX, LCK_NO_WAIT);
 			if (lock->lck_physical == LCK_EX) {
@@ -366,7 +373,7 @@ bool SDW_check_conditional(void)
  *
  **************************************/
 	thread_db* tdbb = JRD_get_thread_data();
-	Database* dbb = tdbb->tdbb_database;
+	Database* dbb = tdbb->getDatabase();
 	CHECK_DBB(dbb);
 
 /* first get rid of any shadows that need to be 
@@ -403,7 +410,7 @@ bool SDW_check_conditional(void)
 
 				gds__log("conditional shadow %d %s activated for database %s",
 						 shadow->sdw_number, shadow->sdw_file->fil_string,
-						 dbb->dbb_file->fil_string);
+						 dbb->dbb_filename.c_str());
 				USHORT file_flags = FILE_shadow;
 				if (shadow->sdw_flags & SDW_manual)
 					file_flags |= FILE_manual;
@@ -452,9 +459,9 @@ void SDW_dump_pages(void)
  *
  **************************************/
 	thread_db* tdbb = JRD_get_thread_data();
-	Database* dbb = tdbb->tdbb_database;
+	Database* dbb = tdbb->getDatabase();
 	gds__log("conditional shadow dumped for database %s",
-			 dbb->dbb_file->fil_string);
+			 dbb->dbb_filename.c_str());
 	const SLONG max = PAG_last_page();
 
 /* mark the first shadow in the list because we don't
@@ -463,7 +470,7 @@ void SDW_dump_pages(void)
 
 /* none of these pages should need any alteration
    since header pages for extend files are not handled at this level */
-	WIN window(-1);
+	WIN window(DB_PAGE_SPACE, -1);
 	window.win_flags = WIN_large_scan;
 	window.win_scans = 1;
 
@@ -532,7 +539,7 @@ void SDW_get_shadows(void)
  *
  **************************************/
 	thread_db* tdbb = JRD_get_thread_data();
-	Database* dbb = tdbb->tdbb_database;
+	Database* dbb = tdbb->getDatabase();
 	CHECK_DBB(dbb);
 
 /* unless we have one, get a shared lock to ensure that we don't miss any
@@ -545,9 +552,8 @@ void SDW_get_shadows(void)
 	if (lock->lck_physical != LCK_SR) {
 		/* fb_assert (lock->lck_physical == LCK_none); */
 
-		WIN window(HEADER_PAGE);
-		const header_page* header =
-			(header_page*) CCH_FETCH(tdbb, &window, LCK_read, pag_header);
+		WIN window(HEADER_PAGE_NUMBER);
+		const header_page* header = (header_page*) CCH_FETCH(tdbb, &window, LCK_read, pag_header);
 		lock->lck_key.lck_long = header->hdr_shadow_count;
 		LCK_lock(tdbb, lock, LCK_SR, LCK_WAIT);
 		CCH_RELEASE(tdbb, &window);
@@ -578,7 +584,7 @@ void SDW_init(bool activate, bool delete_files)
  *
  **************************************/
 	thread_db* tdbb = JRD_get_thread_data();
-	Database* dbb = tdbb->tdbb_database;
+	Database* dbb = tdbb->getDatabase();
 	CHECK_DBB(dbb);
 
 /* set up the lock block for synchronizing addition of new shadows */
@@ -600,7 +606,7 @@ void SDW_init(bool activate, bool delete_files)
 
 /* get current shadow lock count from database header page */
 
-	WIN window(HEADER_PAGE);
+	WIN window(HEADER_PAGE_NUMBER);
 
 	header = (header_page*) CCH_FETCH(tdbb, &window, LCK_read, pag_header);
 	lock->lck_key.lck_long = header->hdr_shadow_count;
@@ -678,16 +684,15 @@ void SDW_notify(void)
  *
  **************************************/
 	thread_db* tdbb = JRD_get_thread_data();
-	Database* dbb = tdbb->tdbb_database;
+	Database* dbb = tdbb->getDatabase();
 	CHECK_DBB(dbb);
 
 /* get current shadow lock count from database header page --
    note that since other processes need the header page to issue locks
    on the shadow count, this is effectively an uninterruptible operation */
 
-	WIN window(HEADER_PAGE);
-	header_page* header =
-		(header_page*) CCH_FETCH(tdbb, &window, LCK_write, pag_header);
+	WIN window(HEADER_PAGE_NUMBER);
+	header_page* header = (header_page*) CCH_FETCH(tdbb, &window, LCK_write, pag_header);
 	CCH_MARK_MUST_WRITE(tdbb, &window);
 
 /* get an exclusive lock on the current shadowing semaphore to
@@ -732,7 +737,8 @@ bool SDW_rollover_to_shadow(jrd_file* file, const bool inAst)
 	thread_db* tdbb = JRD_get_thread_data();
 	Database* dbb = GET_DBB();
 
-	if (file != dbb->dbb_file)
+	PageSpace* pageSpace = dbb->dbb_page_manager.findPageSpace(DB_PAGE_SPACE);
+	if (file != pageSpace->file)
 		return true;
 
 	Lock temp_lock;
@@ -744,7 +750,7 @@ bool SDW_rollover_to_shadow(jrd_file* file, const bool inAst)
 	update_lock->lck_owner_handle =
 		LCK_get_owner_handle(tdbb, update_lock->lck_type);
 	update_lock->lck_parent = dbb->dbb_lock;
-	update_lock->lck_owner = tdbb->tdbb_attachment;
+	update_lock->lck_owner = tdbb->getAttachment();
 
 	SLONG sdw_update_flags = SDW_rollover;
 
@@ -753,16 +759,16 @@ bool SDW_rollover_to_shadow(jrd_file* file, const bool inAst)
 	// CCH_fini(), then consider us accessing the shadow exclusively.
 	// LCK_update_shadow locking isn't going to work anyway. The below
 	// code must be executed for valid active attachments only.
-	if (tdbb->tdbb_attachment->att_flags & ATT_lck_init_done) {
+	if (tdbb->getAttachment()->att_flags & ATT_lck_init_done) {
 		if (update_lock->lck_physical != LCK_EX ||
-			file != dbb->dbb_file || !SDW_lck_update(sdw_update_flags)) 
+			file != pageSpace->file || !SDW_lck_update(sdw_update_flags))
 		{
 			LCK_release(tdbb, update_lock);
 			LCK_lock(tdbb, update_lock, LCK_SR, LCK_NO_WAIT);
 			while (update_lock->lck_physical != LCK_SR) {
 				if (dbb->dbb_ast_flags & DBB_get_shadows)
 					break;
-				if ((file != dbb->dbb_file) || !dbb->dbb_shadow_lock)
+				if ((file != pageSpace->file ) || !dbb->dbb_shadow_lock)
 					break;
 				LCK_lock(tdbb, update_lock, LCK_SR, LCK_NO_WAIT);
 			}
@@ -800,7 +806,7 @@ bool SDW_rollover_to_shadow(jrd_file* file, const bool inAst)
 		return false;
 	}
 
-	if (file != dbb->dbb_file) {
+	if (file != pageSpace->file) {
 		LCK_write_data(shadow_lock, (SLONG) 0);
 		LCK_release(tdbb, update_lock);
 		return true;
@@ -809,10 +815,10 @@ bool SDW_rollover_to_shadow(jrd_file* file, const bool inAst)
 /* close the main database file if possible
    and release all file blocks */
 
-	PIO_close(dbb->dbb_file);
+	PIO_close(pageSpace->file);
 
-	while ( (file = dbb->dbb_file) ) {
-		dbb->dbb_file = file->fil_next;
+	while ( (file = pageSpace->file) ) {
+		pageSpace->file = file->fil_next;
 		delete file;
 	}
 
@@ -823,7 +829,7 @@ bool SDW_rollover_to_shadow(jrd_file* file, const bool inAst)
    that would cause us to create a new shadow block for
    it the next time we do a MET_get_shadow_files () */
 
-	dbb->dbb_file = shadow->sdw_file;
+	pageSpace->file = shadow->sdw_file;
 	shadow->sdw_flags |= SDW_rollover;
 
 /* check conditional does a meta data update - since we were
@@ -846,7 +852,7 @@ bool SDW_rollover_to_shadow(jrd_file* file, const bool inAst)
 	if (start_conditional && !inAst) {
 		CCH_unwind(tdbb, false);
 		SDW_dump_pages();
-		ERR_post(isc_deadlock, 0);
+		ERR_post(isc_deadlock, isc_arg_end);
 	}
 
 	return true;
@@ -912,7 +918,7 @@ void SDW_start(const TEXT* file_name,
 	USHORT header_fetched = 0;
 
 	thread_db* tdbb = JRD_get_thread_data();
-	Database* dbb = tdbb->tdbb_database;
+	Database* dbb = tdbb->getDatabase();
 
 /* check that this shadow has not already been started,
    (unless it is marked as invalid, in which case it may
@@ -937,7 +943,9 @@ void SDW_start(const TEXT* file_name,
 
 	Firebird::PathName expanded_name(file_name);
 	ISC_expand_filename(expanded_name, false);
-	jrd_file* dbb_file = dbb->dbb_file;
+	PageSpace* pageSpace = 
+		dbb->dbb_page_manager.findPageSpace(DB_PAGE_SPACE);
+	jrd_file* dbb_file = pageSpace->file;
 
 	if (dbb_file && dbb_file->fil_string && 
 		expanded_name == dbb_file->fil_string) 
@@ -945,7 +953,7 @@ void SDW_start(const TEXT* file_name,
 		if (shadow && (shadow->sdw_flags & SDW_rollover))
 			return;
 		else
-			ERR_post(isc_shadow_accessed, 0);
+			ERR_post(isc_shadow_accessed, isc_arg_end);
 	}
 
 // Verify shadow file path against DatabaseAccess entry of firebird.conf
@@ -968,15 +976,19 @@ void SDW_start(const TEXT* file_name,
 												1) & ~((U_IPTR) MIN_PAGE_SIZE
 													   - 1)));
 
-	WIN window(-1);
+	WIN window(DB_PAGE_SPACE, -1);
 	jrd_file* shadow_file = 0;
 
 	try {
 
-	shadow_file = PIO_open(dbb, expanded_name, false, 0, file_name, false);
+	shadow_file =
+		PIO_open(dbb, expanded_name, false, file_name, false);
 
-	if (dbb->dbb_flags & DBB_force_write) {
-		PIO_force_write(shadow_file, true);
+	if (dbb->dbb_flags & (DBB_force_write | DBB_no_fs_cache))
+	{
+		PIO_force_write(shadow_file, 
+			dbb->dbb_flags & DBB_force_write, 
+			dbb->dbb_flags & DBB_no_fs_cache);
 	}
 
 	if (!(file_flags & FILE_conditional))
@@ -986,7 +998,7 @@ void SDW_start(const TEXT* file_name,
 		   2. make sure the database and shadow are in sync by checking the creation time/transaction id
 		   3. make sure that the shadow has not already been activated */
 
-		window.win_page = HEADER_PAGE;
+		window.win_page = HEADER_PAGE_NUMBER;
 		const header_page* database_header =
 			(header_page*) CCH_FETCH(tdbb, &window, LCK_read, pag_header);
 		header_fetched++;
@@ -1033,7 +1045,7 @@ void SDW_start(const TEXT* file_name,
 			 database_header->hdr_creation_date[0])
 			|| (shadow_header->hdr_creation_date[1] !=
 				database_header->hdr_creation_date[1])
-			|| (!shadow_header->hdr_flags & hdr_active_shadow))
+			|| !(shadow_header->hdr_flags & hdr_active_shadow))
 		{
 			ERR_punt();
 		}
@@ -1057,7 +1069,7 @@ void SDW_start(const TEXT* file_name,
 		delete[] spare_buffer;
 
 	}	// try
-	catch (const std::exception& ex) {
+	catch (const Firebird::Exception& ex) {
 		Firebird::stuff_exception(tdbb->tdbb_status_vector, ex);
 		if (header_fetched) {
 			CCH_RELEASE(tdbb, &window);
@@ -1070,7 +1082,7 @@ void SDW_start(const TEXT* file_name,
 			delete[] spare_buffer;
 		if (file_flags & FILE_manual && !delete_files) {
 			ERR_post(isc_shadow_missing, isc_arg_number,
-					 (SLONG) shadow_number, 0);
+					 (SLONG) shadow_number, isc_arg_end);
 		}
 		else
 		{
@@ -1113,10 +1125,10 @@ int SDW_start_shadowing(void* ast_object)
 	thread_db thd_context, *tdbb;
 	JRD_set_thread_data(tdbb, thd_context);
 
-	tdbb->tdbb_database = new_dbb;
+	tdbb->setDatabase(new_dbb);
 	tdbb->tdbb_quantum = QUANTUM;
-	tdbb->tdbb_request = NULL;
-	tdbb->tdbb_transaction = NULL;
+	tdbb->setRequest(NULL);
+	tdbb->setTransaction(NULL);
 
 	new_dbb->dbb_ast_flags |= DBB_get_shadows;
 	if (LCK_read_data(lock) & SDW_rollover)
@@ -1146,16 +1158,16 @@ static void activate_shadow(void)
  *
  **************************************/
 	thread_db* tdbb = JRD_get_thread_data();
-	Database* dbb = tdbb->tdbb_database;
+	Database* dbb = tdbb->getDatabase();
 	CHECK_DBB(dbb);
 
-	gds__log("activating shadow file %s", dbb->dbb_file->fil_string);
+	gds__log("activating shadow file %s", dbb->dbb_filename.c_str());
 
 	MET_activate_shadow(tdbb);
 
 /* clear the shadow bit on the header page */
 
-	WIN window(HEADER_PAGE);
+	WIN window(HEADER_PAGE_NUMBER);
 	header_page* header = (header_page*) CCH_FETCH(tdbb, &window, LCK_write, pag_header);
 	CCH_MARK_MUST_WRITE(tdbb, &window);
 	header->hdr_flags &= ~hdr_active_shadow;
@@ -1226,17 +1238,17 @@ static bool check_for_file(const SCHAR* name, USHORT length)
  **************************************/
 
 	thread_db* tdbb = JRD_get_thread_data();
-	Database* dbb = tdbb->tdbb_database;
+	Database* dbb = tdbb->getDatabase();
 	const Firebird::PathName path(name, length);
 
 	try {
-//  This use of PIO_open is NOT checked against DatabaseAccess configuration
-// parameter. It's not required, because here we only check for presence of
-// existing file, never really use (or create) it.
-		jrd_file* temp_file = PIO_open(dbb, path, false, 0, path, false);
+		// This use of PIO_open is NOT checked against DatabaseAccess configuration
+		// parameter. It's not required, because here we only check for presence of
+		// existing file, never really use (or create) it.
+		jrd_file* temp_file = PIO_open(dbb, path, false, path, false);
 		PIO_close(temp_file);
 	}	// try
-	catch (const std::exception& ex) {
+	catch (const Firebird::Exception& ex) {
 		Firebird::stuff_exception(tdbb->tdbb_status_vector, ex);
 		return false;
 	}
@@ -1259,7 +1271,7 @@ static void check_if_got_ast(jrd_file* file)
  *
  **************************************/
 	thread_db* tdbb = JRD_get_thread_data();
-	Database* dbb = tdbb->tdbb_database;
+	Database* dbb = tdbb->getDatabase();
 	CHECK_DBB(dbb);
 
 	Lock* lock = dbb->dbb_shadow_lock;
@@ -1291,13 +1303,13 @@ static void copy_header(void)
  *
  **************************************/
 	thread_db* tdbb = JRD_get_thread_data();
-	Database* dbb = tdbb->tdbb_database;
+	Database* dbb = tdbb->getDatabase();
 	CHECK_DBB(dbb);
 
 /* get the database header page and write it out --
    CCH will take care of modifying it */
 
-	WIN window(HEADER_PAGE);
+	WIN window(HEADER_PAGE_NUMBER);
 	CCH_FETCH(tdbb, &window, LCK_write, pag_header);
 	CCH_MARK_MUST_WRITE(tdbb, &window);
 	CCH_RELEASE(tdbb, &window);
@@ -1334,15 +1346,17 @@ static void update_dbb_to_sdw(Database* dbb)
 /* close the main database file if possible
    and release all file blocks */
 
-	PIO_close(dbb->dbb_file);
+	PageSpace* pageSpace = 
+		dbb->dbb_page_manager.findPageSpace(DB_PAGE_SPACE);
+	PIO_close(pageSpace->file);
 
 	jrd_file* file;
-	while ( (file = dbb->dbb_file) ) {
-		dbb->dbb_file = file->fil_next;
+	while ( (file = pageSpace->file) ) {
+		pageSpace->file = file->fil_next;
 		delete file;
 	}
 
-	dbb->dbb_file = shadow->sdw_file;
+	pageSpace->file = shadow->sdw_file;
 	shadow->sdw_flags |= SDW_rollover;
 }
 
