@@ -1366,14 +1366,14 @@ ISC_STATUS SVC_query2(Service* service,
 	}
 
 	if (info < end)
-		*info = isc_info_end;
+		*info++ = isc_info_end;
 
 	if (start_info && (end - info >= 7))
 	{
-		SLONG number = 1 + (info - start_info);
+		SLONG number = info - start_info;
 		memmove(start_info + 7, start_info, number);
 		USHORT length2 = INF_convert(number, buffer);
-		INF_put_item(isc_info_length, length2, buffer, start_info, end);
+		INF_put_item(isc_info_length, length2, buffer, start_info, end, true);
 	}
 
 	if (!(service->svc_flags & SVC_thd_running))
@@ -2361,7 +2361,7 @@ static void service_get(Service*		service,
 		const time_t elapsed_time = end_time - start_time;
 #endif
 		if ((timeout) && (elapsed_time >= timeout)) {
-			service->svc_flags &= SVC_timeout;
+			service->svc_flags |= SVC_timeout;
 			return;
 		}
 
