@@ -36,8 +36,11 @@
 
 #include "../jrd/jrd_blks.h"
 #include "../include/fb_blk.h"
+#include "../common/classes/array.h"
+#include "../common/classes/SafeArg.h"
 
-void SVC_STATUS_ARG(ISC_STATUS*& status, USHORT type, const void* value);
+void SVC_STATUS_ARG(ISC_STATUS*& status, const MsgFormat::safe_cell& value);
+void SVC_STATUS_ARG(ISC_STATUS*& status, const char* value);
 
 namespace Jrd {
 
@@ -81,6 +84,9 @@ const USHORT isc_action_max				= 14;
 #else
 #define SERVICE_THD_PARAM "-svc"
 #endif
+#ifdef TRUSTED_SERVICES
+#define TRUSTED_USER_SWITCH "-TRUSTED_SVC"
+#endif
 
 /* Macro used to store services thread specific data */
 /* Currently we store empty string, see bug #10394 */
@@ -103,6 +109,8 @@ class Service : public pool_alloc<type_svc>
 {
 private:
 	ISC_STATUS_ARRAY svc_status_array;
+	Firebird::string svc_parsed_sw;		// Here point elements of svc_argv
+
 public:
 	Service(serv_entry *se, Firebird::MemoryPool& p);
 	~Service();
@@ -114,7 +122,7 @@ public:
 	ULONG	svc_stdout_head;
 	ULONG	svc_stdout_tail;
 	UCHAR*	svc_stdout;
-	TEXT**	svc_argv;
+	Firebird::HalfStaticArray<const char*, 20>	svc_argv;
 	ULONG	svc_argc;
 	Firebird::Semaphore	svcStart;
 	serv_entry*	svc_service;
@@ -129,11 +137,15 @@ public:
 	bool	svc_uses_security_database;
 	Firebird::string	svc_username;
 	Firebird::string	svc_enc_password;
+#ifdef TRUSTED_SERVICES
+	Firebird::string	svc_trusted_login;
+#endif
 	Firebird::string	svc_switches;	// Full set of switches
 	Firebird::string	svc_perm_sw;	// Switches, taken from services table 
 										// and/or passed using spb_command_line
 	
 	void	svc_started();
+	void	parseSwitches();			// Create svc_argv, svc_argc and svc_parsed_sw
 };
 
 /* Bitmask values for the svc_flags variable */
