@@ -34,9 +34,10 @@
    operation to the four concatenations of SQL$ and the relation name.
    This blr cannot be generate by gdef.  The modified blr is:
 
+       blr_cast, blr_varying2, 3, 0, 31, 0,
        blr_substring,
        blr_concatenate, 
-          blr_literal, blr_text, 4,0, 'S','Q','L','$',
+          blr_literal, blr_text2, 1, 0, 4, 0, 'S', 'Q', 'L', '$',
           blr_field, 5, 17, 'R','D','B','$','R','E','L','A','T','I','O','N','_','N','A','M','E', 
        blr_literal, blr_long, 0, 0,0,0,0,
        blr_literal, blr_long, 0, 31,0,0,0,
@@ -115,9 +116,10 @@ static const UCHAR trigger1[] =
    than 32 characters.
    This blr cannot be generate by gdef.  The modified blr is:
 
+       blr_cast, blr_varying2, 3, 0, 31, 0,
        blr_substring,
        blr_concatenate, 
-          blr_literal, blr_text, 4,0, 'S','Q','L','$',
+          blr_literal, blr_text2, 1, 0, 4, 0, 'S', 'Q', 'L', '$',
           blr_field, 5, 17, 'R','D','B','$','R','E','L','A','T','I','O','N','_','N','A','M','E', 
        blr_literal, blr_long, 0, 0,0,0,0,
        blr_literal, blr_long, 0, 31,0,0,0,
@@ -563,9 +565,10 @@ static const UCHAR trigger1[] =
 	blr_modify, 6, 15,
 	blr_begin,
 	blr_assignment,
+	blr_cast, blr_varying2, 3, 0, 31, 0,
 	blr_substring,
 	blr_concatenate,
-	blr_literal, blr_text, 4, 0, 'S', 'Q', 'L', '$',
+	blr_literal, blr_text2, 1, 0, 4, 0, 'S', 'Q', 'L', '$',
 	blr_field, 6, 17, 'R', 'D', 'B', '$', 'R', 'E', 'L', 'A', 'T', 'I', 'O',
 		'N', '_', 'N', 'A', 'M', 'E',
 	blr_literal, blr_long, 0, 0, 0, 0, 0,
@@ -577,9 +580,10 @@ static const UCHAR trigger1[] =
 	blr_neq,
 	blr_field, 6, 18, 'R', 'D', 'B', '$', 'S', 'E', 'C', 'U', 'R', 'I', 'T',
 		'Y', '_', 'C', 'L', 'A', 'S', 'S',
+	blr_cast, blr_varying2, 3, 0, 31, 0,
 	blr_substring,
 	blr_concatenate,
-	blr_literal, blr_text, 4, 0, 'S', 'Q', 'L', '$',
+	blr_literal, blr_text2, 1, 0, 4, 0, 'S', 'Q', 'L', '$',
 	blr_field, 6, 17, 'R', 'D', 'B', '$', 'R', 'E', 'L', 'A', 'T', 'I', 'O',
 		'N', '_', 'N', 'A', 'M', 'E',
 	blr_literal, blr_long, 0, 0, 0, 0, 0,
@@ -722,9 +726,10 @@ static const UCHAR trigger1[] =
 	blr_modify, 18, 20,
 	blr_begin,
 	blr_assignment,
+	blr_cast, blr_varying2, 3, 0, 31, 0,
 	blr_substring,
 	blr_concatenate,
-	blr_literal, blr_text, 4, 0, 'S', 'Q', 'L', '$',
+	blr_literal, blr_text2, 1, 0, 4, 0, 'S', 'Q', 'L', '$',
 	blr_field, 18, 18, 'R', 'D', 'B', '$', 'P', 'R', 'O', 'C', 'E', 'D', 'U',
 		'R', 'E', '_', 'N', 'A', 'M', 'E',
 	blr_literal, blr_long, 0, 0, 0, 0, 0,
@@ -736,9 +741,10 @@ static const UCHAR trigger1[] =
 	blr_neq,
 	blr_field, 18, 18, 'R', 'D', 'B', '$', 'S', 'E', 'C', 'U', 'R', 'I', 'T',
 		'Y', '_', 'C', 'L', 'A', 'S', 'S',
+	blr_cast, blr_varying2, 3, 0, 31, 0,
 	blr_substring,
 	blr_concatenate,
-	blr_literal, blr_text, 4, 0, 'S', 'Q', 'L', '$',
+	blr_literal, blr_text2, 1, 0, 4, 0, 'S', 'Q', 'L', '$',
 	blr_field, 18, 18, 'R', 'D', 'B', '$', 'P', 'R', 'O', 'C', 'E', 'D', 'U',
 		'R', 'E', '_', 'N', 'A', 'M', 'E',
 	blr_literal, blr_long, 0, 0, 0, 0, 0,
@@ -1843,9 +1849,56 @@ static const UCHAR trigger22[] = {
 	blr_literal, blr_text, 5, 0, 'C', 'H', 'E', 'C', 'K',
 	blr_end,
 	blr_begin,
+	// ASF: Instead of always throw error here as before, we now check if any (except
+	// RDB$DESCRIPTION and RDB$TRIGGER_SOURCE) field was changed.
+	// We need this to RDB$FIX_METADATA (from metadata charset upgrade script) work.
+	blr_if,
+	blr_not,
+	blr_and,
+	blr_and,
+	blr_and,
+	blr_and,
+	blr_and,
+	blr_and,
+	blr_and,
+	blr_and,
+	blr_and,
+	blr_equiv,
+	blr_field, 0, 16, 'R','D','B','$','T','R','I','G','G','E','R','_','N','A','M','E',
+	blr_field, 1, 16, 'R','D','B','$','T','R','I','G','G','E','R','_','N','A','M','E',
+	blr_equiv,
+	blr_field, 0, 17, 'R','D','B','$','R','E','L','A','T','I','O','N','_','N','A','M','E',
+	blr_field, 1, 17, 'R','D','B','$','R','E','L','A','T','I','O','N','_','N','A','M','E',
+	blr_equiv,
+	blr_field, 0, 20, 'R','D','B','$','T','R','I','G','G','E','R','_','S','E','Q','U','E','N','C','E',
+	blr_field, 1, 20, 'R','D','B','$','T','R','I','G','G','E','R','_','S','E','Q','U','E','N','C','E',
+	blr_equiv,
+	blr_field, 0, 16, 'R','D','B','$','T','R','I','G','G','E','R','_','T','Y','P','E',
+	blr_field, 1, 16, 'R','D','B','$','T','R','I','G','G','E','R','_','T','Y','P','E',
+	blr_equiv,
+	blr_field, 0, 15, 'R','D','B','$','T','R','I','G','G','E','R','_','B','L','R',
+	blr_field, 1, 15, 'R','D','B','$','T','R','I','G','G','E','R','_','B','L','R',
+	blr_equiv,
+	blr_field, 0, 20, 'R','D','B','$','T','R','I','G','G','E','R','_','I','N','A','C','T','I','V','E',
+	blr_field, 1, 20, 'R','D','B','$','T','R','I','G','G','E','R','_','I','N','A','C','T','I','V','E',
+	blr_equiv,
+	blr_field, 0, 15, 'R','D','B','$','S','Y','S','T','E','M','_','F','L','A','G',
+	blr_field, 1, 15, 'R','D','B','$','S','Y','S','T','E','M','_','F','L','A','G',
+	blr_equiv,
+	blr_field, 0, 9, 'R','D','B','$','F','L','A','G','S',
+	blr_field, 1, 9, 'R','D','B','$','F','L','A','G','S',
+	blr_equiv,
+	blr_field, 0, 13, 'R','D','B','$','V','A','L','I','D','_','B','L','R',
+	blr_field, 1, 13, 'R','D','B','$','V','A','L','I','D','_','B','L','R',
+	blr_equiv,
+	blr_field, 0, 14, 'R','D','B','$','D','E','B','U','G','_','I','N','F','O',
+	blr_field, 1, 14, 'R','D','B','$','D','E','B','U','G','_','I','N','F','O',
+    blr_begin,
 	blr_leave, 1,
 	blr_end,
 	blr_end,
+	blr_end,
+	blr_end,
 	blr_eoc,
 };
 
