@@ -24,6 +24,16 @@
 #ifndef LOCK_LOCK_PROTO_H
 #define LOCK_LOCK_PROTO_H
 
+// Lock owner types
+// Placing it here helps avoid massive unneeded includes in lock/manager.cpp
+enum lck_owner_t {
+	LCK_OWNER_process = 1,		/* A process is the owner of the lock */
+	LCK_OWNER_database,			/* A database is the owner of the lock */
+	LCK_OWNER_attachment,		/* An atttachment is the owner of the lock */
+	LCK_OWNER_transaction		/* A transaction is the owner of the lock */
+};
+
+
 bool	LOCK_convert(SLONG, UCHAR, SSHORT, lock_ast_t, void*,
 						ISC_STATUS*);
 int		LOCK_deq(SLONG);
@@ -34,7 +44,7 @@ SLONG	LOCK_enq(SLONG, SLONG, USHORT, const UCHAR*, USHORT, UCHAR,
 bool	LOCK_set_owner_handle(SLONG, SLONG);
 void	LOCK_fini(ISC_STATUS*, SLONG *);
 int		LOCK_init(ISC_STATUS*, bool, LOCK_OWNER_T, UCHAR, SLONG *);
-void	LOCK_manager(SLONG);
+void	LOCK_manager(SLONG*);
 SLONG	LOCK_query_data(SLONG, USHORT, USHORT);
 SLONG	LOCK_read_data(SLONG);
 SLONG	LOCK_read_data2(SLONG, USHORT, const UCHAR*, USHORT, SLONG);
