@@ -348,6 +348,7 @@ double MOVQ_date_to_double(const dsc* desc)
 		temp_desc.dsc_sub_type = 0;
 		date = temp;
 		temp_desc.dsc_address = (UCHAR *) date;
+		QLI_validate_desc(temp_desc);
 		MOVQ_move(desc, &temp_desc);
 	}
 
@@ -695,6 +696,7 @@ int MOVQ_get_string(const dsc* desc, const TEXT** address, vary* temp,
 	temp_desc.dsc_scale = 0;
 	temp_desc.dsc_dtype = dtype_varying;
 	temp_desc.dsc_sub_type = ttype_ascii;
+	QLI_validate_desc(temp_desc);
 	MOVQ_move(desc, &temp_desc);
 	*address = temp->vary_string;
 
@@ -1075,6 +1077,7 @@ static void sql_date_to_text( SLONG date[1], DSC * to)
 	desc.dsc_dtype = dtype_text;
 	desc.dsc_scale = 0;
 	desc.dsc_sub_type = ttype_ascii;
+	QLI_validate_desc(desc);
 
 	MOVQ_move(&desc, to);
 }
@@ -1114,6 +1117,7 @@ static void sql_time_to_text( ULONG date[1], DSC * to)
 	desc.dsc_dtype = dtype_text;
 	desc.dsc_scale = 0;
 	desc.dsc_sub_type = ttype_ascii;
+	QLI_validate_desc(desc);
 
 	MOVQ_move(&desc, to);
 }
@@ -1155,6 +1159,7 @@ static void timestamp_to_text( SLONG date[2], DSC * to)
 	desc.dsc_dtype = dtype_text;
 	desc.dsc_scale = 0;
 	desc.dsc_sub_type = ttype_ascii;
+	QLI_validate_desc(desc);
 
 	MOVQ_move(&desc, to);
 }
@@ -1246,6 +1251,7 @@ static void numeric_to_text(const dsc* from, dsc* to)
 	intermediate.dsc_scale = scale;
 	intermediate.dsc_sub_type = 0;
 	intermediate.dsc_address = (UCHAR *) &n;
+	QLI_validate_desc(intermediate);
 
 	MOVQ_move(from, &intermediate);
 
