@@ -19,8 +19,6 @@
  *
  * All Rights Reserved.
  * Contributor(s): ______________________________________.
- * Added TCP_NO_DELAY option for superserver on Linux
- * FSG 16.03.2001
  *
  * 2002.02.15 Sean Leyne - Code Cleanup, removed obsolete "EPSON" port
  * 2002.02.15 Sean Leyne - Code Cleanup, removed obsolete "XENIX" port
@@ -86,12 +84,9 @@
 
 #if !(defined VMS || defined WIN_NT)
 #include <netinet/tcp.h>
+#include <netinet/in.h>
 #include <netdb.h>
 #include <arpa/inet.h>
-/* EKU: SINIX-Z does not define INADDR_NONE */
-#ifndef INADDR_NONE
-#define INADDR_NONE (unsigned long)-1
-#endif
 #endif // !(defined VMS || defined WIN_NT)
 
 #if (defined DARWIN || defined HPUX)
@@ -136,15 +131,11 @@ const char* GDS_HOSTS_FILE = "";
 const char* PROXY_FILE	= "/etc/gds_proxy";
 const char* HOSTS_FILE	= "/etc/hosts.equiv";
 #ifdef MULTI_THREAD
-const USHORT MAX_PTYPE	= ptype_batch_send;
+const USHORT MAX_PTYPE	= ptype_lazy_send;
 #else
 const USHORT MAX_PTYPE	= ptype_out_of_band;
 #endif
-#ifdef SMALL_FILE_NAMES
-const char* GDS_HOSTS_FILE	= "/etc/gdshosts.eqv";
-#else
 const char* GDS_HOSTS_FILE	= "/etc/gds_hosts.equiv";
-#endif
 #endif // VMS
 
 #ifdef WIN_NT
@@ -189,6 +180,14 @@ const int NOTASOCKET = EBADF;
 #define ENOBUFS	0
 #endif
 
+#ifndef FB_SEND_FLAGS
+#define FB_SEND_FLAGS 0
+#endif
+
+#ifndef FB_SETOPT_FLAGS
+#define FB_SETOPT_FLAGS 0
+#endif
+
 SLONG INET_remote_buffer;
 SLONG INET_max_data;
 static bool first_time = true;
@@ -289,7 +288,7 @@ static int fork(SOCKET, USHORT);
 #endif
 
 static in_addr get_bind_address();
-static in_addr get_host_address(const TEXT *);
+static bool get_host_address(const Firebird::string&, in_addr*);
 
 static void copy_p_cnct_repeat_array(	p_cnct::p_cnct_repeat*			pDest,
 										const p_cnct::p_cnct_repeat*	pSource,
@@ -298,7 +297,9 @@ static void copy_p_cnct_repeat_array(	p_cnct::p_cnct_repeat*			pDest,
 static void		inet_copy(const void*, UCHAR*, int);
 static int		inet_destroy(XDR *);
 static void		inet_gen_error(rem_port*, ISC_STATUS, ...);
+#if !defined(SUPERSERVER) || defined(EMBEDDED)
 static bool_t	inet_getbytes(XDR *, SCHAR *, u_int);
+#endif
 static bool_t	inet_getlong(XDR *, SLONG *);
 static u_int	inet_getpostn(XDR *);
 #if !(defined WIN_NT)
@@ -315,8 +316,8 @@ static rem_port*		inet_try_connect(	PACKET*,
 									Firebird::PathName&,
 									const TEXT*,
 									ISC_STATUS*,
-									const SCHAR*,
-									SSHORT);
+									const UCHAR*,
+									USHORT);
 static bool_t	inet_write(XDR *, int);
 static void		inet_zero(sockaddr_in*, int);
 #if !(defined WIN_NT)
@@ -334,7 +335,7 @@ static rem_port*		receive(rem_port*, PACKET *);
 static rem_port*		select_accept(rem_port*);
 
 static rem_port*		select_port(rem_port*, SLCT *);
-static rem_port*        select_multi(rem_port*, UCHAR* buffer, SSHORT bufsize, SSHORT* length);
+static bool		select_multi(rem_port*, UCHAR* buffer, SSHORT bufsize, SSHORT* length, rem_port*& port);
 static int		select_wait(rem_port*, SLCT *);
 static int		send_full(rem_port*, PACKET *);
 static int		send_partial(rem_port*, PACKET *);
@@ -352,10 +353,10 @@ static XDR::xdr_ops inet_ops =
 {
 	inet_getlong,
 	inet_putlong,
-#if defined(SUPERSERVER) && !defined(EMBEDDED)
-	REMOTE_getbytes,
-#else
+#if !defined(SUPERSERVER) || defined(EMBEDDED)
 	inet_getbytes,
+#else
+	REMOTE_getbytes,
 #endif
 	inet_putbytes,
 	inet_getpostn,
@@ -401,31 +402,27 @@ static WSADATA INET_wsadata;
 
 #ifdef  SUPERSERVER
 
-static MUTX_T	port_mutex;
-static bool		port_mutex_inited = false;
+static Firebird::Mutex	port_mutex;
 
 #define DEFER_PORT_CLEANUP
 
-inline void START_PORT_CRITICAL() {
-	if (!port_mutex_inited) {
-		port_mutex_inited = true;
-	}
+inline void START_PORT_CRITICAL() 
+{
 	THREAD_EXIT();
-	THD_mutex_lock (&port_mutex);
+	port_mutex.enter();
 	THREAD_ENTER();
 }
 
-inline void STOP_PORT_CRITICAL() {
+inline void STOP_PORT_CRITICAL()
+{
 	THREAD_EXIT();
-	THD_mutex_unlock (&port_mutex);
+	port_mutex.leave();
 	THREAD_ENTER();
 }
 
 #else
-inline void START_PORT_CRITICAL() {
-}
-inline void STOP_PORT_CRITICAL() {
-}
+inline void START_PORT_CRITICAL() { }
+inline void STOP_PORT_CRITICAL() { }
 #endif
 
 
@@ -434,8 +431,8 @@ rem_port* INET_analyze(Firebird::PathName& file_name,
 					const TEXT*	node_name,
 					const TEXT*	user_string,
 					bool	uv_flag,
-					const SCHAR*	dpb,
-					SSHORT	dpb_length)
+					const UCHAR*	dpb,
+					USHORT	dpb_length)
 {
 /**************************************
  *
@@ -502,10 +499,11 @@ rem_port* INET_analyze(Firebird::PathName& file_name,
 	static const p_cnct::p_cnct_repeat protocols_to_try1[] =
 	{
 		REMOTE_PROTOCOL(PROTOCOL_VERSION8, ptype_rpc, MAX_PTYPE, 1),
-		REMOTE_PROTOCOL(PROTOCOL_VERSION10, ptype_rpc, MAX_PTYPE, 2)
+		REMOTE_PROTOCOL(PROTOCOL_VERSION10, ptype_rpc, MAX_PTYPE, 2),
+		REMOTE_PROTOCOL(PROTOCOL_VERSION11, ptype_rpc, MAX_PTYPE, 3),
 #ifdef SCROLLABLE_CURSORS
 		,
-		REMOTE_PROTOCOL(PROTOCOL_SCROLLABLE_CURSORS, ptype_rpc, MAX_PTYPE, 3)
+		REMOTE_PROTOCOL(PROTOCOL_SCROLLABLE_CURSORS, ptype_rpc, MAX_PTYPE, 99)
 #endif
 	};
 
@@ -592,9 +590,9 @@ rem_port* INET_analyze(Firebird::PathName& file_name,
 
 /* once we've decided on a protocol, concatenate the version
    string to reflect it...  */
-
 	Firebird::string temp;
-	temp.printf("%s/P%d", port->port_version->str_data, port->port_protocol);
+	temp.printf("%s/P%d", port->port_version->str_data, 
+						  port->port_protocol & FB_PROTOCOL_MASK);
 	ALLR_free(port->port_version);
 	port->port_version = REMOTE_make_string(temp.c_str());
 
@@ -610,13 +608,17 @@ rem_port* INET_analyze(Firebird::PathName& file_name,
 		port->port_flags |= PORT_no_oob;
 	}
 
+	if (packet->p_acpt.p_acpt_type == ptype_lazy_send) {
+		port->port_flags |= PORT_lazy;
+	}
+
 	return port;
 }
 
 rem_port* INET_connect(const TEXT* name,
 							 PACKET* packet,
 							 ISC_STATUS* status_vector,
-							 USHORT flag, const SCHAR* dpb, SSHORT dpb_length)
+							 USHORT flag, const UCHAR* dpb, USHORT dpb_length)
 {
 /**************************************
  *
@@ -647,8 +649,7 @@ rem_port* INET_connect(const TEXT* name,
 
 	rem_port* port = alloc_port(0);
 	port->port_status_vector = status_vector;
-	REMOTE_get_timeout_params(port, reinterpret_cast<const UCHAR*>(dpb),
-							  dpb_length);
+	REMOTE_get_timeout_params(port, dpb, dpb_length);
 	status_vector[0] = isc_arg_gds;
 	status_vector[1] = 0;
 	status_vector[2] = isc_arg_end;
@@ -712,8 +713,6 @@ rem_port* INET_connect(const TEXT* name,
 	}
 #else
 
-	TEXT msg[BUFFER_SMALL];
-
 /* U N I X style sockets */
 
 	address.sin_family = AF_INET;
@@ -722,20 +721,16 @@ rem_port* INET_connect(const TEXT* name,
 
 	if (packet) {
 		// client connection
-		host_addr = get_host_address(host.c_str());
-
-		if (host_addr.s_addr == INADDR_NONE)
+		if (!get_host_address(host, &host_addr))
 		{
-			SNPRINTF(msg, FB_NELEM(msg),
-					"INET/INET_connect: gethostbyname (%s) failed, error code = %d",
-					host.c_str(), H_ERRNO);
-			gds__log(msg, 0);
+			gds__log("INET/INET_connect: gethostbyname (%s) failed, error code = %d",
+					 host.c_str(), H_ERRNO);
 			inet_gen_error(port,
 						   isc_network_error,
 						   isc_arg_string,
 						   port->port_connection->str_data,
 						   isc_arg_gds,
-						   isc_net_lookup_err, isc_arg_gds, isc_host_unknown, 0);
+						   isc_net_lookup_err, isc_arg_gds, isc_host_unknown, isc_arg_end);
 
 			disconnect(port);
 			return NULL;
@@ -798,10 +793,8 @@ rem_port* INET_connect(const TEXT* name,
 		{
 			/* end of modification by FSG */
 			/* this is the original code */
-			SNPRINTF(msg, FB_NELEM(msg),
-					"INET/INET_connect: getservbyname failed, error code = %d",
-					H_ERRNO);
-			gds__log(msg, 0);
+			gds__log("INET/INET_connect: getservbyname failed, error code = %d",
+					 H_ERRNO);
 			inet_gen_error(port,
 						   isc_network_error,
 						   isc_arg_string,
@@ -811,7 +804,8 @@ rem_port* INET_connect(const TEXT* name,
 						   isc_arg_gds,
 						   isc_service_unknown,
 						   isc_arg_string,
-						   protocol.c_str(), isc_arg_string, "tcp", 0);
+						   protocol.c_str(),
+						   isc_arg_string, "tcp", isc_arg_end);
 			return NULL;
 		}						/* else / not hardwired gds_db translation */
 	}
@@ -916,11 +910,12 @@ rem_port* INET_connect(const TEXT* name,
 					break;
 			}
 		}
-		else {
-			inet_error(port, "bind", isc_net_connect_listen_err, INET_ERRNO);
-			disconnect(port);
-			return NULL;
-		}
+	}
+
+	if (n == -1) {
+		inet_error(port, "bind", isc_net_connect_listen_err, INET_ERRNO);
+		disconnect(port);
+		return NULL;
 	}
 
 	n = listen((SOCKET) port->port_handle, 5);
@@ -993,8 +988,8 @@ rem_port* INET_reconnect(HANDLE handle, ISC_STATUS* status_vector)
 	port->port_handle = handle;
 	port->port_server_flags |= SRVR_server;
 
-	int optval = TRUE;
-	int n = setsockopt((SOCKET) port->port_handle, SOL_SOCKET,
+	int n = 0, optval = TRUE;
+	n = setsockopt((SOCKET) port->port_handle, SOL_SOCKET,
 					SO_KEEPALIVE, (SCHAR*) &optval, sizeof(optval));
 	if (n == -1) {
 		gds__log("inet server err: setting KEEPALIVE socket option \n");
@@ -1020,6 +1015,7 @@ rem_port* INET_server(int sock)
  *	established.  Set up port block with the appropriate socket.
  *
  **************************************/
+	int n = 0;
 #ifdef VMS
 	ISC_tcp_setup(ISC_wait, gds__completion_ast);
 #endif
@@ -1028,7 +1024,7 @@ rem_port* INET_server(int sock)
 	port->port_handle = (HANDLE) sock;
 
 	int optval = 1;
-	int n = setsockopt(sock, SOL_SOCKET, SO_KEEPALIVE,
+	n = setsockopt(sock, SOL_SOCKET, SO_KEEPALIVE,
 			   (SCHAR *) & optval, sizeof(optval));
 	if (n == -1) {
 		gds__log("inet server err: setting KEEPALIVE socket option \n");
@@ -1290,12 +1286,8 @@ static rem_port* alloc_port( rem_port* parent)
 		if (WSAStartup(version, &INET_wsadata)) {
 			if (parent)
 				inet_error(parent, "WSAStartup", isc_net_init_error, INET_ERRNO);
-			else {
-				SNPRINTF(buffer, FB_NELEM(buffer),
-						 "INET/alloc_port: WSAStartup failed, error code = %d",
-						 INET_ERRNO);
-				gds__log(buffer, 0);
-			}
+			else
+				gds__log("INET/alloc_port: WSAStartup failed, error code = %d", INET_ERRNO);
 			return NULL;
 		}
 		gds__register_cleanup(exit_handler, 0);
@@ -1313,12 +1305,7 @@ static rem_port* alloc_port( rem_port* parent)
 		}
 		INET_max_data = INET_remote_buffer;
 #ifdef DEBUG
-		{
-			char msg[BUFFER_SMALL];
-			SNPRINTF(msg, FB_NELEM(msg), " Info: Remote Buffer Size set to %ld",
-					 INET_remote_buffer);
-			gds__log(msg, 0);
-		}
+		gds__log(" Info: Remote Buffer Size set to %ld", INET_remote_buffer);
 #endif
 		first_time = false;
 	}
@@ -1516,7 +1503,8 @@ static rem_port* aux_request( rem_port* port, PACKET* packet)
 
 	int optval = TRUE;
 	int ret = setsockopt(n, SOL_SOCKET, SO_REUSEADDR,
-			   (SCHAR *) &optval, sizeof(optval));
+						 (SCHAR *) &optval, sizeof(optval));
+
 	if (ret == -1) {
 		inet_error(port, "setsockopt REUSE", isc_net_event_listen_err, INET_ERRNO);
 		return NULL;
@@ -1559,9 +1547,7 @@ static rem_port* aux_request( rem_port* port, PACKET* packet)
 				(UCHAR*) &address.sin_addr,
 				sizeof(address.sin_addr));
 
-	response->p_resp_data.cstr_address = 
-			reinterpret_cast<UCHAR*>(& response->p_resp_blob_id);
-	response->p_resp_data.cstr_length = sizeof(response->p_resp_blob_id);
+	response->p_resp_data.cstr_length = sizeof(address);
 	inet_copy(&address,
 			  response->p_resp_data.cstr_address,
 			  response->p_resp_data.cstr_length);
@@ -1618,7 +1604,7 @@ static int check_host(
  **************************************
  *
  * Functional description
- *	Check the host on the other end of the socket to see it
+ *	Check the host on the other end of the socket to see if
  *	it's an equivalent host.
  * NB.: First check the ~/.rhosts then the HOSTS_FILE - both have
  *	the same line formats (see parse_line)
@@ -1631,6 +1617,10 @@ static int check_host(
 	if (getpeername((int) port->port_handle, (struct sockaddr*)&address, &length) == -1)
 		return 0;
 
+	// If source address is in the loopback net - trust it
+	if((ntohl(address.sin_addr.s_addr) >> IN_CLASSA_NSHIFT) == IN_LOOPBACKNET)
+		return 1;
+
 	const struct hostent* host = gethostbyaddr((SCHAR *) & address.sin_addr,
 							   sizeof(address.sin_addr), address.sin_family);
 	if (!host)
@@ -1890,6 +1880,10 @@ static void cleanup_port( rem_port* port)
 	delete port->port_queue;
 #endif
 
+#ifdef TRUSTED_AUTH
+	delete port->port_trusted_auth;
+#endif
+
 	ALLR_release((UCHAR *) port);
 	return;
 }
@@ -1966,7 +1960,7 @@ static int fork( SOCKET old_handle, USHORT flag)
 					DUPLICATE_SAME_ACCESS);
 
 	Firebird::string cmdLine;
-	cmdLine.printf("%s -s -i -h %"SLONGFORMAT, name, (SLONG) new_handle);
+	cmdLine.printf("%s -i -h %"SLONGFORMAT, name, (SLONG) new_handle);
 
 	STARTUPINFO start_crud;
 	start_crud.cb = sizeof(STARTUPINFO);
@@ -1979,10 +1973,10 @@ static int fork( SOCKET old_handle, USHORT flag)
 	
 	PROCESS_INFORMATION pi;
 	if (CreateProcess(NULL, cmdLine.begin(), NULL, NULL, TRUE,
-							(flag & SRVR_high_priority ?
-							 HIGH_PRIORITY_CLASS | DETACHED_PROCESS :
-							 NORMAL_PRIORITY_CLASS | DETACHED_PROCESS),
-							NULL, NULL, &start_crud, &pi))
+					  (flag & SRVR_high_priority ?
+						 HIGH_PRIORITY_CLASS | DETACHED_PROCESS :
+						 NORMAL_PRIORITY_CLASS | DETACHED_PROCESS),
+					  NULL, NULL, &start_crud, &pi))
 	{
 		CloseHandle(pi.hThread);
 		CloseHandle(pi.hProcess);
@@ -1993,6 +1987,15 @@ static int fork( SOCKET old_handle, USHORT flag)
 }
 #endif
 
+#ifdef WIN_NT
+bool inet_aton(const char* name, in_addr* address)
+{
+	address->s_addr = inet_addr(name);
+	return address->s_addr != INADDR_NONE;
+}
+#endif
+
+
 static in_addr get_bind_address()
 {
 /**************************************
@@ -2008,15 +2011,23 @@ static in_addr get_bind_address()
 	in_addr config_address;
 
 	const char* config_option = Config::getRemoteBindAddress();
-	config_address.s_addr =
-		(config_option) ? inet_addr(config_option) : INADDR_NONE;
-	if (config_address.s_addr == INADDR_NONE) {
+	if (config_option)
+	{
+		if (!inet_aton(config_option, &config_address))
+		{
+			gds__log("Wrong RemoteBindAddress '%s' in firebird.conf - "
+					 "binding to loopback interface", config_option);
+			config_address.s_addr = htonl(INADDR_LOOPBACK);
+		}
+	}
+	else
+	{
 		config_address.s_addr = INADDR_ANY;
 	}
 	return config_address;
 }
 
-static in_addr get_host_address(const TEXT* name)
+static bool get_host_address(const Firebird::string& name, in_addr* address)
 {
 /**************************************
  *
@@ -2028,15 +2039,13 @@ static in_addr get_host_address(const TEXT* name)
  *	Return host address.
  *
  **************************************/
-	in_addr address;
+	bool rc = true;
 
 	THREAD_EXIT();
 
-	address.s_addr = inet_addr(name);
-
-	if (address.s_addr == INADDR_NONE) {
-
-		const hostent* host = gethostbyname(name);
+	if (!inet_aton(name.c_str(), address)) {
+		rc = false;
+		const hostent* host = gethostbyname(name.c_str());
 
 		/* On Windows NT/9x, gethostbyname can only accomodate
 		 * 1 call at a time.  In this case it returns the error
@@ -2049,20 +2058,21 @@ static in_addr get_host_address(const TEXT* name)
 		if (!host) {
 			if (H_ERRNO == INET_RETRY_ERRNO) {
 				for (int retry = 0; retry < INET_RETRY_CALL; retry++) {
-					if ( (host = gethostbyname(name)) )
+					if ( (host = gethostbyname(name.c_str())) )
 						break;
 				}
 			}
 		}
 
 		if (host) {
-			inet_copy(host->h_addr, (UCHAR*) &address, sizeof(address));
+			rc = true;
+			inet_copy(host->h_addr, (UCHAR*) address, sizeof(*address));
 		}
 	}
 
 	THREAD_ENTER();
 
-	return address;
+	return rc;
 }
 
 //____________________________________________________________
@@ -2196,7 +2206,7 @@ static int parse_line(
 
 	if (strcmp(entry1, host_name))
 	{
-#if (defined UNIX) && !(defined SINIXZ) && !(defined NETBSD)
+#if (defined UNIX) && !(defined NETBSD)
 		if (entry1[1] == '@')
 		{
 			if (!innetgr(&entry1[2], host_name, 0, 0))
@@ -2225,7 +2235,7 @@ static int parse_line(
 
 /* if they're in the user group: + they're in, - they're out */
 
-#if (defined UNIX) && !(defined SINIXZ) && !(defined NETBSD)
+#if (defined UNIX) && !(defined NETBSD)
 	if (entry2[1] == '@') {
 		if (innetgr(&entry2[2], 0, user_name, 0)) {
 			if (entry2[0] == '+')
@@ -2364,7 +2374,7 @@ static rem_port* receive( rem_port* main_port, PACKET * packet)
 #endif //SUPERSERVER
 }
 
-static rem_port* select_multi(rem_port* main_port, UCHAR* buffer, SSHORT bufsize, SSHORT* length)
+static bool select_multi(rem_port* main_port, UCHAR* buffer, SSHORT bufsize, SSHORT* length, rem_port*& port)
 {
 /**************************************
  *
@@ -2380,21 +2390,21 @@ static rem_port* select_multi(rem_port* main_port, UCHAR* buffer, SSHORT bufsize
  *
  **************************************/
 	fb_assert(main_port->port_server_flags & SRVR_multi_client);
-	
+
 	for (;;) 
 	{
-		rem_port* port = select_port(main_port, &INET_select);
+		port = select_port(main_port, &INET_select);
 		if (port == main_port) 
 		{
-			if (port = select_accept(main_port)) 
+			if ( (port = select_accept(main_port)) )
 			{
 				if (!packet_receive(port, buffer, bufsize, length))
 				{
 					*length = 0;
 				}
-				return port;
+				return (*length) ? true : false;
 			}
-			
+
 			continue;
 		}
 		if (port) 
@@ -2403,20 +2413,24 @@ static rem_port* select_multi(rem_port* main_port, UCHAR* buffer, SSHORT bufsize
 			{
 				port->port_dummy_timeout = port->port_dummy_packet_interval;
 				if (port->port_flags & PORT_async ||
-					port->port_protocol < PROTOCOL_VERSION8) continue;
+					port->port_protocol < PROTOCOL_VERSION8)
+				{
+					continue;
+				}
 				*length = 0;
-				return port;
+				return true;
 			}
-			
+
 			if (!packet_receive(port, buffer, bufsize, length))
 			{
 				*length = 0;
 			}
-			return port;
+			return (*length) ? true : false;
 		}
 		if (!select_wait(main_port, &INET_select))
 		{
-			return NULL;
+			port = NULL;
+			return false;
 		}
 	}
 }
@@ -2460,10 +2474,7 @@ static rem_port* select_accept( rem_port* main_port)
 	if (n >= INET_max_clients) {
 		main_port->port_state = state_closed;
 		SOCLOSE((int) main_port->port_handle);
-		TEXT msg[BUFFER_SMALL];
-		SNPRINTF(msg, FB_NELEM(msg),
-				"INET/select_accept: exec new server at client limit: %d", n);
-		gds__log(msg, 0);
+		gds__log("INET/select_accept: exec new server at client limit: %d", n);
 
 		setreuid(0, 0);
 		kill(getppid(), SIGUSR1);
@@ -2566,7 +2577,6 @@ static int select_wait( rem_port* main_port, SLCT * selct)
  *	to read from them.
  *
  **************************************/
-	TEXT msg[BUFFER_SMALL];
 	struct timeval timeout;
 	bool checkPorts = false;
 
@@ -2665,8 +2675,7 @@ static int select_wait( rem_port* main_port, SLCT * selct)
 
 		if (!found)
 		{
-			gds__log("INET/select_wait: client rundown complete, server exiting",
-				 0);
+			gds__log("INET/select_wait: client rundown complete, server exiting");
 			return FALSE;
 		}
 
@@ -2718,10 +2727,7 @@ static int select_wait( rem_port* main_port, SLCT * selct)
 			}
 			else {
 				THREAD_ENTER();
-				SNPRINTF(msg, FB_NELEM(msg),
-						 "INET/select_wait: select failed, errno = %d",
-						 inetErrNo);
-				gds__log(msg, 0);
+				gds__log("INET/select_wait: select failed, errno = %d", inetErrNo);
 				return FALSE;
 			}
 		}	// for (;;)
@@ -2879,6 +2885,7 @@ static void inet_gen_error( rem_port* port, ISC_STATUS status, ...)
 	}
 }
 
+#if !defined(SUPERSERVER) || defined(EMBEDDED)
 static bool_t inet_getbytes( XDR * xdrs, SCHAR * buff, u_int count)
 {
 /**************************************
@@ -2938,6 +2945,7 @@ static bool_t inet_getbytes( XDR * xdrs, SCHAR * buff, u_int count)
 
 	return TRUE;
 }
+#endif
 
 static bool_t inet_getlong( XDR * xdrs, SLONG * lp)
 {
@@ -3043,7 +3051,6 @@ static void inet_error(
  *	to format the status vector if any.
  *
  **************************************/
-	TEXT msg[BUFFER_SMALL];
 
 	if (status) {
 #ifdef VMS
@@ -3054,18 +3061,16 @@ static void inet_error(
 						   isc_arg_string,
 						   (ISC_STATUS) port->port_connection->str_data,
 						   isc_arg_gds, operation, isc_arg_string,
-						   (ISC_STATUS) win_errlist[status], 0);
+						   (ISC_STATUS) win_errlist[status], isc_arg_end);
 		}
 		else
 #endif
 			inet_gen_error(port, isc_network_error,
 						   isc_arg_string,
 						   (ISC_STATUS) port->port_connection->str_data,
-						   isc_arg_gds, operation, SYS_ERR, status, 0);
+						   isc_arg_gds, operation, SYS_ERR, status, isc_arg_end);
 
-		SNPRINTF(msg, FB_NELEM(msg),
-				 "INET/inet_error: %s errno = %d", function, status);
-		gds__log(msg, 0);
+		gds__log("INET/inet_error: %s errno = %d", function, status);
 	}
 	else {
 		/* No status value, just format the basic arguments. */
@@ -3073,7 +3078,7 @@ static void inet_error(
 		inet_gen_error(port, isc_network_error,
 					   isc_arg_string,
 					   (ISC_STATUS) port->port_connection->str_data, isc_arg_gds,
-					   operation, 0);
+					   operation, isc_arg_end);
 	}
 }
 
@@ -3245,8 +3250,8 @@ static rem_port* inet_try_connect(
 							 Firebird::PathName& file_name,
 							 const TEXT* node_name, 
 							 ISC_STATUS* status_vector,
-							 const SCHAR* dpb, 
-							 SSHORT dpb_length)
+							 const UCHAR* dpb,
+							 USHORT dpb_length)
 {
 /**************************************
  *
@@ -3576,7 +3581,7 @@ static int packet_receive(
 	}
 
 	if (!n) {
-		inet_error(port, "read end_of_file", isc_net_read_err, 0);
+		inet_error(port, "read end_of_file", isc_net_read_err, isc_arg_end);
 		return FALSE;
 	}
 
@@ -3591,7 +3596,7 @@ static int packet_receive(
 		if (INET_force_error == 0) {
 			INET_force_error = 1;
 			inet_error(port, "simulated error - read",
-					   isc_net_read_err, 0);
+					   isc_net_read_err, isc_arg_end);
 			return FALSE;
 		}
 	}
@@ -3603,6 +3608,7 @@ static int packet_receive(
 	return TRUE;
 }
 
+
 static bool_t packet_send( rem_port* port, const SCHAR* buffer, SSHORT buffer_length)
 {
 /**************************************
@@ -3616,12 +3622,7 @@ static bool_t packet_send( rem_port* port, const SCHAR* buffer, SSHORT buffer_le
  *
  **************************************/
 
-#ifdef SINIXZ
-// Please systems with ill-defined send() function, like SINIX-Z.
-	char* data = const_cast<char*>(buffer);
-#else
 	const char* data = buffer;
-#endif
 	SSHORT length = buffer_length;
 
 	while (length) {
@@ -3633,7 +3634,7 @@ static bool_t packet_send( rem_port* port, const SCHAR* buffer, SSHORT buffer_le
 		}
 #endif
 		SSHORT n = -1;
-		n = send((SOCKET) port->port_handle, data, length, 0);
+		n = send((SOCKET) port->port_handle, data, length, FB_SEND_FLAGS);
 #ifdef DEBUG
 		if (INET_trace & TRACE_operations) {
 			fprintf(stdout, "After Send n is %d\n", n);
@@ -3670,12 +3671,9 @@ static bool_t packet_send( rem_port* port, const SCHAR* buffer, SSHORT buffer_le
 		int count = 0;
 		SSHORT n;
 		int inetErrNo;
-#ifdef SINIXZ
-		char* b = const_cast<char*>(buffer);
-#else
 		const char* b = buffer;
-#endif
-		while ((n = send((SOCKET) port->port_handle, b, 1, MSG_OOB)) == -1 &&
+
+		while ((n = send((SOCKET) port->port_handle, b, 1, MSG_OOB | FB_SEND_FLAGS)) == -1 &&
 				(INET_ERRNO == ENOBUFS || INTERRUPT_ERROR(INET_ERRNO)))
 		{
 			inetErrNo = INET_ERRNO;
@@ -3743,7 +3741,7 @@ static bool_t packet_send( rem_port* port, const SCHAR* buffer, SSHORT buffer_le
 		if (INET_force_error == 0) {
 			INET_force_error = 1;
 			inet_error(port, "simulated error - send",
-					   isc_net_write_err, 0);
+					   isc_net_write_err, isc_arg_end);
 			return FALSE;
 		}
 	}
