@@ -25,9 +25,9 @@
 #define REMOTE_CNTL_PROTO_H
 
 void	CNTL_init(ThreadEntryPoint*, const TEXT*);
-void*	CNTL_insert_thread(void);
+void*	CNTL_insert_thread();
 void	WINAPI CNTL_main_thread(DWORD, char*[]);
-void	CNTL_remove_thread(void *);
+void	CNTL_remove_thread(void*);
 void	CNTL_shutdown_service(const TEXT*);
 
 #endif // REMOTE_CNTL_PROTO_H
