@@ -289,6 +289,7 @@ bool_t xdr_protocol(XDR* xdrs, PACKET* p)
 	P_SQLST *prep_stmt;
 	P_SQLDATA *sqldata;
 	P_TRRQ *trrq;
+	P_TRAU *trau;
 #ifdef DEBUG
 	xdr_save_size = xdrs->x_handy;
 #endif
@@ -786,14 +787,42 @@ bool_t xdr_protocol(XDR* xdrs, PACKET* p)
 		DEBUG_PRINTSIZE(xdrs, p->p_operation);
 		return P_TRUE(xdrs, p);
 
+	// the following added to have formal vulcan compatibility
+	case op_update_account_info:
+		{
+		p_update_account *stuff = &p->p_account_update;
+		MAP(xdr_short, reinterpret_cast < SSHORT & >(stuff->p_account_database));
+		MAP(xdr_cstring_const, stuff->p_account_apb);
+		DEBUG_PRINTSIZE(xdrs, p->p_operation);
+
+		return P_TRUE(xdrs, p);
+		}
+			
+	case op_authenticate_user:
+		{
+		p_authenticate *stuff = &p->p_authenticate_user;
+		MAP(xdr_short, reinterpret_cast < SSHORT & >(stuff->p_auth_database));
+		MAP(xdr_cstring_const, stuff->p_auth_dpb);
+		MAP(xdr_cstring, stuff->p_auth_items);
+		MAP(xdr_short, reinterpret_cast < SSHORT & >(stuff->p_auth_buffer_length));
+		DEBUG_PRINTSIZE(xdrs, p->p_operation);
+
+		return P_TRUE(xdrs, p);
+		}
+
+	case op_trusted_auth:
+		trau = &p->p_trau;
+		MAP(xdr_cstring, trau->p_trau_data);
+		DEBUG_PRINTSIZE(xdrs, p->p_operation);
+
+		return P_TRUE(xdrs, p);
+
 	default:
-#ifdef DEBUG
+#ifdef DEV_BUILD
 		if (xdrs->x_op != XDR_FREE)
-			fprintf(stderr, "xdr_packet: operation %d not recognized\n",
-					   p->p_operation);
-#endif
-#ifndef SOLARIS
-		fb_assert(xdrs->x_op == XDR_FREE);
+		{
+			gds__log("xdr_packet: operation %d not recognized\n", p->p_operation);
+		}
 #endif
 		return P_FALSE(xdrs, p);
 	}
@@ -1300,10 +1329,10 @@ static bool_t xdr_quad( XDR* xdrs, struct bid* ip)
 	switch (xdrs->x_op) {
 	case XDR_ENCODE:
 		if ((*xdrs->x_ops->x_putlong)
-			(xdrs, reinterpret_cast<SLONG*>(&ip->bid_relation_id))
+			(xdrs, reinterpret_cast<SLONG*>(&ip->bid_quad_high))
 			&& (*xdrs->x_ops->x_putlong) (xdrs,
 										  reinterpret_cast<
-										  SLONG*>(&ip->bid_number)))
+										  SLONG*>(&ip->bid_quad_low)))
 		{
 			return TRUE;
 		}
@@ -1312,12 +1341,12 @@ static bool_t xdr_quad( XDR* xdrs, struct bid* ip)
 	case XDR_DECODE:
 		if (!(*xdrs->x_ops->x_getlong)
 			(xdrs,
-			 reinterpret_cast<SLONG*>(&ip->bid_relation_id)))
+			 reinterpret_cast<SLONG*>(&ip->bid_quad_high)))
 		{
 			return FALSE;
 		}
 		return (*xdrs->x_ops->x_getlong) (xdrs,
-										  reinterpret_cast<SLONG*>(&ip->bid_number));
+										  reinterpret_cast<SLONG*>(&ip->bid_quad_low));
 
 	case XDR_FREE:
 		return TRUE;
@@ -1644,7 +1673,9 @@ static bool_t xdr_sql_blr(
 	rem_port* port = (rem_port*) xdrs->x_public;
 	RSR statement;
 	if (statement_id >= 0) {
-		if (!port->port_objects || statement_id >= port->port_object_vector->vec_count)
+		if (!port->port_objects)
+			return FALSE;
+		if (static_cast<ULONG>(statement_id) >= port->port_object_vector->vec_count)
 			return FALSE;
 		if (!(statement = (RSR) port->port_objects[statement_id]))
 			return FALSE;
@@ -1736,7 +1767,9 @@ static bool_t xdr_sql_message( XDR* xdrs, SLONG statement_id)
 
 	rem_port* port = (rem_port*) xdrs->x_public;
 	if (statement_id >= 0) {
-		if (!port->port_objects || statement_id >= port->port_object_vector->vec_count)
+		if (!port->port_objects)
+			return FALSE;
+		if (static_cast<ULONG>(statement_id) >= port->port_object_vector->vec_count)
 			return FALSE;
 		statement = (RSR) port->port_objects[statement_id];
 	}
