@@ -36,12 +36,16 @@
 
 // 30 Dec 2002. Nickolay Samofatov 
 // This needs to be checked for all supported platforms
-// The simpliest way to check it is to issue
-// "select abs(2.0/3.0) from rdb$database" from correct client
+// The simpliest way to check it is to issue from correct client:
+// declare external function abs2 double precision
+//   returns double precision by value
+//   entry_point 'IB_UDF_abs' module_name 'ib_udf';
+// select abs2(2.0 / 3.0) from rdb$database;
 // It will return big strange value in case of invalid define
-#if defined(i386) || defined(I386) || defined(_M_IX86) || defined(AMD64)
+// ASF: Currently, all little-endian are SWAP_DOUBLE and big-endian aren't.
+#if defined(i386) || defined(I386) || defined(_M_IX86) || defined(AMD64) || defined(ARM) || defined(MIPSEL) || defined(DARWIN64) || defined(IA64)
 #define		SWAP_DOUBLE
-#elif defined(sparc) || defined(PowerPC) || defined(PPC) || defined(__ppc__) || defined(HPUX)
+#elif defined(sparc) || defined(PowerPC) || defined(PPC) || defined(__ppc__) || defined(HPUX) || defined(MIPS) || defined(__ppc64__)
 #undef		SWAP_DOUBLE
 #else
 #error "Define SWAP_DOUBLE for your platform correctly !"
@@ -83,8 +87,9 @@ inline void DEBUG_XDR_FREE(XDR* xdrs, const void* xdrvar, const void* addr, ULON
 #endif /* DEBUG_XDR_MEMORY */
 
 /* Sun's XDR documentation says this should be "MAXUNSIGNED", but
-   for InterBase purposes, limiting strings to 65K is more than
+   for Firebird purposes, limiting strings to 65K is more than
    sufficient. */
+// This setting may be related to our max DSQL statement size.
 
 const u_int MAXSTRING_FOR_WRAPSTRING	= 65535;
 
