@@ -4,33 +4,33 @@
  *	DESCRIPTION:	Main file to provide GUI based server control functions
  *					for Firebird 2.0 Super Server
  *
- *  The contents of this file are subject to the Initial Developer's 
- *  Public License Version 1.0 (the "License"); you may not use this 
- *  file except in compliance with the License. You may obtain a copy 
+ *  The contents of this file are subject to the Initial Developer's
+ *  Public License Version 1.0 (the "License"); you may not use this
+ *  file except in compliance with the License. You may obtain a copy
  *  of the License here:
  *
  *    http://www.ibphoenix.com?a=ibphoenix&page=ibp_idpl.
  *
- *  Software distributed under the License is distributed on an "AS 
- *  IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or 
+ *  Software distributed under the License is distributed on an "AS
+ *  IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
  *  implied. See the License for the specific language governing rights
  *  and limitations under the License.
- *  
+ *
  *  The Initial Developer of the Original Code is Paul Reeves.
  *
  *  The Original Code is (C) 2003 Paul Reeves.
  *
  *  All Rights Reserved.
- *  
- *  Contributor(s): ______________________________________. 
+ *
+ *  Contributor(s): ______________________________________.
  *
  *	History:
  *		This current version is derived from the Fb 1.0 control panel applet
  *		It was adapted to support management of a dual Super Server and
- *		Classic Server install, allowing easy switching between the two server 
- *		types. Unfortunately it became obvious too late in the development 
- *		cycle that such support was not feasible without delaying release 
- *		further. Consequently the applet has been converted back to manage 
+ *		Classic Server install, allowing easy switching between the two server
+ *		types. Unfortunately it became obvious too late in the development
+ *		cycle that such support was not feasible without delaying release
+ *		further. Consequently the applet has been converted back to manage
  *		Super Server only. The relevant code for Classic has been ifdef'ed out.
  *
  *
@@ -50,7 +50,7 @@ static char BASED_CODE THIS_FILE[] = __FILE__;
 
 
 /////////////////////////////////////////////////////////////////////////////
-// CFBDialog 
+// CFBDialog
 
 
 CFBDialog::CFBDialog(CWnd* pParent /*=NULL*/)
@@ -62,13 +62,13 @@ CFBDialog::CFBDialog(CWnd* pParent /*=NULL*/)
 	//}}AFX_DATA_INIT
 
 	m_uiTimer = 0;
-    hScManager = 0;
+	hScManager = 0;
 	initialised = false;
 
 	m_Guardian_Name		= ISCGUARD_EXECUTABLE;
-	m_SS_Server_Name	= REMOTE_SS_EXECUTABLE; 
+	m_SS_Server_Name	= REMOTE_SS_EXECUTABLE;
 #ifdef MANAGE_CLASSIC
-	m_CS_Server_Name	= REMOTE_CS_EXECUTABLE; 
+	m_CS_Server_Name	= REMOTE_CS_EXECUTABLE;
 #endif
 
 	fb_status.AutoStart = false;
@@ -130,13 +130,13 @@ END_MESSAGE_MAP()
 // CFBDialog message handlers
 
 
-BOOL CFBDialog::OnInitDialog() 
+BOOL CFBDialog::OnInitDialog()
 // This method is meant to do the minimum, one-time setup stuff
-// UpdateServerStatus does most of the work, and is called on 
+// UpdateServerStatus does most of the work, and is called on
 // a timer, so we don't need to repeat that work here.
 {
 	CDialog::OnInitDialog();
-	
+
 	m_Reset_Display_To_Existing_Values = TRUE;
 
 	fb_status.ServicesAvailable = ServiceSupportAvailable();
@@ -172,12 +172,12 @@ BOOL CFBDialog::OnInitDialog()
 	m_Button_Stop.EnableWindow( fb_status.SufficientUserRights );
 
 	m_uiTimer = SetTimer(1, 500, NULL);
-	
+
 	return TRUE;  // return TRUE unless you set the focus to a control
 }
 
 
-void CFBDialog::UpdateServerStatus() 
+void CFBDialog::UpdateServerStatus()
 // This is one of the key functions. It is called
 // from the timer and sets the display.
 // It is in three sections
@@ -194,7 +194,7 @@ void CFBDialog::UpdateServerStatus()
 
 #ifdef MANAGE_CLASSIC
 	if (fb_status.WasRunning && !fb_status.UseService)
-	{ 
+	{
 		fb_status.UseClassic = (bool) GetClassicServerHandle();
 	}
 	else
@@ -208,20 +208,20 @@ void CFBDialog::UpdateServerStatus()
 	m_Firebird_Status.Format(fb_status.ServerStatus);
 
 #ifdef MANAGE_CLASSIC
-	if (fb_status.UseClassic) 
+	if (fb_status.UseClassic)
 	{
-		fb_status.ServiceExecutable = m_CS_Server_Name; 
+		fb_status.ServiceExecutable = m_CS_Server_Name;
 	}
 	else
 #endif
 	{
-		fb_status.ServiceExecutable = m_SS_Server_Name; 
+		fb_status.ServiceExecutable = m_SS_Server_Name;
 	}
 
 //========Start of code that updates GUI================
 	if ( fb_status.WasRunning )
 	{
-		m_Icon.SetIcon(AfxGetApp()->LoadIcon(IDI_ICON1)); 
+		m_Icon.SetIcon(AfxGetApp()->LoadIcon(IDI_ICON1));
 		m_Button_Stop.SetWindowText("&Stop");
 	}
 	else
@@ -230,10 +230,10 @@ void CFBDialog::UpdateServerStatus()
 		m_Button_Stop.SetWindowText("&Start");
 	}
 
-	//Reset check boxes 
+	//Reset check boxes
 	if (m_Reset_Display_To_Existing_Values)
 	{
-		// This is always called on startup. This method 
+		// This is always called on startup. This method
 		// is also called if an attempt to install
 		// the service fails.
 		ResetCheckBoxes( fb_status );
@@ -242,16 +242,16 @@ void CFBDialog::UpdateServerStatus()
 	}
 
 	m_Reset_Display_To_Existing_Values = false;
-	
-	UpdateData(false);		
+
+	UpdateData(false);
 
 	// This will be false the first time round.
-	// It is needed because the Config class doesn't 
+	// It is needed because the Config class doesn't
 	// refresh after the conf file has been updated.
 	initialised = true;
 
-	// The only time new_settings differs from fb_status is 
-	// during the ApplyChanges method. 
+	// The only time new_settings differs from fb_status is
+	// during the ApplyChanges method.
 	new_settings = fb_status;
 }
 
@@ -270,11 +270,13 @@ bool CFBDialog::CheckServiceInstalled( LPCTSTR service )
 
 bool CFBDialog::ValidateInstalledServices()
 // Check if services are installed.
-// If Guardian installed but not Server service 
+// If Guardian installed but not Server service
 // then return false;
 {
-	fb_status.UseService = CheckServiceInstalled(REMOTE_SERVICE);
-	fb_status.UseGuardian = CheckServiceInstalled(ISCGUARD_SERVICE);	
+	fb_status.UseService =
+		CheckServiceInstalled(GetServiceName(REMOTE_SERVICE));
+	fb_status.UseGuardian =
+		CheckServiceInstalled(GetServiceName(ISCGUARD_SERVICE));
 
 	if (!fb_status.UseService && fb_status.UseGuardian)
 		return false;
@@ -282,10 +284,16 @@ bool CFBDialog::ValidateInstalledServices()
 		return true;
 }
 
+CString CFBDialog::GetServiceName(const char* name) const
+{
+	CString serviceName;
+	serviceName.Format(name, FB_DEFAULT_INSTANCE);
+	return serviceName;
+}
 
 int CFBDialog::GetServerStatus()
-// This is called by UpdateServerStatus, 
-// which is called on the timer, so our status 
+// This is called by UpdateServerStatus,
+// which is called on the timer, so our status
 // should always be 'up-to-date'.
 // Returns:
 //		fb_status.ServerStatus
@@ -296,13 +304,14 @@ int CFBDialog::GetServerStatus()
 {
 	int result = IDS_APPLICATION_STOPPED;
 
-	fb_status.UseService = CheckServiceInstalled(REMOTE_SERVICE);
+	const CString serviceName = GetServiceName(REMOTE_SERVICE);
+	fb_status.UseService = CheckServiceInstalled(serviceName);
 
 	if ( fb_status.UseService )
 	{
 		OpenServiceManager( GENERIC_READ );
-		SC_HANDLE hService = OpenService (hScManager, REMOTE_SERVICE, GENERIC_READ );
-		QueryServiceStatus( hService, &service_status ); 
+		SC_HANDLE hService = OpenService (hScManager, serviceName, GENERIC_READ );
+		QueryServiceStatus( hService, &service_status );
 		CloseServiceHandle ( hService );
 		CloseServiceManager();
 		switch ( service_status.dwCurrentState )
@@ -313,16 +322,16 @@ int CFBDialog::GetServerStatus()
 			case SERVICE_START_PENDING :
 				result = IDS_SERVICE_START_PENDING;
 				break;
-			case SERVICE_STOP_PENDING : 
+			case SERVICE_STOP_PENDING :
 				result = IDS_SERVICE_STOP_PENDING;
 				break;
-			case SERVICE_RUNNING : 
+			case SERVICE_RUNNING :
 				result = IDS_SERVICE_RUNNING;
 				break;
 			case SERVICE_CONTINUE_PENDING :
 				result = IDS_SERVICE_CONTINUE_PENDING;
 				break;
-			case SERVICE_PAUSE_PENDING : 
+			case SERVICE_PAUSE_PENDING :
 				result = IDS_SERVICE_PAUSE_PENDING;
 				break;
 			case SERVICE_PAUSED :
@@ -333,7 +342,7 @@ int CFBDialog::GetServerStatus()
 	else
 	{
 		//Is Firebird running as an application...
-		if ( FirebirdRunning() ) 
+		if ( FirebirdRunning() )
 		{
 			result = IDS_APPLICATION_RUNNING;
 		}
@@ -343,13 +352,13 @@ int CFBDialog::GetServerStatus()
 		}
 	}
 
-	fb_status.WasRunning = ( (fb_status.ServerStatus == IDS_SERVICE_RUNNING) 
+	fb_status.WasRunning = ( (fb_status.ServerStatus == IDS_SERVICE_RUNNING)
 							|| (fb_status.ServerStatus == IDS_APPLICATION_RUNNING) );
 
-	// If running as an application and not set to run automatically on start up 
+	// If running as an application and not set to run automatically on start up
 	// we still don't  know if we the guardian is running.
 	if (fb_status.WasRunning && !fb_status.UseService)
-	{ 
+	{
 		fb_status.UseGuardian = (bool) GetGuardianHandle();
 	}
 
@@ -358,34 +367,36 @@ int CFBDialog::GetServerStatus()
 
 
 void CFBDialog::ViewRegistryEntries()
-// Find out what we have in the non-Firebird section of the 
+// Find out what we have in the non-Firebird section of the
 // registry ie, in Services or Application Run
 //
 // The following variables will be set on return:
-//	fb_status.UseGuardian 
+//	fb_status.UseGuardian
 //	fb_status.AutoStart
 //	fb_status.SystemLogin
 //	fb_status.UseClassic
 //	fb_status.ServerName
 {
-	
+
 	fb_status.AutoStart = false;
 
-	fb_status.UseService = CheckServiceInstalled(REMOTE_SERVICE);
+	fb_status.UseService =
+		CheckServiceInstalled(GetServiceName(REMOTE_SERVICE));
+
 	if ( fb_status.UseService )
 	{
 		OpenServiceManager( GENERIC_READ );
 
-		const char* service = ISCGUARD_SERVICE;
-		const char* display_name = ISCGUARD_DISPLAY_NAME;
+		CString service = GetServiceName(ISCGUARD_SERVICE);
+		CString display_name = GetServiceName(ISCGUARD_DISPLAY_NAME);
 		SC_HANDLE hService = OpenService (hScManager, service, SERVICE_QUERY_CONFIG);
 		fb_status.UseGuardian = hService;
 		if (hService != NULL) // then we are running as a Service
 		{
-			DWORD dwBytesNeeded; 
-			LPQUERY_SERVICE_CONFIG status_info = (LPQUERY_SERVICE_CONFIG) LocalAlloc(LPTR, 4096); 
+			DWORD dwBytesNeeded;
+			LPQUERY_SERVICE_CONFIG status_info = (LPQUERY_SERVICE_CONFIG) LocalAlloc(LPTR, 4096);
 			if (!QueryServiceConfig(hService, status_info, 4096, &dwBytesNeeded))
-			{	
+			{
 				LocalFree( status_info );
 				HandleError(false, "ViewRegistryEntries - Cannot query Guardian service.");
 			}
@@ -402,14 +413,14 @@ void CFBDialog::ViewRegistryEntries()
 		}
 
 		//Now do the same again, but this time only look at the server itself
-		service = REMOTE_SERVICE;
-		display_name = REMOTE_DISPLAY_NAME;
+		service = GetServiceName(REMOTE_SERVICE);
+		display_name = GetServiceName(REMOTE_DISPLAY_NAME);
 		hService = OpenService (hScManager, service, SERVICE_QUERY_CONFIG);
 		CloseServiceManager();
 		if (hService != NULL) // then we are running as a Service
 		{
-			DWORD dwBytesNeeded; 
-			LPQUERY_SERVICE_CONFIG status_info = (LPQUERY_SERVICE_CONFIG) LocalAlloc(LPTR, 4096); 
+			DWORD dwBytesNeeded;
+			LPQUERY_SERVICE_CONFIG status_info = (LPQUERY_SERVICE_CONFIG) LocalAlloc(LPTR, 4096);
 			if (!QueryServiceConfig(hService, status_info, 4096, &dwBytesNeeded))
 			{
 				LocalFree( status_info );
@@ -435,26 +446,26 @@ void CFBDialog::ViewRegistryEntries()
 	}
 	else //Installed as Application, so look for an entry in registry
 	{
-	    HKEY hkey;
-		if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, 
-			"Software\\Microsoft\\Windows\\CurrentVersion\\Run", 0, 
+		HKEY hkey;
+		if (RegOpenKeyEx(HKEY_LOCAL_MACHINE,
+			"Software\\Microsoft\\Windows\\CurrentVersion\\Run", 0,
 			KEY_QUERY_VALUE, &hkey) == ERROR_SUCCESS)
 		{
 			DWORD dwType;
 			DWORD dwSize = MAX_PATH;
-			fb_status.AutoStart = (RegQueryValueEx(hkey, "Firebird", NULL, 
-				&dwType, (LPBYTE) fb_status.ServerName.GetBuffer(dwSize / sizeof(TCHAR)), 
+			fb_status.AutoStart = (RegQueryValueEx(hkey, "Firebird", NULL,
+				&dwType, (LPBYTE) fb_status.ServerName.GetBuffer(dwSize / sizeof(TCHAR)),
 				&dwSize) == ERROR_SUCCESS );
 			fb_status.ServerName.ReleaseBuffer(-1);
 			if (fb_status.AutoStart)
 				fb_status.UseGuardian = ( fb_status.ServerName.Find("fbguard") == ERROR_SUCCESS );
-				
-		    RegCloseKey (hkey);
+
+			RegCloseKey (hkey);
 		}
 	}
 
 #ifdef MANAGE_CLASSIC
-	//Our look in the registry has probably given us 
+	//Our look in the registry has probably given us
 	//the filename of the installed server
 	if ( fb_status.ServerName.Find("fb_inet_server") == ERROR_SUCCESS )
 	{
@@ -464,9 +475,9 @@ void CFBDialog::ViewRegistryEntries()
 	{
 		fb_status.UseClassic = false;
 	}
-	if ( fb_status.ServerName == "" ) 
+	if ( fb_status.ServerName == "" )
 	{
-		// Nothing is stored in the registry so we must look to see if 
+		// Nothing is stored in the registry so we must look to see if
 		// Firebird.conf has a preference
 		fb_status.UseClassic = (bool) GetPreferredArchitecture();
 	}
@@ -479,7 +490,7 @@ void CFBDialog::ApplyChanges()
  *
  *	It all happens here.
  *
- *	With the addition of support for classic we now need to 
+ *	With the addition of support for classic we now need to
  *  evaluate 16 possible states.
  *
  */
@@ -489,9 +500,9 @@ void CFBDialog::ApplyChanges()
 
 	//find out what has changed and implement the changes
 	try
-	{	
+	{
 	//Stage 1
-		// Stop the Server 
+		// Stop the Server
 		// We don't try to restart unless we were running
 		// and we successfully stopped the server.
 		if ( fb_status.WasRunning )
@@ -499,10 +510,10 @@ void CFBDialog::ApplyChanges()
 			if ( ServerStop() )
 			{
 #if defined(_DEBUG)
-				// If we are in debug mode it is useful to 
+				// If we are in debug mode it is useful to
 				// reset the display - but UpdateServerStatus
-				// resyncs new_settings with fb_status, so 
-				// it must never be called in ApplyChanges() 
+				// resyncs new_settings with fb_status, so
+				// it must never be called in ApplyChanges()
 				// after this point.
 				UpdateServerStatus();
 				ProcessMessages();
@@ -521,24 +532,24 @@ void CFBDialog::ApplyChanges()
 	//Stage 2 - Gather details of changes to make
 
 		//Manage change to startup - from/to manual or auto
-		bool ChangeStartType = (	( fb_status.AutoStart &&  m_Manual_Start.GetCheck() ) || 
+		bool ChangeStartType = (	( fb_status.AutoStart &&  m_Manual_Start.GetCheck() ) ||
 			( !fb_status.AutoStart &&  m_Auto_Start.GetCheck() ) );
-		
+
 		if ( ChangeStartType )
 			new_settings.AutoStart = !fb_status.AutoStart;
-		else 
+		else
 			new_settings.AutoStart = fb_status.AutoStart;
-		
+
 #ifdef MANAGE_CLASSIC
 		//Do we use Super Server or Classic
-		bool ChangeServerArchitecture = ( ( !fb_status.UseClassic && m_Classic_Server.GetCheck() ) || 
+		bool ChangeServerArchitecture = ( ( !fb_status.UseClassic && m_Classic_Server.GetCheck() ) ||
 			( fb_status.UseClassic && m_Super_Server.GetCheck() ) );
 		if ( ChangeServerArchitecture )
 			new_settings.UseClassic = !fb_status.UseClassic;
 		else
 			new_settings.UseClassic = fb_status.UseClassic;
 
-		if (new_settings.UseClassic) 
+		if (new_settings.UseClassic)
 		{
 			new_settings.ServiceExecutable = m_CS_Server_Name;
 		}
@@ -549,7 +560,7 @@ void CFBDialog::ApplyChanges()
 #endif
 
 		//Do we change Guardian Usage?
-		bool ChangeGuardianUse = ( ( !fb_status.UseGuardian && m_Use_Guardian.GetCheck() ) || 
+		bool ChangeGuardianUse = ( ( !fb_status.UseGuardian && m_Use_Guardian.GetCheck() ) ||
 			( fb_status.UseGuardian && !m_Use_Guardian.GetCheck() ) );
 		if ( ChangeGuardianUse )
 			new_settings.UseGuardian = !fb_status.UseGuardian;
@@ -564,7 +575,7 @@ void CFBDialog::ApplyChanges()
 		else
 			new_settings.UseService = fb_status.UseService;
 
-		
+
 	//Stage 3 - implement changes
 
 #if !defined(_DEBUG)
@@ -583,7 +594,7 @@ void CFBDialog::ApplyChanges()
 				AppRemove();
 		}
 
-		
+
 #ifdef FBCPL_UPDATE_CONF
 		// b) update firebird.conf
 		if ( ChangeGuardianUse )
@@ -599,10 +610,10 @@ void CFBDialog::ApplyChanges()
 #endif
 
 		// c) install the new configuration
-		if ( ChangeRunStyle || ChangeGuardianUse 
+		if ( ChangeRunStyle || ChangeGuardianUse
 #ifdef MANAGE_CLASSIC
 
-			 || ChangeServerArchitecture 
+			 || ChangeServerArchitecture
 #endif
 			)
 		{
@@ -643,7 +654,7 @@ void CFBDialog::ApplyChanges()
 	}
 	catch ( ... )
 	{
-		// Oops, something bad happened. Which 
+		// Oops, something bad happened. Which
 		// means the apply button is still enabled.
 		// We should probably do a reset here.
 
@@ -651,7 +662,7 @@ void CFBDialog::ApplyChanges()
 	}
 
 #if !defined(_DEBUG)
-    EndWaitCursor();
+	EndWaitCursor();
 #endif
 
 	//Whatever the outcome of ApplyChanges we need to refresh the dialog
@@ -672,7 +683,7 @@ void CFBDialog::ResetCheckBoxes(CFBDialog::STATUS status)
 		m_Run_As_Application.SetCheck(0);
 		m_Run_As_Service.SetCheck(1);
 	}
-	
+
 	//Now are we starting automatically or not?
 	if (status.AutoStart)
 	{
@@ -684,7 +695,7 @@ void CFBDialog::ResetCheckBoxes(CFBDialog::STATUS status)
 		m_Auto_Start.SetCheck(0);
 		m_Manual_Start.SetCheck(1);
 	}
-	
+
 	m_Use_Guardian.SetCheck(status.UseGuardian);
 
 #ifdef MANAGE_CLASSIC
@@ -700,9 +711,9 @@ void CFBDialog::ResetCheckBoxes(CFBDialog::STATUS status)
 	}
 #endif
 
-	// The server can now be controlled by a specific 
-	// username/password. If it is set then for now we 
-	// will disable all config options and add 
+	// The server can now be controlled by a specific
+	// username/password. If it is set then for now we
+	// will disable all config options and add
 	// support at a later date.
 /*	m_Classic_Server.EnableWindow( status.SystemLogin );
 	m_Super_Server.EnableWindow( status.SystemLogin );
@@ -737,19 +748,19 @@ bool CFBDialog::ServerStart( CFBDialog::STATUS status )
 	BeginWaitCursor();
 #endif
 
-	if ( status.UseService ) 
+	if ( status.UseService )
 	{
-		const char * service = "";
-		const char * display_name = "";
+		CString service;
+		CString display_name;
 		if (status.UseGuardian)
 		{
-			service = ISCGUARD_SERVICE;
-			display_name = ISCGUARD_DISPLAY_NAME;
+			service = GetServiceName(ISCGUARD_SERVICE);
+			display_name = GetServiceName(ISCGUARD_DISPLAY_NAME);
 		}
 		else
 		{
-			service = REMOTE_SERVICE;
-			display_name = REMOTE_DISPLAY_NAME;
+			service = GetServiceName(REMOTE_SERVICE);
+			display_name = GetServiceName(REMOTE_DISPLAY_NAME);
 		}
 
 		OpenServiceManager(GENERIC_READ | GENERIC_EXECUTE | GENERIC_WRITE);
@@ -761,7 +772,7 @@ bool CFBDialog::ServerStart( CFBDialog::STATUS status )
 				if (m_Error_Status == FB_SUCCESS)
 					result = true;
 			}
-			catch( ... ) 
+			catch ( ... )
 			{
 			}
 		}
@@ -779,26 +790,26 @@ bool CFBDialog::ServerStart( CFBDialog::STATUS status )
 			sa.nLength = sizeof (sa);
 			sa.lpSecurityDescriptor = NULL;
 			sa.bInheritHandle = TRUE;
-			
+
 			char full_name[MAX_PATH + 15] = "";
 			GetFullAppPath( status, full_name );
-			
+
 			if (!CreateProcess (NULL, full_name, &sa, NULL, FALSE, 0, NULL, NULL, &si, &pi))
 				HandleError(false, "Application Start");
-			else 
+			else
 			{
 				result = true;
 				CloseHandle(pi.hProcess);
 				CloseHandle(pi.hThread);
 			}
 		}
-		catch( ... )
+		catch ( ... )
 		{
 		}
 	}
 
 #if !defined(_DEBUG)
-    EndWaitCursor();
+	EndWaitCursor();
 #endif
 
 	return result;
@@ -820,22 +831,31 @@ bool CFBDialog::ServerStop()
 #endif
 				OpenServiceManager( GENERIC_READ | GENERIC_EXECUTE | GENERIC_WRITE );
 
+				CString service;
+				CString display_name;
 				if (fb_status.UseGuardian)
-					m_Error_Status = SERVICES_stop(hScManager, ISCGUARD_SERVICE, ISCGUARD_DISPLAY_NAME, svc_error);
+				{
+					service = GetServiceName(ISCGUARD_SERVICE);
+					display_name = GetServiceName(ISCGUARD_DISPLAY_NAME);
+				}
 				else
-					m_Error_Status = SERVICES_stop(hScManager, REMOTE_SERVICE, REMOTE_DISPLAY_NAME, svc_error);
+				{
+					service = GetServiceName(REMOTE_SERVICE);
+					display_name = GetServiceName(REMOTE_DISPLAY_NAME);
+				}
+				m_Error_Status = SERVICES_stop(hScManager, service, display_name, svc_error);
 
 				result = !m_Error_Status;
-		
+
 			}
-			catch (...) 
+			catch (...)
 			{
 				MessageBeep(-1);
 			}
 
 			CloseServiceManager();
 #if !defined(_DEBUG)
-		    EndWaitCursor();
+			EndWaitCursor();
 #endif
 		}
 		else
@@ -848,11 +868,11 @@ bool CFBDialog::ServerStop()
 				KillApp();
 				result = true;
 			}
-			catch (...) 
+			catch (...)
 			{
 			}
 #if !defined(_DEBUG)
-		    EndWaitCursor();
+			EndWaitCursor();
 #endif
 		}
 	}
@@ -863,9 +883,9 @@ bool CFBDialog::ServerStop()
 
 void CFBDialog::KillApp()
 {
-	// Under Win2K3 and WinXP there seem to be timing issues that don't 
-	// exist in earlier platforms. Killing the server will kill the 
-	// Guardian, but it won't always do it quickly enough for the CPL 
+	// Under Win2K3 and WinXP there seem to be timing issues that don't
+	// exist in earlier platforms. Killing the server will kill the
+	// Guardian, but it won't always do it quickly enough for the CPL
 	// applet, so we try and do that before killing the server.
 	LRESULT result;
 	HWND hTmpWnd = GetGuardianHandle();
@@ -887,34 +907,41 @@ void CFBDialog::KillApp()
 
 
 bool CFBDialog::ServiceInstall( CFBDialog::STATUS status )
-{				   
+{
 	OpenServiceManager( GENERIC_READ | GENERIC_EXECUTE | GENERIC_WRITE );
 
 	if (hScManager)
 	{
 		const char* ServerPath = m_Root_Path;
-		if (new_settings.UseGuardian) 
+
+		CString guard_service = GetServiceName(ISCGUARD_SERVICE);
+		CString guard_display_name = GetServiceName(ISCGUARD_DISPLAY_NAME);
+		CString remote_service = GetServiceName(REMOTE_SERVICE);
+		CString remote_display_name = GetServiceName(REMOTE_DISPLAY_NAME);
+
+		if (new_settings.UseGuardian)
 		{
-			m_Error_Status = SERVICES_install (hScManager, ISCGUARD_SERVICE,
-				ISCGUARD_DISPLAY_NAME, ISCGUARD_DISPLAY_DESCR,
-				ISCGUARD_EXECUTABLE, ServerPath, NULL, status.AutoStart,
-				NULL, NULL, false, svc_error);
+			m_Error_Status = SERVICES_install (hScManager,
+				guard_service, guard_display_name,
+				ISCGUARD_DISPLAY_DESCR, ISCGUARD_EXECUTABLE,
+				ServerPath, NULL, NULL, status.AutoStart,
+				NULL, NULL, false, true, svc_error);
 			if (m_Error_Status != FB_SUCCESS)
 			{
 				CloseServiceManager();
 				return false;
 			}
-			
+
 			/* Set AutoStart to manual in preparation for installing the ib_server service */
 			status.AutoStart = false;
-			
+
 		}
 		/* do the install of server */
-		m_Error_Status = SERVICES_install (hScManager, REMOTE_SERVICE,
-			REMOTE_DISPLAY_NAME, REMOTE_DISPLAY_DESCR,
-			(LPCTSTR) status.ServiceExecutable, 
-			ServerPath, NULL, status.AutoStart, 
-			NULL, NULL, false, svc_error);
+		m_Error_Status = SERVICES_install (hScManager,
+			remote_service, remote_display_name,
+			REMOTE_DISPLAY_DESCR, (LPCTSTR) status.ServiceExecutable,
+			ServerPath, NULL, NULL, status.AutoStart,
+			NULL, NULL, false, !new_settings.UseGuardian, svc_error);
 		if (m_Error_Status != FB_SUCCESS)
 		{
 			CloseServiceManager();
@@ -927,10 +954,10 @@ bool CFBDialog::ServiceInstall( CFBDialog::STATUS status )
 			catch ( ... )
 			{
 			}
-			
+
 			return false;
 		}
-		
+
 		CloseServiceManager();
 		return true;
 	}
@@ -944,16 +971,21 @@ bool CFBDialog::ServiceRemove()
 	OpenServiceManager( GENERIC_READ | GENERIC_EXECUTE | GENERIC_WRITE );
 	if (hScManager)
 	{
-		m_Error_Status = SERVICES_remove (hScManager, ISCGUARD_SERVICE, 
-			ISCGUARD_DISPLAY_NAME, svc_error);
+		CString guard_service = GetServiceName(ISCGUARD_SERVICE);
+		CString guard_display_name = GetServiceName(ISCGUARD_DISPLAY_NAME);
+		CString remote_service = GetServiceName(REMOTE_SERVICE);
+		CString remote_display_name = GetServiceName(REMOTE_DISPLAY_NAME);
+
+		m_Error_Status = SERVICES_remove (hScManager, guard_service,
+			guard_display_name, svc_error);
 		if (m_Error_Status == IB_SERVICE_RUNNING)
 		{
 			CloseServiceManager();
 			return false;
 		}
-		
-		m_Error_Status = SERVICES_remove (hScManager, REMOTE_SERVICE, 
-			REMOTE_DISPLAY_NAME, svc_error);
+
+		m_Error_Status = SERVICES_remove (hScManager, remote_service,
+			remote_display_name, svc_error);
 		if (m_Error_Status == IB_SERVICE_RUNNING)
 		{
 			CloseServiceManager();
@@ -968,8 +1000,8 @@ bool CFBDialog::ServiceRemove()
 
 
 bool CFBDialog::AppInstall( CFBDialog::STATUS status)
-// This method is supplied as a corollary to ServiceInstall, 
-// but doesn't do very much as there isn't much to do. 
+// This method is supplied as a corollary to ServiceInstall,
+// but doesn't do very much as there isn't much to do.
 {
 	return ConfigureRegistryForApp( status );
 }
@@ -977,18 +1009,18 @@ bool CFBDialog::AppInstall( CFBDialog::STATUS status)
 
 bool CFBDialog::ConfigureRegistryForApp(CFBDialog::STATUS status)
 {
-	// The calling procedure will have already removed the 
+	// The calling procedure will have already removed the
 	// service. All we need to do now
 	// is configure the registry if AutoStart has been set.
 	if ( status.AutoStart )
 	{
-		//Add line to registry 
+		//Add line to registry
 		HKEY hkey;
-		if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, 
-			"Software\\Microsoft\\Windows\\CurrentVersion\\Run", 
+		if (RegOpenKeyEx(HKEY_LOCAL_MACHINE,
+			"Software\\Microsoft\\Windows\\CurrentVersion\\Run",
 			0, KEY_WRITE, &hkey) == ERROR_SUCCESS)
 		{
-			
+
 			char full_name[MAX_PATH + 15] = "";
 			GetFullAppPath( status, full_name);
 			if (!RegSetValueEx (hkey, "Firebird", 0,REG_SZ, (CONST BYTE*) full_name, sizeof(full_name) ) == ERROR_SUCCESS)
@@ -1007,12 +1039,12 @@ bool CFBDialog::ConfigureRegistryForApp(CFBDialog::STATUS status)
 	{
 		//Remove registry entry if set to start automatically on boot.
 		HKEY hkey;
-		if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, 
-			"Software\\Microsoft\\Windows\\CurrentVersion\\Run", 
+		if (RegOpenKeyEx(HKEY_LOCAL_MACHINE,
+			"Software\\Microsoft\\Windows\\CurrentVersion\\Run",
 			0, KEY_QUERY_VALUE | KEY_WRITE, &hkey) == ERROR_SUCCESS)
-			
+
 		{
-			if (RegQueryValueEx(hkey, "Firebird", NULL, NULL, NULL, NULL) 
+			if (RegQueryValueEx(hkey, "Firebird", NULL, NULL, NULL, NULL)
 				== ERROR_SUCCESS)
 			{
 				if (RegDeleteValue(hkey, "Firebird") == ERROR_SUCCESS)
@@ -1071,7 +1103,7 @@ static USHORT svc_error (SLONG	error_status, const TEXT* string, SC_HANDLE servi
 
 	if (RaiseError)
 		 CFBDialog::HandleSvcError(error_status, string);
-	
+
 	if (service != NULL)
 		CloseServiceHandle (service);
 
@@ -1082,12 +1114,12 @@ static USHORT svc_error (SLONG	error_status, const TEXT* string, SC_HANDLE servi
 void CFBDialog::ProcessMessages()
 {
 	MSG Msg;
-	
+
 	while (::PeekMessage(&Msg, NULL, 0, 0, PM_NOREMOVE))
 	{
 		if (!AfxGetApp()->PumpMessage())
 		{
-			::PostQuitMessage(0); 
+			::PostQuitMessage(0);
 			return;
 		}
 	}
@@ -1100,7 +1132,7 @@ void CFBDialog::ProcessMessages()
 
 void CFBDialog::HandleSvcError(SLONG error_status, const TEXT* string )
 // This method supports the static svc_error() function
-// and essentially duplicates HandleError. Oh to be rid of the 
+// and essentially duplicates HandleError. Oh to be rid of the
 // legacy code.
 {
 	if (!error_status)
@@ -1108,16 +1140,16 @@ void CFBDialog::HandleSvcError(SLONG error_status, const TEXT* string )
 
 	LPTSTR lpMsgBuf = 0;
 	CString error_title = "";
-	
-	const DWORD Size = FormatMessage( 
-		FORMAT_MESSAGE_ALLOCATE_BUFFER | 
-		FORMAT_MESSAGE_FROM_SYSTEM | 
+
+	const DWORD Size = FormatMessage(
+		FORMAT_MESSAGE_ALLOCATE_BUFFER |
+		FORMAT_MESSAGE_FROM_SYSTEM |
 		FORMAT_MESSAGE_IGNORE_INSERTS,
 		NULL,
 		error_status,
 		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
 		(LPTSTR) &lpMsgBuf,	0, NULL );
-	
+
 	if (Size)
 		error_title.Format("Error '%s' raised in %s", lpMsgBuf, string);
 	else
@@ -1149,17 +1181,17 @@ void CFBDialog::HandleError(bool silent, const TEXT *string )
 		LPTSTR lpMsgBuf = 0;
 		CString error_title = "";
 
-		const DWORD Size = FormatMessage( 
-			FORMAT_MESSAGE_ALLOCATE_BUFFER | 
-			FORMAT_MESSAGE_FROM_SYSTEM | 
+		const DWORD Size = FormatMessage(
+			FORMAT_MESSAGE_ALLOCATE_BUFFER |
+			FORMAT_MESSAGE_FROM_SYSTEM |
 			FORMAT_MESSAGE_IGNORE_INSERTS,
 			NULL,
 			error_code,
 			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
 			(LPTSTR) &lpMsgBuf,	0, NULL );
-	
+
 		if (Size)
-			error_title.Format("Error '%s' raised in %s", lpMsgBuf, string);			
+			error_title.Format("Error '%s' raised in %s", lpMsgBuf, string);
 		else
 			error_title.Format("Error Code %d raised in %s", error_code, string );
 
@@ -1176,16 +1208,16 @@ void CFBDialog::ShowError( LPTSTR lpMsgBuf, CString error_title )
 }
 
 
-void CFBDialog::OnButtonStop() 
+void CFBDialog::OnButtonStop()
 {
 	if ( fb_status.WasRunning )
 		ServerStop();
-	else	
+	else
 		ServerStart( fb_status );
 }
 
 
-void CFBDialog::OnOK() 
+void CFBDialog::OnOK()
 {
 	// Extra validation can be added here
 
@@ -1194,7 +1226,7 @@ void CFBDialog::OnOK()
 	{
 		OnApply();
 	}
-	
+
 	CDialog::OnOK();
 }
 
@@ -1213,13 +1245,13 @@ void CFBDialog::DisableApplyButton()
 }
 
 
-void CFBDialog::OnTimer(UINT nIDEvent) 
+void CFBDialog::OnTimer(UINT_PTR nIDEvent)
 {
 	UpdateServerStatus();
 }
 
 
-void CFBDialog::OnDestroy() 
+void CFBDialog::OnDestroy()
 {
 	// Kill the update timer
 	if (m_uiTimer) KillTimer(m_uiTimer);
@@ -1228,56 +1260,56 @@ void CFBDialog::OnDestroy()
 }
 
 
-void CFBDialog::OnService() 
+void CFBDialog::OnService()
 {
 	EnableApplyButton();
 }
 
 
-void CFBDialog::OnManualStart() 
+void CFBDialog::OnManualStart()
 {
 	EnableApplyButton();
 }
 
 
-void CFBDialog::OnApplication() 
+void CFBDialog::OnApplication()
 {
 	EnableApplyButton();
 }
 
 
-void CFBDialog::OnAutoStart() 
+void CFBDialog::OnAutoStart()
 {
 	EnableApplyButton();
 }
 
 
-void CFBDialog::OnUseGuardian() 
+void CFBDialog::OnUseGuardian()
 {
 	EnableApplyButton();
 }
 
 
-void CFBDialog::OnSuperServer() 
+void CFBDialog::OnSuperServer()
 {
 	EnableApplyButton();
 }
 
 
-void CFBDialog::OnClassicServer() 
+void CFBDialog::OnClassicServer()
 {
 	EnableApplyButton();
 }
 
 
-void CFBDialog::OnApply() 
+void CFBDialog::OnApply()
 {
 	ApplyChanges();
 }
 
 
 #ifdef FBCPL_UPDATE_CONF
-// Currently (Fb 1.5.1) there is no longer any 
+// Currently (Fb 1.5.1) there is no longer any
 // need to change the .conf file
 void CFBDialog::SetGuardianUseInConf( bool UseGuardian )
 {
@@ -1293,7 +1325,7 @@ void CFBDialog::SetGuardianUseInConf( bool UseGuardian )
 */
 	if (UpdateFirebirdConf("GuardianOption", newvalue ) )
 	{
-		// Do we assign here? or wait for the 
+		// Do we assign here? or wait for the
 		// update status routine to pick this up?
 		fb_status.UseGuardian = UseGuardian;
 	}
@@ -1320,7 +1352,7 @@ void CFBDialog::SetPreferredArchitectureInConf( bool UseClassic )
 */
 	if ( UpdateFirebirdConf("PreferClassicServer", newvalue ) )
 	{
-		// Do we assign here? or wait for the 
+		// Do we assign here? or wait for the
 		// update status routine to pick this up?
 		fb_status.UseClassic = UseClassic;
 	}
@@ -1342,7 +1374,7 @@ bool CFBDialog::UpdateFirebirdConf(CString option, CString value)
 
 	if (!FbConfFile.Open(FirebirdConfFilename, CFile::modeReadWrite))
 		return false;
-	if (!FbConfFileNew.Open(FirebirdConfNewname, CFile::modeCreate | 
+	if (!FbConfFileNew.Open(FirebirdConfNewname, CFile::modeCreate |
 							CFile::shareExclusive | CFile::modeWrite))
 	{
 		return false;
@@ -1357,19 +1389,19 @@ bool CFBDialog::UpdateFirebirdConf(CString option, CString value)
 			{
 				FirebirdConfLine = option + " = " + value;
 			}
-	
+
 			FbConfFileNew.WriteString( FirebirdConfLine + '\n');
-		
+
 		}
 	}
 	 catch (CFileException *e)
 	{
-    #ifdef _DEBUG
-        afxDump << "Problem updating " << e->m_strFileName << ".\n \
+	#ifdef _DEBUG
+		afxDump << "Problem updating " << e->m_strFileName << ".\n \
 					cause = " << e->m_cause << "\n";
-    #endif
+	#endif
 	}
-	
+
 
 	bool result = false;
 	FbConfFile.Close();
@@ -1379,7 +1411,7 @@ bool CFBDialog::UpdateFirebirdConf(CString option, CString value)
 	{
 		CFile::Rename(FirebirdConfFilename, FirebirdConfOldname);
 		CFile::Rename(FirebirdConfNewname, FirebirdConfFilename);
-		
+
 		//If we get this far then all is well and we can return good news
 		result = true;
 	}
@@ -1401,71 +1433,70 @@ void CFBDialog::SetAutoStart( CFBDialog::STATUS status )
 	if (status.UseService)
 	{
 		OpenServiceManager( GENERIC_READ | GENERIC_EXECUTE | GENERIC_WRITE );
-		// Need to acquire database lock before reconfiguring. 
+		// Need to acquire database lock before reconfiguring.
 		if (hScManager)
 		{
-			SC_LOCK sclLock = LockServiceDatabase(hScManager); 
-			
-			// If the database cannot be locked, report the details. 
-			if (sclLock == NULL) 
+			SC_LOCK sclLock = LockServiceDatabase(hScManager);
+
+			// If the database cannot be locked, report the details.
+			if (sclLock == NULL)
 			{
-				HandleError(false, "SetAutoStart - Could not lock service database"); 
+				HandleError(false, "SetAutoStart - Could not lock service database");
 				return;
 			}
-			
-			// The database is locked, so it is safe to make changes. 
-			
-			const char* service = "";
-			const char* display_name = "";
-			
-			if ( status.UseGuardian )
+
+			// The database is locked, so it is safe to make changes.
+
+			CString service;
+			CString display_name;
+			if (status.UseGuardian)
 			{
-				service = ISCGUARD_SERVICE;
-				display_name = ISCGUARD_DISPLAY_NAME;
+				service = GetServiceName(ISCGUARD_SERVICE);
+				display_name = GetServiceName(ISCGUARD_DISPLAY_NAME);
 			}
 			else
 			{
-				service = REMOTE_SERVICE;
-				display_name = REMOTE_DISPLAY_NAME;
+				service = GetServiceName(REMOTE_SERVICE);
+				display_name = GetServiceName(REMOTE_DISPLAY_NAME);
 			}
-			
-			// Open a handle to the service. 
-			SC_HANDLE hService = OpenService( 
-				hScManager,				// SCManager database 
-				service,				// name of service 
-				SERVICE_CHANGE_CONFIG);	// need CHANGE access 
-			if (hService) 
-			{		
-				DWORD dwStartType = status.AutoStart ? SERVICE_AUTO_START : SERVICE_DEMAND_START; 
-				
-				if (! ChangeServiceConfig( 
-					hService,			// handle of service 
-					SERVICE_NO_CHANGE,	// service type: no change 
-					dwStartType,		// change service start type 
-					SERVICE_NO_CHANGE,	// error control: no change 
-					NULL,				// binary path: no change 
-					NULL,				// load order group: no change 
-					NULL,				// tag ID: no change 
-					NULL,				// dependencies: no change 
-					NULL,				// account name: no change 
-					NULL,				// password: no change 
+
+			// Open a handle to the service.
+			SC_HANDLE hService = OpenService(
+				hScManager,				// SCManager database
+				service,				// name of service
+				SERVICE_CHANGE_CONFIG);	// need CHANGE access
+			if (hService)
+			{
+				DWORD dwStartType = status.AutoStart ? SERVICE_AUTO_START : SERVICE_DEMAND_START;
+
+				if (! ChangeServiceConfig(
+					hService,			// handle of service
+					SERVICE_NO_CHANGE,	// service type: no change
+					dwStartType,		// change service start type
+					SERVICE_NO_CHANGE,	// error control: no change
+					NULL,				// binary path: no change
+					NULL,				// load order group: no change
+					NULL,				// tag ID: no change
+					NULL,				// dependencies: no change
+					NULL,				// account name: no change
+					NULL,				// password: no change
 					display_name ) )
 				{
-					HandleError(false, "ChangeServiceConfig in SetAutoStart"); 
+					HandleError(false, "ChangeServiceConfig in SetAutoStart");
 				}
 			}
 			else
-				HandleError(false, "OpenService in SetAutoStart"); 
-			
-			// Release the database lock. 
-			UnlockServiceDatabase(sclLock); 
-			
-			// Close the handle to the service. 
-			CloseServiceHandle(hService); 
+				HandleError(false, "OpenService in SetAutoStart");
+
+			// Release the database lock.
+			UnlockServiceDatabase(sclLock);
+
+			// Close the handle to the service.
+			CloseServiceHandle(hService);
 		}
 
 		CloseServiceManager();
-	
+
 	}
 	else
 	{
@@ -1578,7 +1609,7 @@ void CFBDialog::CloseServiceManager()
 
 
 void CFBDialog::GetFullAppPath( CFBDialog::STATUS status, char * app)
-// This returns the fully qualified path and name of the application 
+// This returns the fully qualified path and name of the application
 // to start along with parameters -a and -c .
 {
 	CString AppName = m_Root_Path;
@@ -1588,7 +1619,7 @@ void CFBDialog::GetFullAppPath( CFBDialog::STATUS status, char * app)
 		AppName += m_Guardian_Name;
 
 #ifdef MANAGE_CLASSIC
-		if (status.UseClassic) 
+		if (status.UseClassic)
 		{
 			AppName += ".exe -c -a";
 		}
